/**
 * Copyright (C) 2016 Red Hat, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package io.syndesis.model;

import java.io.Serializable;
import java.util.Optional;

import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import org.immutables.value.Value;

/**
 * A ChangeEvent is used to notify clients about changes
 * to rest API resources.
 */
@Value.Immutable
@JsonDeserialize(builder = ChangeEvent.Builder.class)
public interface ChangeEvent extends ToJson, Serializable {

    Optional<String> getAction();
    ChangeEvent withAction(String kind);

    Optional<String> getKind();
    ChangeEvent withKind(String kind);

    Optional<String> getId();

    default ChangeEvent withId(String id) {
        return new ChangeEvent.Builder().createFrom(this).id(id).build();
    }

    static ChangeEvent of(String action, String kind, String id) {
        return new ChangeEvent.Builder().action(action).kind(kind).id(id).build();
    }

    class Builder extends ImmutableChangeEvent.Builder {
    }

}
