/**
 * Copyright (C) 2016 Red Hat, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package io.syndesis.model.connection;

import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import com.fasterxml.jackson.annotation.JsonProperty;
import edu.umd.cs.findbugs.annotations.SuppressFBWarnings;
import io.syndesis.model.WithName;
import io.syndesis.model.WithTags;
import java.io.ObjectStreamException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.NavigableSet;
import java.util.Objects;
import java.util.Optional;
import java.util.Set;
import java.util.SortedSet;
import java.util.TreeSet;
import javax.annotation.Generated;
import javax.validation.ConstraintViolation;
import javax.validation.ConstraintViolationException;
import javax.validation.Validation;
import javax.validation.Validator;

/**
 * Immutable implementation of {@link Action}.
 * <p>
 * Use the builder to create immutable instances:
 * {@code new Action.Builder()}.
 * Use the static factory method to create immutable instances:
 * {@code ImmutableAction.of()}.
 */
@SuppressWarnings({"all"})
@SuppressFBWarnings
@Generated({"Immutables.generator", "Action"})
@JsonIgnoreProperties(value = {"properties", "inputDataShape", "outputDataShape"}, allowGetters = true)
final class ImmutableAction implements Action {
  private final String connectorId;
  private final String description;
  private final String camelConnectorGAV;
  private final String camelConnectorPrefix;
  private final ActionDefinition definition;
  private final String id;
  private final String name;
  private final SortedSet<String> tags;

  private ImmutableAction(
      String connectorId,
      String description,
      String camelConnectorGAV,
      String camelConnectorPrefix,
      ActionDefinition definition,
      Optional<String> id,
      String name,
      Iterable<String> tags) {
    this.connectorId = connectorId;
    this.description = description;
    this.camelConnectorGAV = camelConnectorGAV;
    this.camelConnectorPrefix = camelConnectorPrefix;
    this.definition = definition;
    this.id = id.orElse(null);
    this.name = name;
    this.tags = createUnmodifiableSortedSet(false, createSafeList(tags, false, true));
  }

  private ImmutableAction(
      ImmutableAction original,
      String connectorId,
      String description,
      String camelConnectorGAV,
      String camelConnectorPrefix,
      ActionDefinition definition,
      String id,
      String name,
      SortedSet<String> tags) {
    this.connectorId = connectorId;
    this.description = description;
    this.camelConnectorGAV = camelConnectorGAV;
    this.camelConnectorPrefix = camelConnectorPrefix;
    this.definition = definition;
    this.id = id;
    this.name = name;
    this.tags = tags;
  }

  /**
   * @return The value of the {@code connectorId} attribute
   */
  @JsonProperty("connectorId")
  @Override
  public String getConnectorId() {
    return connectorId;
  }

  /**
   * @return The value of the {@code description} attribute
   */
  @JsonProperty("description")
  @Override
  public String getDescription() {
    return description;
  }

  /**
   * @return The value of the {@code camelConnectorGAV} attribute
   */
  @JsonProperty("camelConnectorGAV")
  @Override
  public String getCamelConnectorGAV() {
    return camelConnectorGAV;
  }

  /**
   * @return The value of the {@code camelConnectorPrefix} attribute
   */
  @JsonProperty("camelConnectorPrefix")
  @Override
  public String getCamelConnectorPrefix() {
    return camelConnectorPrefix;
  }

  /**
   * @return The value of the {@code definition} attribute
   */
  @JsonProperty("definition")
  @Override
  public ActionDefinition getDefinition() {
    return definition;
  }

  /**
   * @return The value of the {@code id} attribute
   */
  @JsonProperty("id")
  @Override
  public Optional<String> getId() {
    return Optional.ofNullable(id);
  }

  /**
   * @return The value of the {@code name} attribute
   */
  @JsonProperty("name")
  @Override
  public String getName() {
    return name;
  }

  /**
   * @return The value of the {@code tags} attribute
   */
  @JsonProperty("tags")
  @Override
  public SortedSet<String> getTags() {
    return tags;
  }

  /**
   * Copy the current immutable object by setting a value for the {@link Action#getConnectorId() connectorId} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for connectorId (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableAction withConnectorId(String value) {
    if (Objects.equals(this.connectorId, value)) return this;
    return validate(new ImmutableAction(
        this,
        value,
        this.description,
        this.camelConnectorGAV,
        this.camelConnectorPrefix,
        this.definition,
        this.id,
        this.name,
        this.tags));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link Action#getDescription() description} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for description (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableAction withDescription(String value) {
    if (Objects.equals(this.description, value)) return this;
    return validate(new ImmutableAction(
        this,
        this.connectorId,
        value,
        this.camelConnectorGAV,
        this.camelConnectorPrefix,
        this.definition,
        this.id,
        this.name,
        this.tags));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link Action#getCamelConnectorGAV() camelConnectorGAV} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for camelConnectorGAV (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableAction withCamelConnectorGAV(String value) {
    if (Objects.equals(this.camelConnectorGAV, value)) return this;
    return validate(new ImmutableAction(
        this,
        this.connectorId,
        this.description,
        value,
        this.camelConnectorPrefix,
        this.definition,
        this.id,
        this.name,
        this.tags));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link Action#getCamelConnectorPrefix() camelConnectorPrefix} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for camelConnectorPrefix (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableAction withCamelConnectorPrefix(String value) {
    if (Objects.equals(this.camelConnectorPrefix, value)) return this;
    return validate(new ImmutableAction(
        this,
        this.connectorId,
        this.description,
        this.camelConnectorGAV,
        value,
        this.definition,
        this.id,
        this.name,
        this.tags));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link Action#getDefinition() definition} attribute.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for definition (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableAction withDefinition(ActionDefinition value) {
    if (this.definition == value) return this;
    return validate(new ImmutableAction(
        this,
        this.connectorId,
        this.description,
        this.camelConnectorGAV,
        this.camelConnectorPrefix,
        value,
        this.id,
        this.name,
        this.tags));
  }

  /**
   * Copy the current immutable object by setting a <i>present</i> value for the optional {@link Action#getId() id} attribute.
   * @param value The value for id
   * @return A modified copy of {@code this} object
   */
  public final ImmutableAction withId(String value) {
    String newValue = Objects.requireNonNull(value, "id");
    if (Objects.equals(this.id, newValue)) return this;
    return validate(new ImmutableAction(
        this,
        this.connectorId,
        this.description,
        this.camelConnectorGAV,
        this.camelConnectorPrefix,
        this.definition,
        newValue,
        this.name,
        this.tags));
  }

  /**
   * Copy the current immutable object by setting an optional value for the {@link Action#getId() id} attribute.
   * An equality check is used on inner nullable value to prevent copying of the same value by returning {@code this}.
   * @param optional A value for id
   * @return A modified copy of {@code this} object
   */
  public final ImmutableAction withId(Optional<String> optional) {
    String value = optional.orElse(null);
    if (Objects.equals(this.id, value)) return this;
    return validate(new ImmutableAction(
        this,
        this.connectorId,
        this.description,
        this.camelConnectorGAV,
        this.camelConnectorPrefix,
        this.definition,
        value,
        this.name,
        this.tags));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link Action#getName() name} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for name (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableAction withName(String value) {
    if (Objects.equals(this.name, value)) return this;
    return validate(new ImmutableAction(
        this,
        this.connectorId,
        this.description,
        this.camelConnectorGAV,
        this.camelConnectorPrefix,
        this.definition,
        this.id,
        value,
        this.tags));
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link Action#getTags() tags}.
   * @param elements The elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableAction withTags(String... elements) {
    SortedSet<String> newValue = createUnmodifiableSortedSet(false, createSafeList(Arrays.asList(elements), false, true));
    return validate(new ImmutableAction(
        this,
        this.connectorId,
        this.description,
        this.camelConnectorGAV,
        this.camelConnectorPrefix,
        this.definition,
        this.id,
        this.name,
        newValue));
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link Action#getTags() tags}.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param elements An iterable of tags elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableAction withTags(Iterable<String> elements) {
    if (this.tags == elements) return this;
    SortedSet<String> newValue = createUnmodifiableSortedSet(false, createSafeList(elements, false, true));
    return validate(new ImmutableAction(
        this,
        this.connectorId,
        this.description,
        this.camelConnectorGAV,
        this.camelConnectorPrefix,
        this.definition,
        this.id,
        this.name,
        newValue));
  }

  /**
   * This instance is equal to all instances of {@code ImmutableAction} that have equal attribute values.
   * @return {@code true} if {@code this} is equal to {@code another} instance
   */
  @Override
  public boolean equals(Object another) {
    if (this == another) return true;
    return another instanceof ImmutableAction
        && equalTo((ImmutableAction) another);
  }

  private boolean equalTo(ImmutableAction another) {
    return Objects.equals(connectorId, another.connectorId)
        && Objects.equals(description, another.description)
        && Objects.equals(camelConnectorGAV, another.camelConnectorGAV)
        && Objects.equals(camelConnectorPrefix, another.camelConnectorPrefix)
        && Objects.equals(definition, another.definition)
        && Objects.equals(id, another.id)
        && Objects.equals(name, another.name)
        && tags.equals(another.tags);
  }

  /**
   * Computes a hash code from attributes: {@code connectorId}, {@code description}, {@code camelConnectorGAV}, {@code camelConnectorPrefix}, {@code definition}, {@code id}, {@code name}, {@code tags}.
   * @return hashCode value
   */
  @Override
  public int hashCode() {
    int h = 5381;
    h += (h << 5) + Objects.hashCode(connectorId);
    h += (h << 5) + Objects.hashCode(description);
    h += (h << 5) + Objects.hashCode(camelConnectorGAV);
    h += (h << 5) + Objects.hashCode(camelConnectorPrefix);
    h += (h << 5) + Objects.hashCode(definition);
    h += (h << 5) + Objects.hashCode(id);
    h += (h << 5) + Objects.hashCode(name);
    h += (h << 5) + tags.hashCode();
    return h;
  }

  /**
   * Prints the immutable value {@code Action} with attribute values.
   * @return A string representation of the value
   */
  @Override
  public String toString() {
    StringBuilder builder = new StringBuilder("Action{");
    if (connectorId != null) {
      builder.append("connectorId=").append(connectorId);
    }
    if (description != null) {
      if (builder.length() > 7) builder.append(", ");
      builder.append("description=").append(description);
    }
    if (camelConnectorGAV != null) {
      if (builder.length() > 7) builder.append(", ");
      builder.append("camelConnectorGAV=").append(camelConnectorGAV);
    }
    if (camelConnectorPrefix != null) {
      if (builder.length() > 7) builder.append(", ");
      builder.append("camelConnectorPrefix=").append(camelConnectorPrefix);
    }
    if (definition != null) {
      if (builder.length() > 7) builder.append(", ");
      builder.append("definition=").append(definition);
    }
    if (id != null) {
      if (builder.length() > 7) builder.append(", ");
      builder.append("id=").append(id);
    }
    if (name != null) {
      if (builder.length() > 7) builder.append(", ");
      builder.append("name=").append(name);
    }
    if (builder.length() > 7) builder.append(", ");
    builder.append("tags=").append(tags);
    return builder.append("}").toString();
  }

  /**
   * Construct a new immutable {@code Action} instance.
   * @param connectorId The value for the {@code connectorId} attribute
   * @param description The value for the {@code description} attribute
   * @param camelConnectorGAV The value for the {@code camelConnectorGAV} attribute
   * @param camelConnectorPrefix The value for the {@code camelConnectorPrefix} attribute
   * @param definition The value for the {@code definition} attribute
   * @param id The value for the {@code id} attribute
   * @param name The value for the {@code name} attribute
   * @param tags The value for the {@code tags} attribute
   * @return An immutable Action instance
   */
  public static ImmutableAction of(String connectorId, String description, String camelConnectorGAV, String camelConnectorPrefix, ActionDefinition definition, Optional<String> id, String name, SortedSet<String> tags) {
    return of(connectorId, description, camelConnectorGAV, camelConnectorPrefix, definition, id, name, (Iterable<String>) tags);
  }

  /**
   * Construct a new immutable {@code Action} instance.
   * @param connectorId The value for the {@code connectorId} attribute
   * @param description The value for the {@code description} attribute
   * @param camelConnectorGAV The value for the {@code camelConnectorGAV} attribute
   * @param camelConnectorPrefix The value for the {@code camelConnectorPrefix} attribute
   * @param definition The value for the {@code definition} attribute
   * @param id The value for the {@code id} attribute
   * @param name The value for the {@code name} attribute
   * @param tags The value for the {@code tags} attribute
   * @return An immutable Action instance
   */
  public static ImmutableAction of(String connectorId, String description, String camelConnectorGAV, String camelConnectorPrefix, ActionDefinition definition, Optional<String> id, String name, Iterable<String> tags) {
    return validate(new ImmutableAction(connectorId, description, camelConnectorGAV, camelConnectorPrefix, definition, id, name, tags));
  }

  private static final Validator validator = Validation.buildDefaultValidatorFactory().getValidator();

  private static ImmutableAction validate(ImmutableAction instance) {
    Set<ConstraintViolation<ImmutableAction>> constraintViolations = validator.validate(instance);
    if (!constraintViolations.isEmpty()) {
      throw new ConstraintViolationException(constraintViolations);
    }
    return instance;
  }

  /**
   * Creates an immutable copy of a {@link Action} value.
   * Uses accessors to get values to initialize the new immutable instance.
   * If an instance is already immutable, it is returned as is.
   * @param instance The instance to copy
   * @return A copied immutable Action instance
   */
  public static ImmutableAction copyOf(Action instance) {
    if (instance instanceof ImmutableAction) {
      return (ImmutableAction) instance;
    }
    return new Action.Builder()
        .createFrom(instance)
        .build();
  }

  private Object readResolve() throws ObjectStreamException {
    return validate(this);
  }

  /**
   * Builds instances of type {@link ImmutableAction ImmutableAction}.
   * Initialize attributes and then invoke the {@link #build()} method to create an
   * immutable instance.
   * <p><em>{@code Builder} is not thread-safe and generally should not be stored in a field or collection,
   * but instead used immediately to create instances.</em>
   */
  @JsonIgnoreProperties(value = {"properties", "inputDataShape", "outputDataShape"}, allowGetters = true)
  static class Builder {
    private String connectorId;
    private String description;
    private String camelConnectorGAV;
    private String camelConnectorPrefix;
    private ActionDefinition definition;
    private String id;
    private String name;
    private List<String> tags = new ArrayList<String>();

    /**
     * Creates a builder for {@link ImmutableAction ImmutableAction} instances.
     */
    Builder() {
      if (!(this instanceof Action.Builder)) {
        throw new UnsupportedOperationException("Use: new Action.Builder()");
      }
    }

    /**
     * Fill a builder with attribute values from the provided {@code io.syndesis.model.connection.Action} instance.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    public final Action.Builder createFrom(Action instance) {
      Objects.requireNonNull(instance, "instance");
      from((Object) instance);
      return (Action.Builder) this;
    }

    /**
     * Fill a builder with attribute values from the provided {@code io.syndesis.model.WithName} instance.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    public final Action.Builder createFrom(WithName instance) {
      Objects.requireNonNull(instance, "instance");
      from((Object) instance);
      return (Action.Builder) this;
    }

    /**
     * Fill a builder with attribute values from the provided {@code io.syndesis.model.WithTags} instance.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    public final Action.Builder createFrom(WithTags instance) {
      Objects.requireNonNull(instance, "instance");
      from((Object) instance);
      return (Action.Builder) this;
    }

    private void from(Object object) {
      if (object instanceof Action) {
        Action instance = (Action) object;
        String descriptionValue = instance.getDescription();
        if (descriptionValue != null) {
          description(descriptionValue);
        }
        ActionDefinition definitionValue = instance.getDefinition();
        if (definitionValue != null) {
          definition(definitionValue);
        }
        Optional<String> idOptional = instance.getId();
        if (idOptional.isPresent()) {
          id(idOptional);
        }
        String connectorIdValue = instance.getConnectorId();
        if (connectorIdValue != null) {
          connectorId(connectorIdValue);
        }
        String camelConnectorPrefixValue = instance.getCamelConnectorPrefix();
        if (camelConnectorPrefixValue != null) {
          camelConnectorPrefix(camelConnectorPrefixValue);
        }
        String camelConnectorGAVValue = instance.getCamelConnectorGAV();
        if (camelConnectorGAVValue != null) {
          camelConnectorGAV(camelConnectorGAVValue);
        }
      }
      if (object instanceof WithName) {
        WithName instance = (WithName) object;
        String nameValue = instance.getName();
        if (nameValue != null) {
          name(nameValue);
        }
      }
      if (object instanceof WithTags) {
        WithTags instance = (WithTags) object;
        addAllTags(instance.getTags());
      }
    }

    /**
     * Initializes the value for the {@link Action#getConnectorId() connectorId} attribute.
     * @param connectorId The value for connectorId (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("connectorId")
    public final Action.Builder connectorId(String connectorId) {
      this.connectorId = connectorId;
      return (Action.Builder) this;
    }

    /**
     * Initializes the value for the {@link Action#getDescription() description} attribute.
     * @param description The value for description (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("description")
    public final Action.Builder description(String description) {
      this.description = description;
      return (Action.Builder) this;
    }

    /**
     * Initializes the value for the {@link Action#getCamelConnectorGAV() camelConnectorGAV} attribute.
     * @param camelConnectorGAV The value for camelConnectorGAV (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("camelConnectorGAV")
    public final Action.Builder camelConnectorGAV(String camelConnectorGAV) {
      this.camelConnectorGAV = camelConnectorGAV;
      return (Action.Builder) this;
    }

    /**
     * Initializes the value for the {@link Action#getCamelConnectorPrefix() camelConnectorPrefix} attribute.
     * @param camelConnectorPrefix The value for camelConnectorPrefix (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("camelConnectorPrefix")
    public final Action.Builder camelConnectorPrefix(String camelConnectorPrefix) {
      this.camelConnectorPrefix = camelConnectorPrefix;
      return (Action.Builder) this;
    }

    /**
     * Initializes the value for the {@link Action#getDefinition() definition} attribute.
     * @param definition The value for definition (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("definition")
    public final Action.Builder definition(ActionDefinition definition) {
      this.definition = definition;
      return (Action.Builder) this;
    }

    /**
     * Initializes the optional value {@link Action#getId() id} to id.
     * @param id The value for id
     * @return {@code this} builder for chained invocation
     */
    public final Action.Builder id(String id) {
      this.id = Objects.requireNonNull(id, "id");
      return (Action.Builder) this;
    }

    /**
     * Initializes the optional value {@link Action#getId() id} to id.
     * @param id The value for id
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("id")
    public final Action.Builder id(Optional<String> id) {
      this.id = id.orElse(null);
      return (Action.Builder) this;
    }

    /**
     * Initializes the value for the {@link Action#getName() name} attribute.
     * @param name The value for name (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("name")
    public final Action.Builder name(String name) {
      this.name = name;
      return (Action.Builder) this;
    }

    /**
     * Adds one element to {@link Action#getTags() tags} sortedSet.
     * @param element A tags element
     * @return {@code this} builder for use in a chained invocation
     */
    public final Action.Builder addTag(String element) {
      if (element != null) this.tags.add(element);
      return (Action.Builder) this;
    }

    /**
     * Adds elements to {@link Action#getTags() tags} sortedSet.
     * @param elements An array of tags elements
     * @return {@code this} builder for use in a chained invocation
     */
    public final Action.Builder addTag(String... elements) {
      for (String element : elements) {
        if (element != null) this.tags.add(element);
      }
      return (Action.Builder) this;
    }

    /**
     * Sets or replaces all elements for {@link Action#getTags() tags} sortedSet.
     * @param elements An iterable of tags elements
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("tags")
    public final Action.Builder tags(Iterable<String> elements) {
      this.tags.clear();
      return addAllTags(elements);
    }

    /**
     * Adds elements to {@link Action#getTags() tags} sortedSet.
     * @param elements An iterable of tags elements
     * @return {@code this} builder for use in a chained invocation
     */
    public final Action.Builder addAllTags(Iterable<String> elements) {
      for (String element : elements) {
        if (element != null) this.tags.add(element);
      }
      return (Action.Builder) this;
    }

    /**
     * Builds a new {@link ImmutableAction ImmutableAction}.
     * @return An immutable instance of Action
     * @throws java.lang.IllegalStateException if any required attributes are missing
     */
    public ImmutableAction build() {
      return ImmutableAction.validate(new ImmutableAction(
          null,
          connectorId,
          description,
          camelConnectorGAV,
          camelConnectorPrefix,
          definition,
          id,
          name,
          createUnmodifiableSortedSet(false, createSafeList(tags, false, false))));
    }
  }

  private static <T> List<T> createSafeList(Iterable<? extends T> iterable, boolean checkNulls, boolean skipNulls) {
    ArrayList<T> list;
    if (iterable instanceof Collection<?>) {
      int size = ((Collection<?>) iterable).size();
      if (size == 0) return Collections.emptyList();
      list = new ArrayList<T>();
    } else {
      list = new ArrayList<T>();
    }
    for (T element : iterable) {
      if (skipNulls && element == null) continue;
      if (checkNulls) Objects.requireNonNull(element, "element");
      list.add(element);
    }
    return list;
  }

  private static <T extends Comparable<T>> NavigableSet<T> createUnmodifiableSortedSet(boolean reverse, List<T> list) {
    TreeSet<T> set = reverse
        ? new TreeSet<T>(Collections.reverseOrder())
        : new TreeSet<T>();
    set.addAll(list);
    return Collections.unmodifiableNavigableSet(set);
  }
}
