/**
 * Copyright (C) 2016 Red Hat, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package io.syndesis.model.connection;

import com.fasterxml.jackson.annotation.JsonProperty;
import edu.umd.cs.findbugs.annotations.SuppressFBWarnings;
import java.io.ObjectStreamException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.Objects;
import java.util.Optional;
import java.util.Set;
import javax.annotation.Generated;
import javax.validation.ConstraintViolation;
import javax.validation.ConstraintViolationException;
import javax.validation.Validation;
import javax.validation.Validator;

/**
 * Immutable implementation of {@link ActionDefinition}.
 * <p>
 * Use the builder to create immutable instances:
 * {@code new ActionDefinition.Builder()}.
 * Use the static factory method to create immutable instances:
 * {@code ImmutableActionDefinition.of()}.
 */
@SuppressWarnings({"all"})
@SuppressFBWarnings
@Generated({"Immutables.generator", "ActionDefinition"})
final class ImmutableActionDefinition implements ActionDefinition {
  private final DataShape inputDataShape;
  private final DataShape outputDataShape;
  private final List<ActionDefinition.ActionDefinitionStep> propertyDefinitionSteps;

  private ImmutableActionDefinition(
      Optional<DataShape> inputDataShape,
      Optional<DataShape> outputDataShape,
      Iterable<? extends ActionDefinition.ActionDefinitionStep> propertyDefinitionSteps) {
    this.inputDataShape = inputDataShape.orElse(null);
    this.outputDataShape = outputDataShape.orElse(null);
    this.propertyDefinitionSteps = createUnmodifiableList(false, createSafeList(propertyDefinitionSteps, true, false));
  }

  private ImmutableActionDefinition(
      ImmutableActionDefinition original,
      DataShape inputDataShape,
      DataShape outputDataShape,
      List<ActionDefinition.ActionDefinitionStep> propertyDefinitionSteps) {
    this.inputDataShape = inputDataShape;
    this.outputDataShape = outputDataShape;
    this.propertyDefinitionSteps = propertyDefinitionSteps;
  }

  /**
   * @return The value of the {@code inputDataShape} attribute
   */
  @JsonProperty("inputDataShape")
  @Override
  public Optional<DataShape> getInputDataShape() {
    return Optional.ofNullable(inputDataShape);
  }

  /**
   * @return The value of the {@code outputDataShape} attribute
   */
  @JsonProperty("outputDataShape")
  @Override
  public Optional<DataShape> getOutputDataShape() {
    return Optional.ofNullable(outputDataShape);
  }

  /**
   * @return The value of the {@code propertyDefinitionSteps} attribute
   */
  @JsonProperty("propertyDefinitionSteps")
  @Override
  public List<ActionDefinition.ActionDefinitionStep> getPropertyDefinitionSteps() {
    return propertyDefinitionSteps;
  }

  /**
   * Copy the current immutable object by setting a <i>present</i> value for the optional {@link ActionDefinition#getInputDataShape() inputDataShape} attribute.
   * @param value The value for inputDataShape
   * @return A modified copy of {@code this} object
   */
  public final ImmutableActionDefinition withInputDataShape(DataShape value) {
    DataShape newValue = Objects.requireNonNull(value, "inputDataShape");
    if (this.inputDataShape == newValue) return this;
    return validate(new ImmutableActionDefinition(this, newValue, this.outputDataShape, this.propertyDefinitionSteps));
  }

  /**
   * Copy the current immutable object by setting an optional value for the {@link ActionDefinition#getInputDataShape() inputDataShape} attribute.
   * A shallow reference equality check is used on unboxed optional value to prevent copying of the same value by returning {@code this}.
   * @param optional A value for inputDataShape
   * @return A modified copy of {@code this} object
   */
  public final ImmutableActionDefinition withInputDataShape(Optional<? extends DataShape> optional) {
    DataShape value = optional.orElse(null);
    if (this.inputDataShape == value) return this;
    return validate(new ImmutableActionDefinition(this, value, this.outputDataShape, this.propertyDefinitionSteps));
  }

  /**
   * Copy the current immutable object by setting a <i>present</i> value for the optional {@link ActionDefinition#getOutputDataShape() outputDataShape} attribute.
   * @param value The value for outputDataShape
   * @return A modified copy of {@code this} object
   */
  public final ImmutableActionDefinition withOutputDataShape(DataShape value) {
    DataShape newValue = Objects.requireNonNull(value, "outputDataShape");
    if (this.outputDataShape == newValue) return this;
    return validate(new ImmutableActionDefinition(this, this.inputDataShape, newValue, this.propertyDefinitionSteps));
  }

  /**
   * Copy the current immutable object by setting an optional value for the {@link ActionDefinition#getOutputDataShape() outputDataShape} attribute.
   * A shallow reference equality check is used on unboxed optional value to prevent copying of the same value by returning {@code this}.
   * @param optional A value for outputDataShape
   * @return A modified copy of {@code this} object
   */
  public final ImmutableActionDefinition withOutputDataShape(Optional<? extends DataShape> optional) {
    DataShape value = optional.orElse(null);
    if (this.outputDataShape == value) return this;
    return validate(new ImmutableActionDefinition(this, this.inputDataShape, value, this.propertyDefinitionSteps));
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link ActionDefinition#getPropertyDefinitionSteps() propertyDefinitionSteps}.
   * @param elements The elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableActionDefinition withPropertyDefinitionSteps(ActionDefinition.ActionDefinitionStep... elements) {
    List<ActionDefinition.ActionDefinitionStep> newValue = createUnmodifiableList(false, createSafeList(Arrays.asList(elements), true, false));
    return validate(new ImmutableActionDefinition(this, this.inputDataShape, this.outputDataShape, newValue));
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link ActionDefinition#getPropertyDefinitionSteps() propertyDefinitionSteps}.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param elements An iterable of propertyDefinitionSteps elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableActionDefinition withPropertyDefinitionSteps(Iterable<? extends ActionDefinition.ActionDefinitionStep> elements) {
    if (this.propertyDefinitionSteps == elements) return this;
    List<ActionDefinition.ActionDefinitionStep> newValue = createUnmodifiableList(false, createSafeList(elements, true, false));
    return validate(new ImmutableActionDefinition(this, this.inputDataShape, this.outputDataShape, newValue));
  }

  /**
   * This instance is equal to all instances of {@code ImmutableActionDefinition} that have equal attribute values.
   * @return {@code true} if {@code this} is equal to {@code another} instance
   */
  @Override
  public boolean equals(Object another) {
    if (this == another) return true;
    return another instanceof ImmutableActionDefinition
        && equalTo((ImmutableActionDefinition) another);
  }

  private boolean equalTo(ImmutableActionDefinition another) {
    return Objects.equals(inputDataShape, another.inputDataShape)
        && Objects.equals(outputDataShape, another.outputDataShape)
        && propertyDefinitionSteps.equals(another.propertyDefinitionSteps);
  }

  /**
   * Computes a hash code from attributes: {@code inputDataShape}, {@code outputDataShape}, {@code propertyDefinitionSteps}.
   * @return hashCode value
   */
  @Override
  public int hashCode() {
    int h = 5381;
    h += (h << 5) + Objects.hashCode(inputDataShape);
    h += (h << 5) + Objects.hashCode(outputDataShape);
    h += (h << 5) + propertyDefinitionSteps.hashCode();
    return h;
  }

  /**
   * Prints the immutable value {@code ActionDefinition} with attribute values.
   * @return A string representation of the value
   */
  @Override
  public String toString() {
    StringBuilder builder = new StringBuilder("ActionDefinition{");
    if (inputDataShape != null) {
      builder.append("inputDataShape=").append(inputDataShape);
    }
    if (outputDataShape != null) {
      if (builder.length() > 17) builder.append(", ");
      builder.append("outputDataShape=").append(outputDataShape);
    }
    if (builder.length() > 17) builder.append(", ");
    builder.append("propertyDefinitionSteps=").append(propertyDefinitionSteps);
    return builder.append("}").toString();
  }

  /**
   * Construct a new immutable {@code ActionDefinition} instance.
   * @param inputDataShape The value for the {@code inputDataShape} attribute
   * @param outputDataShape The value for the {@code outputDataShape} attribute
   * @param propertyDefinitionSteps The value for the {@code propertyDefinitionSteps} attribute
   * @return An immutable ActionDefinition instance
   */
  public static ImmutableActionDefinition of(Optional<DataShape> inputDataShape, Optional<DataShape> outputDataShape, List<ActionDefinition.ActionDefinitionStep> propertyDefinitionSteps) {
    return of(inputDataShape, outputDataShape, (Iterable<? extends ActionDefinition.ActionDefinitionStep>) propertyDefinitionSteps);
  }

  /**
   * Construct a new immutable {@code ActionDefinition} instance.
   * @param inputDataShape The value for the {@code inputDataShape} attribute
   * @param outputDataShape The value for the {@code outputDataShape} attribute
   * @param propertyDefinitionSteps The value for the {@code propertyDefinitionSteps} attribute
   * @return An immutable ActionDefinition instance
   */
  public static ImmutableActionDefinition of(Optional<DataShape> inputDataShape, Optional<DataShape> outputDataShape, Iterable<? extends ActionDefinition.ActionDefinitionStep> propertyDefinitionSteps) {
    return validate(new ImmutableActionDefinition(inputDataShape, outputDataShape, propertyDefinitionSteps));
  }

  private static final Validator validator = Validation.buildDefaultValidatorFactory().getValidator();

  private static ImmutableActionDefinition validate(ImmutableActionDefinition instance) {
    Set<ConstraintViolation<ImmutableActionDefinition>> constraintViolations = validator.validate(instance);
    if (!constraintViolations.isEmpty()) {
      throw new ConstraintViolationException(constraintViolations);
    }
    return instance;
  }

  /**
   * Creates an immutable copy of a {@link ActionDefinition} value.
   * Uses accessors to get values to initialize the new immutable instance.
   * If an instance is already immutable, it is returned as is.
   * @param instance The instance to copy
   * @return A copied immutable ActionDefinition instance
   */
  public static ImmutableActionDefinition copyOf(ActionDefinition instance) {
    if (instance instanceof ImmutableActionDefinition) {
      return (ImmutableActionDefinition) instance;
    }
    return new ActionDefinition.Builder()
        .createFrom(instance)
        .build();
  }

  private Object readResolve() throws ObjectStreamException {
    return validate(this);
  }

  /**
   * Builds instances of type {@link ImmutableActionDefinition ImmutableActionDefinition}.
   * Initialize attributes and then invoke the {@link #build()} method to create an
   * immutable instance.
   * <p><em>{@code Builder} is not thread-safe and generally should not be stored in a field or collection,
   * but instead used immediately to create instances.</em>
   */
  static class Builder {
    private DataShape inputDataShape;
    private DataShape outputDataShape;
    private List<ActionDefinition.ActionDefinitionStep> propertyDefinitionSteps = new ArrayList<ActionDefinition.ActionDefinitionStep>();

    /**
     * Creates a builder for {@link ImmutableActionDefinition ImmutableActionDefinition} instances.
     */
    Builder() {
      if (!(this instanceof ActionDefinition.Builder)) {
        throw new UnsupportedOperationException("Use: new ActionDefinition.Builder()");
      }
    }

    /**
     * Fill a builder with attribute values from the provided {@code ActionDefinition} instance.
     * Regular attribute values will be replaced with those from the given instance.
     * Absent optional values will not replace present values.
     * Collection elements and entries will be added, not replaced.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    public final ActionDefinition.Builder createFrom(ActionDefinition instance) {
      Objects.requireNonNull(instance, "instance");
      Optional<DataShape> inputDataShapeOptional = instance.getInputDataShape();
      if (inputDataShapeOptional.isPresent()) {
        inputDataShape(inputDataShapeOptional);
      }
      Optional<DataShape> outputDataShapeOptional = instance.getOutputDataShape();
      if (outputDataShapeOptional.isPresent()) {
        outputDataShape(outputDataShapeOptional);
      }
      addAllPropertyDefinitionSteps(instance.getPropertyDefinitionSteps());
      return (ActionDefinition.Builder) this;
    }

    /**
     * Initializes the optional value {@link ActionDefinition#getInputDataShape() inputDataShape} to inputDataShape.
     * @param inputDataShape The value for inputDataShape
     * @return {@code this} builder for chained invocation
     */
    public final ActionDefinition.Builder inputDataShape(DataShape inputDataShape) {
      this.inputDataShape = Objects.requireNonNull(inputDataShape, "inputDataShape");
      return (ActionDefinition.Builder) this;
    }

    /**
     * Initializes the optional value {@link ActionDefinition#getInputDataShape() inputDataShape} to inputDataShape.
     * @param inputDataShape The value for inputDataShape
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("inputDataShape")
    public final ActionDefinition.Builder inputDataShape(Optional<? extends DataShape> inputDataShape) {
      this.inputDataShape = inputDataShape.orElse(null);
      return (ActionDefinition.Builder) this;
    }

    /**
     * Initializes the optional value {@link ActionDefinition#getOutputDataShape() outputDataShape} to outputDataShape.
     * @param outputDataShape The value for outputDataShape
     * @return {@code this} builder for chained invocation
     */
    public final ActionDefinition.Builder outputDataShape(DataShape outputDataShape) {
      this.outputDataShape = Objects.requireNonNull(outputDataShape, "outputDataShape");
      return (ActionDefinition.Builder) this;
    }

    /**
     * Initializes the optional value {@link ActionDefinition#getOutputDataShape() outputDataShape} to outputDataShape.
     * @param outputDataShape The value for outputDataShape
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("outputDataShape")
    public final ActionDefinition.Builder outputDataShape(Optional<? extends DataShape> outputDataShape) {
      this.outputDataShape = outputDataShape.orElse(null);
      return (ActionDefinition.Builder) this;
    }

    /**
     * Adds one element to {@link ActionDefinition#getPropertyDefinitionSteps() propertyDefinitionSteps} list.
     * @param element A propertyDefinitionSteps element
     * @return {@code this} builder for use in a chained invocation
     */
    public final ActionDefinition.Builder addPropertyDefinitionStep(ActionDefinition.ActionDefinitionStep element) {
      this.propertyDefinitionSteps.add(Objects.requireNonNull(element, "propertyDefinitionSteps element"));
      return (ActionDefinition.Builder) this;
    }

    /**
     * Adds elements to {@link ActionDefinition#getPropertyDefinitionSteps() propertyDefinitionSteps} list.
     * @param elements An array of propertyDefinitionSteps elements
     * @return {@code this} builder for use in a chained invocation
     */
    public final ActionDefinition.Builder addPropertyDefinitionStep(ActionDefinition.ActionDefinitionStep... elements) {
      for (ActionDefinition.ActionDefinitionStep element : elements) {
        this.propertyDefinitionSteps.add(Objects.requireNonNull(element, "propertyDefinitionSteps element"));
      }
      return (ActionDefinition.Builder) this;
    }

    /**
     * Sets or replaces all elements for {@link ActionDefinition#getPropertyDefinitionSteps() propertyDefinitionSteps} list.
     * @param elements An iterable of propertyDefinitionSteps elements
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("propertyDefinitionSteps")
    public final ActionDefinition.Builder propertyDefinitionSteps(Iterable<? extends ActionDefinition.ActionDefinitionStep> elements) {
      this.propertyDefinitionSteps.clear();
      return addAllPropertyDefinitionSteps(elements);
    }

    /**
     * Adds elements to {@link ActionDefinition#getPropertyDefinitionSteps() propertyDefinitionSteps} list.
     * @param elements An iterable of propertyDefinitionSteps elements
     * @return {@code this} builder for use in a chained invocation
     */
    public final ActionDefinition.Builder addAllPropertyDefinitionSteps(Iterable<? extends ActionDefinition.ActionDefinitionStep> elements) {
      for (ActionDefinition.ActionDefinitionStep element : elements) {
        this.propertyDefinitionSteps.add(Objects.requireNonNull(element, "propertyDefinitionSteps element"));
      }
      return (ActionDefinition.Builder) this;
    }

    /**
     * Builds a new {@link ImmutableActionDefinition ImmutableActionDefinition}.
     * @return An immutable instance of ActionDefinition
     * @throws java.lang.IllegalStateException if any required attributes are missing
     */
    public ImmutableActionDefinition build() {
      return ImmutableActionDefinition.validate(new ImmutableActionDefinition(null, inputDataShape, outputDataShape, createUnmodifiableList(true, propertyDefinitionSteps)));
    }
  }

  private static <T> List<T> createSafeList(Iterable<? extends T> iterable, boolean checkNulls, boolean skipNulls) {
    ArrayList<T> list;
    if (iterable instanceof Collection<?>) {
      int size = ((Collection<?>) iterable).size();
      if (size == 0) return Collections.emptyList();
      list = new ArrayList<T>();
    } else {
      list = new ArrayList<T>();
    }
    for (T element : iterable) {
      if (skipNulls && element == null) continue;
      if (checkNulls) Objects.requireNonNull(element, "element");
      list.add(element);
    }
    return list;
  }

  private static <T> List<T> createUnmodifiableList(boolean clone, List<T> list) {
    switch(list.size()) {
    case 0: return Collections.emptyList();
    case 1: return Collections.singletonList(list.get(0));
    default:
      if (clone) {
        return Collections.unmodifiableList(new ArrayList<T>(list));
      } else {
        if (list instanceof ArrayList<?>) {
          ((ArrayList<?>) list).trimToSize();
        }
        return Collections.unmodifiableList(list);
      }
    }
  }
}
