/**
 * Copyright (C) 2016 Red Hat, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package io.syndesis.model.connection;

import com.fasterxml.jackson.annotation.JsonProperty;
import edu.umd.cs.findbugs.annotations.SuppressFBWarnings;
import io.syndesis.model.WithConfigurationProperties;
import io.syndesis.model.WithName;
import io.syndesis.model.WithProperties;
import java.io.ObjectStreamException;
import java.util.Collections;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.Objects;
import java.util.Set;
import javax.annotation.Generated;
import javax.validation.ConstraintViolation;
import javax.validation.ConstraintViolationException;
import javax.validation.Validation;
import javax.validation.Validator;

/**
 * Immutable implementation of {@link ActionDefinition.ActionDefinitionStep}.
 * <p>
 * Use the builder to create immutable instances:
 * {@code new ActionDefinition.ActionDefinitionStep.Builder()}.
 * Use the static factory method to create immutable instances:
 * {@code ImmutableActionDefinitionStep.of()}.
 */
@SuppressWarnings({"all"})
@SuppressFBWarnings
@Generated({"Immutables.generator", "ActionDefinition.ActionDefinitionStep"})
final class ImmutableActionDefinitionStep
    implements ActionDefinition.ActionDefinitionStep {
  private final String description;
  private final String name;
  private final Map<String, String> configuredProperties;
  private final Map<String, ConfigurationProperty> properties;

  private ImmutableActionDefinitionStep(
      String description,
      String name,
      Map<String, ? extends String> configuredProperties,
      Map<String, ? extends ConfigurationProperty> properties) {
    this.description = description;
    this.name = name;
    this.configuredProperties = createUnmodifiableMap(true, false, configuredProperties);
    this.properties = createUnmodifiableMap(true, false, properties);
  }

  private ImmutableActionDefinitionStep(
      ImmutableActionDefinitionStep original,
      String description,
      String name,
      Map<String, String> configuredProperties,
      Map<String, ConfigurationProperty> properties) {
    this.description = description;
    this.name = name;
    this.configuredProperties = configuredProperties;
    this.properties = properties;
  }

  /**
   * @return The value of the {@code description} attribute
   */
  @JsonProperty("description")
  @Override
  public String getDescription() {
    return description;
  }

  /**
   * @return The value of the {@code name} attribute
   */
  @JsonProperty("name")
  @Override
  public String getName() {
    return name;
  }

  /**
   * @return The value of the {@code configuredProperties} attribute
   */
  @JsonProperty("configuredProperties")
  @Override
  public Map<String, String> getConfiguredProperties() {
    return configuredProperties;
  }

  /**
   * @return The value of the {@code properties} attribute
   */
  @JsonProperty("properties")
  @Override
  public Map<String, ConfigurationProperty> getProperties() {
    return properties;
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ActionDefinition.ActionDefinitionStep#getDescription() description} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for description (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableActionDefinitionStep withDescription(String value) {
    if (Objects.equals(this.description, value)) return this;
    return validate(new ImmutableActionDefinitionStep(this, value, this.name, this.configuredProperties, this.properties));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ActionDefinition.ActionDefinitionStep#getName() name} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for name (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableActionDefinitionStep withName(String value) {
    if (Objects.equals(this.name, value)) return this;
    return validate(new ImmutableActionDefinitionStep(this, this.description, value, this.configuredProperties, this.properties));
  }

  /**
   * Copy the current immutable object by replacing the {@link ActionDefinition.ActionDefinitionStep#getConfiguredProperties() configuredProperties} map with the specified map.
   * Nulls are not permitted as keys or values.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param entries The entries to be added to the configuredProperties map
   * @return A modified copy of {@code this} object
   */
  public final ImmutableActionDefinitionStep withConfiguredProperties(Map<String, ? extends String> entries) {
    if (this.configuredProperties == entries) return this;
    Map<String, String> newValue = createUnmodifiableMap(true, false, entries);
    return validate(new ImmutableActionDefinitionStep(this, this.description, this.name, newValue, this.properties));
  }

  /**
   * Copy the current immutable object by replacing the {@link ActionDefinition.ActionDefinitionStep#getProperties() properties} map with the specified map.
   * Nulls are not permitted as keys or values.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param entries The entries to be added to the properties map
   * @return A modified copy of {@code this} object
   */
  public final ImmutableActionDefinitionStep withProperties(Map<String, ? extends ConfigurationProperty> entries) {
    if (this.properties == entries) return this;
    Map<String, ConfigurationProperty> newValue = createUnmodifiableMap(true, false, entries);
    return validate(new ImmutableActionDefinitionStep(this, this.description, this.name, this.configuredProperties, newValue));
  }

  /**
   * This instance is equal to all instances of {@code ImmutableActionDefinitionStep} that have equal attribute values.
   * @return {@code true} if {@code this} is equal to {@code another} instance
   */
  @Override
  public boolean equals(Object another) {
    if (this == another) return true;
    return another instanceof ImmutableActionDefinitionStep
        && equalTo((ImmutableActionDefinitionStep) another);
  }

  private boolean equalTo(ImmutableActionDefinitionStep another) {
    return Objects.equals(description, another.description)
        && Objects.equals(name, another.name)
        && configuredProperties.equals(another.configuredProperties)
        && properties.equals(another.properties);
  }

  /**
   * Computes a hash code from attributes: {@code description}, {@code name}, {@code configuredProperties}, {@code properties}.
   * @return hashCode value
   */
  @Override
  public int hashCode() {
    int h = 5381;
    h += (h << 5) + Objects.hashCode(description);
    h += (h << 5) + Objects.hashCode(name);
    h += (h << 5) + configuredProperties.hashCode();
    h += (h << 5) + properties.hashCode();
    return h;
  }

  /**
   * Prints the immutable value {@code ActionDefinitionStep} with attribute values.
   * @return A string representation of the value
   */
  @Override
  public String toString() {
    return "ActionDefinitionStep{"
        + "description=" + description
        + ", name=" + name
        + ", configuredProperties=" + configuredProperties
        + ", properties=" + properties
        + "}";
  }

  /**
   * Construct a new immutable {@code ActionDefinitionStep} instance.
   * @param description The value for the {@code description} attribute
   * @param name The value for the {@code name} attribute
   * @param configuredProperties The value for the {@code configuredProperties} attribute
   * @param properties The value for the {@code properties} attribute
   * @return An immutable ActionDefinitionStep instance
   */
  public static ImmutableActionDefinitionStep of(String description, String name, Map<String, ? extends String> configuredProperties, Map<String, ? extends ConfigurationProperty> properties) {
    return validate(new ImmutableActionDefinitionStep(description, name, configuredProperties, properties));
  }

  private static final Validator validator = Validation.buildDefaultValidatorFactory().getValidator();

  private static ImmutableActionDefinitionStep validate(ImmutableActionDefinitionStep instance) {
    Set<ConstraintViolation<ImmutableActionDefinitionStep>> constraintViolations = validator.validate(instance);
    if (!constraintViolations.isEmpty()) {
      throw new ConstraintViolationException(constraintViolations);
    }
    return instance;
  }

  /**
   * Creates an immutable copy of a {@link ActionDefinition.ActionDefinitionStep} value.
   * Uses accessors to get values to initialize the new immutable instance.
   * If an instance is already immutable, it is returned as is.
   * @param instance The instance to copy
   * @return A copied immutable ActionDefinitionStep instance
   */
  public static ImmutableActionDefinitionStep copyOf(ActionDefinition.ActionDefinitionStep instance) {
    if (instance instanceof ImmutableActionDefinitionStep) {
      return (ImmutableActionDefinitionStep) instance;
    }
    return new ActionDefinition.ActionDefinitionStep.Builder()
        .createFrom(instance)
        .build();
  }

  private Object readResolve() throws ObjectStreamException {
    return validate(this);
  }

  /**
   * Builds instances of type {@link ImmutableActionDefinitionStep ImmutableActionDefinitionStep}.
   * Initialize attributes and then invoke the {@link #build()} method to create an
   * immutable instance.
   * <p><em>{@code Builder} is not thread-safe and generally should not be stored in a field or collection,
   * but instead used immediately to create instances.</em>
   */
  static class Builder {
    private String description;
    private String name;
    private Map<String, String> configuredProperties = new LinkedHashMap<String, String>();
    private Map<String, ConfigurationProperty> properties = new LinkedHashMap<String, ConfigurationProperty>();

    /**
     * Creates a builder for {@link ImmutableActionDefinitionStep ImmutableActionDefinitionStep} instances.
     */
    Builder() {
      if (!(this instanceof ActionDefinition.ActionDefinitionStep.Builder)) {
        throw new UnsupportedOperationException("Use: new ActionDefinition.ActionDefinitionStep.Builder()");
      }
    }

    /**
     * Fill a builder with attribute values from the provided {@code io.syndesis.model.WithConfigurationProperties} instance.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    public final ActionDefinition.ActionDefinitionStep.Builder createFrom(WithConfigurationProperties instance) {
      Objects.requireNonNull(instance, "instance");
      from((Object) instance);
      return (ActionDefinition.ActionDefinitionStep.Builder) this;
    }

    /**
     * Fill a builder with attribute values from the provided {@code io.syndesis.model.WithProperties} instance.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    public final ActionDefinition.ActionDefinitionStep.Builder createFrom(WithProperties instance) {
      Objects.requireNonNull(instance, "instance");
      from((Object) instance);
      return (ActionDefinition.ActionDefinitionStep.Builder) this;
    }

    /**
     * Fill a builder with attribute values from the provided {@code io.syndesis.model.connection.ActionDefinition.ActionDefinitionStep} instance.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    public final ActionDefinition.ActionDefinitionStep.Builder createFrom(ActionDefinition.ActionDefinitionStep instance) {
      Objects.requireNonNull(instance, "instance");
      from((Object) instance);
      return (ActionDefinition.ActionDefinitionStep.Builder) this;
    }

    /**
     * Fill a builder with attribute values from the provided {@code io.syndesis.model.WithName} instance.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    public final ActionDefinition.ActionDefinitionStep.Builder createFrom(WithName instance) {
      Objects.requireNonNull(instance, "instance");
      from((Object) instance);
      return (ActionDefinition.ActionDefinitionStep.Builder) this;
    }

    private void from(Object object) {
      if (object instanceof WithConfigurationProperties) {
        WithConfigurationProperties instance = (WithConfigurationProperties) object;
        putAllProperties(instance.getProperties());
      }
      if (object instanceof WithProperties) {
        WithProperties instance = (WithProperties) object;
        putAllConfiguredProperties(instance.getConfiguredProperties());
      }
      if (object instanceof ActionDefinition.ActionDefinitionStep) {
        ActionDefinition.ActionDefinitionStep instance = (ActionDefinition.ActionDefinitionStep) object;
        String descriptionValue = instance.getDescription();
        if (descriptionValue != null) {
          description(descriptionValue);
        }
      }
      if (object instanceof WithName) {
        WithName instance = (WithName) object;
        String nameValue = instance.getName();
        if (nameValue != null) {
          name(nameValue);
        }
      }
    }

    /**
     * Initializes the value for the {@link ActionDefinition.ActionDefinitionStep#getDescription() description} attribute.
     * @param description The value for description (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("description")
    public final ActionDefinition.ActionDefinitionStep.Builder description(String description) {
      this.description = description;
      return (ActionDefinition.ActionDefinitionStep.Builder) this;
    }

    /**
     * Initializes the value for the {@link ActionDefinition.ActionDefinitionStep#getName() name} attribute.
     * @param name The value for name (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("name")
    public final ActionDefinition.ActionDefinitionStep.Builder name(String name) {
      this.name = name;
      return (ActionDefinition.ActionDefinitionStep.Builder) this;
    }

    /**
     * Put one entry to the {@link ActionDefinition.ActionDefinitionStep#getConfiguredProperties() configuredProperties} map.
     * @param key The key in the configuredProperties map
     * @param value The associated value in the configuredProperties map
     * @return {@code this} builder for use in a chained invocation
     */
    public final ActionDefinition.ActionDefinitionStep.Builder putConfiguredProperty(String key, String value) {
      this.configuredProperties.put(
          Objects.requireNonNull(key, "configuredProperties key"),
          Objects.requireNonNull(value, "configuredProperties value"));
      return (ActionDefinition.ActionDefinitionStep.Builder) this;
    }

    /**
     * Put one entry to the {@link ActionDefinition.ActionDefinitionStep#getConfiguredProperties() configuredProperties} map. Nulls are not permitted
     * @param entry The key and value entry
     * @return {@code this} builder for use in a chained invocation
     */
    public final ActionDefinition.ActionDefinitionStep.Builder putConfiguredProperty(Map.Entry<String, ? extends String> entry) {
      String k = entry.getKey();
      String v = entry.getValue();
      this.configuredProperties.put(
          Objects.requireNonNull(k, "configuredProperties key"),
          Objects.requireNonNull(v, "configuredProperties value"));
      return (ActionDefinition.ActionDefinitionStep.Builder) this;
    }

    /**
     * Sets or replaces all mappings from the specified map as entries for the {@link ActionDefinition.ActionDefinitionStep#getConfiguredProperties() configuredProperties} map. Nulls are not permitted
     * @param configuredProperties The entries that will be added to the configuredProperties map
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("configuredProperties")
    public final ActionDefinition.ActionDefinitionStep.Builder configuredProperties(Map<String, ? extends String> configuredProperties) {
      this.configuredProperties.clear();
      return putAllConfiguredProperties(configuredProperties);
    }

    /**
     * Put all mappings from the specified map as entries to {@link ActionDefinition.ActionDefinitionStep#getConfiguredProperties() configuredProperties} map. Nulls are not permitted
     * @param configuredProperties The entries that will be added to the configuredProperties map
     * @return {@code this} builder for use in a chained invocation
     */
    public final ActionDefinition.ActionDefinitionStep.Builder putAllConfiguredProperties(Map<String, ? extends String> configuredProperties) {
      for (Map.Entry<String, ? extends String> entry : configuredProperties.entrySet()) {
        String k = entry.getKey();
        String v = entry.getValue();
        this.configuredProperties.put(
            Objects.requireNonNull(k, "configuredProperties key"),
            Objects.requireNonNull(v, "configuredProperties value"));
      }
      return (ActionDefinition.ActionDefinitionStep.Builder) this;
    }

    /**
     * Put one entry to the {@link ActionDefinition.ActionDefinitionStep#getProperties() properties} map.
     * @param key The key in the properties map
     * @param value The associated value in the properties map
     * @return {@code this} builder for use in a chained invocation
     */
    public final ActionDefinition.ActionDefinitionStep.Builder putProperty(String key, ConfigurationProperty value) {
      this.properties.put(
          Objects.requireNonNull(key, "properties key"),
          Objects.requireNonNull(value, "properties value"));
      return (ActionDefinition.ActionDefinitionStep.Builder) this;
    }

    /**
     * Put one entry to the {@link ActionDefinition.ActionDefinitionStep#getProperties() properties} map. Nulls are not permitted
     * @param entry The key and value entry
     * @return {@code this} builder for use in a chained invocation
     */
    public final ActionDefinition.ActionDefinitionStep.Builder putProperty(Map.Entry<String, ? extends ConfigurationProperty> entry) {
      String k = entry.getKey();
      ConfigurationProperty v = entry.getValue();
      this.properties.put(
          Objects.requireNonNull(k, "properties key"),
          Objects.requireNonNull(v, "properties value"));
      return (ActionDefinition.ActionDefinitionStep.Builder) this;
    }

    /**
     * Sets or replaces all mappings from the specified map as entries for the {@link ActionDefinition.ActionDefinitionStep#getProperties() properties} map. Nulls are not permitted
     * @param properties The entries that will be added to the properties map
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("properties")
    public final ActionDefinition.ActionDefinitionStep.Builder properties(Map<String, ? extends ConfigurationProperty> properties) {
      this.properties.clear();
      return putAllProperties(properties);
    }

    /**
     * Put all mappings from the specified map as entries to {@link ActionDefinition.ActionDefinitionStep#getProperties() properties} map. Nulls are not permitted
     * @param properties The entries that will be added to the properties map
     * @return {@code this} builder for use in a chained invocation
     */
    public final ActionDefinition.ActionDefinitionStep.Builder putAllProperties(Map<String, ? extends ConfigurationProperty> properties) {
      for (Map.Entry<String, ? extends ConfigurationProperty> entry : properties.entrySet()) {
        String k = entry.getKey();
        ConfigurationProperty v = entry.getValue();
        this.properties.put(
            Objects.requireNonNull(k, "properties key"),
            Objects.requireNonNull(v, "properties value"));
      }
      return (ActionDefinition.ActionDefinitionStep.Builder) this;
    }

    /**
     * Builds a new {@link ImmutableActionDefinitionStep ImmutableActionDefinitionStep}.
     * @return An immutable instance of ActionDefinitionStep
     * @throws java.lang.IllegalStateException if any required attributes are missing
     */
    public ImmutableActionDefinitionStep build() {
      return ImmutableActionDefinitionStep.validate(new ImmutableActionDefinitionStep(
          null,
          description,
          name,
          createUnmodifiableMap(false, false, configuredProperties),
          createUnmodifiableMap(false, false, properties)));
    }
  }

  private static <K, V> Map<K, V> createUnmodifiableMap(boolean checkNulls, boolean skipNulls, Map<? extends K, ? extends V> map) {
    switch (map.size()) {
    case 0: return Collections.emptyMap();
    case 1: {
      Map.Entry<? extends K, ? extends V> e = map.entrySet().iterator().next();
      K k = e.getKey();
      V v = e.getValue();
      if (checkNulls) {
        Objects.requireNonNull(k, "key");
        Objects.requireNonNull(v, "value");
      }
      if (skipNulls && (k == null || v == null)) {
        return Collections.emptyMap();
      }
      return Collections.singletonMap(k, v);
    }
    default: {
      Map<K, V> linkedMap = new LinkedHashMap<K, V>(map.size());
      if (skipNulls || checkNulls) {
        for (Map.Entry<? extends K, ? extends V> e : map.entrySet()) {
          K k = e.getKey();
          V v = e.getValue();
          if (skipNulls) {
            if (k == null || v == null) continue;
          } else if (checkNulls) {
            Objects.requireNonNull(k, "key");
            Objects.requireNonNull(v, "value");
          }
          linkedMap.put(k, v);
        }
      } else {
        linkedMap.putAll(map);
      }
      return Collections.unmodifiableMap(linkedMap);
    }
    }
  }
}
