/**
 * Copyright (C) 2016 Red Hat, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package io.syndesis.model.connection;

import com.fasterxml.jackson.annotation.JsonProperty;
import edu.umd.cs.findbugs.annotations.SuppressFBWarnings;
import io.syndesis.model.WithTags;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.NavigableSet;
import java.util.Objects;
import java.util.Set;
import java.util.SortedSet;
import java.util.TreeSet;
import javax.annotation.Generated;
import javax.validation.ConstraintViolation;
import javax.validation.ConstraintViolationException;
import javax.validation.Validation;
import javax.validation.Validator;

/**
 * Immutable implementation of {@link ConfigurationProperty}.
 * <p>
 * Use the builder to create immutable instances:
 * {@code new ConfigurationProperty.Builder()}.
 * Use the static factory method to create immutable instances:
 * {@code ImmutableConfigurationProperty.of()}.
 */
@SuppressWarnings({"all"})
@SuppressFBWarnings
@Generated({"Immutables.generator", "ConfigurationProperty"})
final class ImmutableConfigurationProperty
    implements ConfigurationProperty {
  private final Boolean componentProperty;
  private final String defaultValue;
  private final Boolean deprecated;
  private final String description;
  private final String displayName;
  private final List<ConfigurationProperty.PropertyValue> getEnum;
  private final String group;
  private final String javaType;
  private final String kind;
  private final String label;
  private final Boolean required;
  private final Boolean secret;
  private final String type;
  private final SortedSet<String> tags;

  private ImmutableConfigurationProperty(
      Boolean componentProperty,
      String defaultValue,
      Boolean deprecated,
      String description,
      String displayName,
      Iterable<? extends ConfigurationProperty.PropertyValue> getEnum,
      String group,
      String javaType,
      String kind,
      String label,
      Boolean required,
      Boolean secret,
      String type,
      Iterable<String> tags) {
    this.componentProperty = componentProperty;
    this.defaultValue = defaultValue;
    this.deprecated = deprecated;
    this.description = description;
    this.displayName = displayName;
    this.getEnum = createUnmodifiableList(false, createSafeList(getEnum, true, false));
    this.group = group;
    this.javaType = javaType;
    this.kind = kind;
    this.label = label;
    this.required = required;
    this.secret = secret;
    this.type = type;
    this.tags = createUnmodifiableSortedSet(false, createSafeList(tags, false, true));
  }

  private ImmutableConfigurationProperty(
      ImmutableConfigurationProperty original,
      Boolean componentProperty,
      String defaultValue,
      Boolean deprecated,
      String description,
      String displayName,
      List<ConfigurationProperty.PropertyValue> getEnum,
      String group,
      String javaType,
      String kind,
      String label,
      Boolean required,
      Boolean secret,
      String type,
      SortedSet<String> tags) {
    this.componentProperty = componentProperty;
    this.defaultValue = defaultValue;
    this.deprecated = deprecated;
    this.description = description;
    this.displayName = displayName;
    this.getEnum = getEnum;
    this.group = group;
    this.javaType = javaType;
    this.kind = kind;
    this.label = label;
    this.required = required;
    this.secret = secret;
    this.type = type;
    this.tags = tags;
  }

  /**
   * @return The value of the {@code componentProperty} attribute
   */
  @JsonProperty("componentProperty")
  @Override
  public Boolean getComponentProperty() {
    return componentProperty;
  }

  /**
   * @return The value of the {@code defaultValue} attribute
   */
  @JsonProperty("defaultValue")
  @Override
  public String getDefaultValue() {
    return defaultValue;
  }

  /**
   * @return The value of the {@code deprecated} attribute
   */
  @JsonProperty("deprecated")
  @Override
  public Boolean getDeprecated() {
    return deprecated;
  }

  /**
   * @return The value of the {@code description} attribute
   */
  @JsonProperty("description")
  @Override
  public String getDescription() {
    return description;
  }

  /**
   * @return The value of the {@code displayName} attribute
   */
  @JsonProperty("displayName")
  @Override
  public String getDisplayName() {
    return displayName;
  }

  /**
   * @return The value of the {@code getEnum} attribute
   */
  @JsonProperty("enum")
  @Override
  public List<ConfigurationProperty.PropertyValue> getEnum() {
    return getEnum;
  }

  /**
   * @return The value of the {@code group} attribute
   */
  @JsonProperty("group")
  @Override
  public String getGroup() {
    return group;
  }

  /**
   * @return The value of the {@code javaType} attribute
   */
  @JsonProperty("javaType")
  @Override
  public String getJavaType() {
    return javaType;
  }

  /**
   * @return The value of the {@code kind} attribute
   */
  @JsonProperty("kind")
  @Override
  public String getKind() {
    return kind;
  }

  /**
   * @return The value of the {@code label} attribute
   */
  @JsonProperty("label")
  @Override
  public String getLabel() {
    return label;
  }

  /**
   * @return The value of the {@code required} attribute
   */
  @JsonProperty("required")
  @Override
  public Boolean getRequired() {
    return required;
  }

  /**
   * @return The value of the {@code secret} attribute
   */
  @JsonProperty("secret")
  @Override
  public Boolean getSecret() {
    return secret;
  }

  /**
   * @return The value of the {@code type} attribute
   */
  @JsonProperty("type")
  @Override
  public String getType() {
    return type;
  }

  /**
   * @return The value of the {@code tags} attribute
   */
  @JsonProperty("tags")
  @Override
  public SortedSet<String> getTags() {
    return tags;
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ConfigurationProperty#getComponentProperty() componentProperty} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for componentProperty (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableConfigurationProperty withComponentProperty(Boolean value) {
    if (Objects.equals(this.componentProperty, value)) return this;
    return validate(new ImmutableConfigurationProperty(
        this,
        value,
        this.defaultValue,
        this.deprecated,
        this.description,
        this.displayName,
        this.getEnum,
        this.group,
        this.javaType,
        this.kind,
        this.label,
        this.required,
        this.secret,
        this.type,
        this.tags));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ConfigurationProperty#getDefaultValue() defaultValue} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for defaultValue (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableConfigurationProperty withDefaultValue(String value) {
    if (Objects.equals(this.defaultValue, value)) return this;
    return validate(new ImmutableConfigurationProperty(
        this,
        this.componentProperty,
        value,
        this.deprecated,
        this.description,
        this.displayName,
        this.getEnum,
        this.group,
        this.javaType,
        this.kind,
        this.label,
        this.required,
        this.secret,
        this.type,
        this.tags));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ConfigurationProperty#getDeprecated() deprecated} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for deprecated (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableConfigurationProperty withDeprecated(Boolean value) {
    if (Objects.equals(this.deprecated, value)) return this;
    return validate(new ImmutableConfigurationProperty(
        this,
        this.componentProperty,
        this.defaultValue,
        value,
        this.description,
        this.displayName,
        this.getEnum,
        this.group,
        this.javaType,
        this.kind,
        this.label,
        this.required,
        this.secret,
        this.type,
        this.tags));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ConfigurationProperty#getDescription() description} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for description (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableConfigurationProperty withDescription(String value) {
    if (Objects.equals(this.description, value)) return this;
    return validate(new ImmutableConfigurationProperty(
        this,
        this.componentProperty,
        this.defaultValue,
        this.deprecated,
        value,
        this.displayName,
        this.getEnum,
        this.group,
        this.javaType,
        this.kind,
        this.label,
        this.required,
        this.secret,
        this.type,
        this.tags));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ConfigurationProperty#getDisplayName() displayName} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for displayName (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableConfigurationProperty withDisplayName(String value) {
    if (Objects.equals(this.displayName, value)) return this;
    return validate(new ImmutableConfigurationProperty(
        this,
        this.componentProperty,
        this.defaultValue,
        this.deprecated,
        this.description,
        value,
        this.getEnum,
        this.group,
        this.javaType,
        this.kind,
        this.label,
        this.required,
        this.secret,
        this.type,
        this.tags));
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link ConfigurationProperty#getEnum() enum}.
   * @param elements The elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableConfigurationProperty withEnum(ConfigurationProperty.PropertyValue... elements) {
    List<ConfigurationProperty.PropertyValue> newValue = createUnmodifiableList(false, createSafeList(Arrays.asList(elements), true, false));
    return validate(new ImmutableConfigurationProperty(
        this,
        this.componentProperty,
        this.defaultValue,
        this.deprecated,
        this.description,
        this.displayName,
        newValue,
        this.group,
        this.javaType,
        this.kind,
        this.label,
        this.required,
        this.secret,
        this.type,
        this.tags));
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link ConfigurationProperty#getEnum() enum}.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param elements An iterable of getEnum elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableConfigurationProperty withEnum(Iterable<? extends ConfigurationProperty.PropertyValue> elements) {
    if (this.getEnum == elements) return this;
    List<ConfigurationProperty.PropertyValue> newValue = createUnmodifiableList(false, createSafeList(elements, true, false));
    return validate(new ImmutableConfigurationProperty(
        this,
        this.componentProperty,
        this.defaultValue,
        this.deprecated,
        this.description,
        this.displayName,
        newValue,
        this.group,
        this.javaType,
        this.kind,
        this.label,
        this.required,
        this.secret,
        this.type,
        this.tags));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ConfigurationProperty#getGroup() group} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for group (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableConfigurationProperty withGroup(String value) {
    if (Objects.equals(this.group, value)) return this;
    return validate(new ImmutableConfigurationProperty(
        this,
        this.componentProperty,
        this.defaultValue,
        this.deprecated,
        this.description,
        this.displayName,
        this.getEnum,
        value,
        this.javaType,
        this.kind,
        this.label,
        this.required,
        this.secret,
        this.type,
        this.tags));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ConfigurationProperty#getJavaType() javaType} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for javaType (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableConfigurationProperty withJavaType(String value) {
    if (Objects.equals(this.javaType, value)) return this;
    return validate(new ImmutableConfigurationProperty(
        this,
        this.componentProperty,
        this.defaultValue,
        this.deprecated,
        this.description,
        this.displayName,
        this.getEnum,
        this.group,
        value,
        this.kind,
        this.label,
        this.required,
        this.secret,
        this.type,
        this.tags));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ConfigurationProperty#getKind() kind} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for kind (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableConfigurationProperty withKind(String value) {
    if (Objects.equals(this.kind, value)) return this;
    return validate(new ImmutableConfigurationProperty(
        this,
        this.componentProperty,
        this.defaultValue,
        this.deprecated,
        this.description,
        this.displayName,
        this.getEnum,
        this.group,
        this.javaType,
        value,
        this.label,
        this.required,
        this.secret,
        this.type,
        this.tags));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ConfigurationProperty#getLabel() label} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for label (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableConfigurationProperty withLabel(String value) {
    if (Objects.equals(this.label, value)) return this;
    return validate(new ImmutableConfigurationProperty(
        this,
        this.componentProperty,
        this.defaultValue,
        this.deprecated,
        this.description,
        this.displayName,
        this.getEnum,
        this.group,
        this.javaType,
        this.kind,
        value,
        this.required,
        this.secret,
        this.type,
        this.tags));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ConfigurationProperty#getRequired() required} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for required (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableConfigurationProperty withRequired(Boolean value) {
    if (Objects.equals(this.required, value)) return this;
    return validate(new ImmutableConfigurationProperty(
        this,
        this.componentProperty,
        this.defaultValue,
        this.deprecated,
        this.description,
        this.displayName,
        this.getEnum,
        this.group,
        this.javaType,
        this.kind,
        this.label,
        value,
        this.secret,
        this.type,
        this.tags));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ConfigurationProperty#getSecret() secret} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for secret (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableConfigurationProperty withSecret(Boolean value) {
    if (Objects.equals(this.secret, value)) return this;
    return validate(new ImmutableConfigurationProperty(
        this,
        this.componentProperty,
        this.defaultValue,
        this.deprecated,
        this.description,
        this.displayName,
        this.getEnum,
        this.group,
        this.javaType,
        this.kind,
        this.label,
        this.required,
        value,
        this.type,
        this.tags));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link ConfigurationProperty#getType() type} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for type (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableConfigurationProperty withType(String value) {
    if (Objects.equals(this.type, value)) return this;
    return validate(new ImmutableConfigurationProperty(
        this,
        this.componentProperty,
        this.defaultValue,
        this.deprecated,
        this.description,
        this.displayName,
        this.getEnum,
        this.group,
        this.javaType,
        this.kind,
        this.label,
        this.required,
        this.secret,
        value,
        this.tags));
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link ConfigurationProperty#getTags() tags}.
   * @param elements The elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableConfigurationProperty withTags(String... elements) {
    SortedSet<String> newValue = createUnmodifiableSortedSet(false, createSafeList(Arrays.asList(elements), false, true));
    return validate(new ImmutableConfigurationProperty(
        this,
        this.componentProperty,
        this.defaultValue,
        this.deprecated,
        this.description,
        this.displayName,
        this.getEnum,
        this.group,
        this.javaType,
        this.kind,
        this.label,
        this.required,
        this.secret,
        this.type,
        newValue));
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link ConfigurationProperty#getTags() tags}.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param elements An iterable of tags elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableConfigurationProperty withTags(Iterable<String> elements) {
    if (this.tags == elements) return this;
    SortedSet<String> newValue = createUnmodifiableSortedSet(false, createSafeList(elements, false, true));
    return validate(new ImmutableConfigurationProperty(
        this,
        this.componentProperty,
        this.defaultValue,
        this.deprecated,
        this.description,
        this.displayName,
        this.getEnum,
        this.group,
        this.javaType,
        this.kind,
        this.label,
        this.required,
        this.secret,
        this.type,
        newValue));
  }

  /**
   * This instance is equal to all instances of {@code ImmutableConfigurationProperty} that have equal attribute values.
   * @return {@code true} if {@code this} is equal to {@code another} instance
   */
  @Override
  public boolean equals(Object another) {
    if (this == another) return true;
    return another instanceof ImmutableConfigurationProperty
        && equalTo((ImmutableConfigurationProperty) another);
  }

  private boolean equalTo(ImmutableConfigurationProperty another) {
    return Objects.equals(componentProperty, another.componentProperty)
        && Objects.equals(defaultValue, another.defaultValue)
        && Objects.equals(deprecated, another.deprecated)
        && Objects.equals(description, another.description)
        && Objects.equals(displayName, another.displayName)
        && getEnum.equals(another.getEnum)
        && Objects.equals(group, another.group)
        && Objects.equals(javaType, another.javaType)
        && Objects.equals(kind, another.kind)
        && Objects.equals(label, another.label)
        && Objects.equals(required, another.required)
        && Objects.equals(secret, another.secret)
        && Objects.equals(type, another.type)
        && tags.equals(another.tags);
  }

  /**
   * Computes a hash code from attributes: {@code componentProperty}, {@code defaultValue}, {@code deprecated}, {@code description}, {@code displayName}, {@code getEnum}, {@code group}, {@code javaType}, {@code kind}, {@code label}, {@code required}, {@code secret}, {@code type}, {@code tags}.
   * @return hashCode value
   */
  @Override
  public int hashCode() {
    int h = 5381;
    h += (h << 5) + Objects.hashCode(componentProperty);
    h += (h << 5) + Objects.hashCode(defaultValue);
    h += (h << 5) + Objects.hashCode(deprecated);
    h += (h << 5) + Objects.hashCode(description);
    h += (h << 5) + Objects.hashCode(displayName);
    h += (h << 5) + getEnum.hashCode();
    h += (h << 5) + Objects.hashCode(group);
    h += (h << 5) + Objects.hashCode(javaType);
    h += (h << 5) + Objects.hashCode(kind);
    h += (h << 5) + Objects.hashCode(label);
    h += (h << 5) + Objects.hashCode(required);
    h += (h << 5) + Objects.hashCode(secret);
    h += (h << 5) + Objects.hashCode(type);
    h += (h << 5) + tags.hashCode();
    return h;
  }

  /**
   * Prints the immutable value {@code ConfigurationProperty} with attribute values.
   * @return A string representation of the value
   */
  @Override
  public String toString() {
    return "ConfigurationProperty{"
        + "componentProperty=" + componentProperty
        + ", defaultValue=" + defaultValue
        + ", deprecated=" + deprecated
        + ", description=" + description
        + ", displayName=" + displayName
        + ", enum=" + getEnum
        + ", group=" + group
        + ", javaType=" + javaType
        + ", kind=" + kind
        + ", label=" + label
        + ", required=" + required
        + ", secret=" + secret
        + ", type=" + type
        + ", tags=" + tags
        + "}";
  }

  /**
   * Construct a new immutable {@code ConfigurationProperty} instance.
   * @param componentProperty The value for the {@code componentProperty} attribute
   * @param defaultValue The value for the {@code defaultValue} attribute
   * @param deprecated The value for the {@code deprecated} attribute
   * @param description The value for the {@code description} attribute
   * @param displayName The value for the {@code displayName} attribute
   * @param getEnum The value for the {@code getEnum} attribute
   * @param group The value for the {@code group} attribute
   * @param javaType The value for the {@code javaType} attribute
   * @param kind The value for the {@code kind} attribute
   * @param label The value for the {@code label} attribute
   * @param required The value for the {@code required} attribute
   * @param secret The value for the {@code secret} attribute
   * @param type The value for the {@code type} attribute
   * @param tags The value for the {@code tags} attribute
   * @return An immutable ConfigurationProperty instance
   */
  public static ImmutableConfigurationProperty of(Boolean componentProperty, String defaultValue, Boolean deprecated, String description, String displayName, List<ConfigurationProperty.PropertyValue> getEnum, String group, String javaType, String kind, String label, Boolean required, Boolean secret, String type, SortedSet<String> tags) {
    return of(componentProperty, defaultValue, deprecated, description, displayName, (Iterable<? extends ConfigurationProperty.PropertyValue>) getEnum, group, javaType, kind, label, required, secret, type, (Iterable<String>) tags);
  }

  /**
   * Construct a new immutable {@code ConfigurationProperty} instance.
   * @param componentProperty The value for the {@code componentProperty} attribute
   * @param defaultValue The value for the {@code defaultValue} attribute
   * @param deprecated The value for the {@code deprecated} attribute
   * @param description The value for the {@code description} attribute
   * @param displayName The value for the {@code displayName} attribute
   * @param getEnum The value for the {@code getEnum} attribute
   * @param group The value for the {@code group} attribute
   * @param javaType The value for the {@code javaType} attribute
   * @param kind The value for the {@code kind} attribute
   * @param label The value for the {@code label} attribute
   * @param required The value for the {@code required} attribute
   * @param secret The value for the {@code secret} attribute
   * @param type The value for the {@code type} attribute
   * @param tags The value for the {@code tags} attribute
   * @return An immutable ConfigurationProperty instance
   */
  public static ImmutableConfigurationProperty of(Boolean componentProperty, String defaultValue, Boolean deprecated, String description, String displayName, Iterable<? extends ConfigurationProperty.PropertyValue> getEnum, String group, String javaType, String kind, String label, Boolean required, Boolean secret, String type, Iterable<String> tags) {
    return validate(new ImmutableConfigurationProperty(componentProperty, defaultValue, deprecated, description, displayName, getEnum, group, javaType, kind, label, required, secret, type, tags));
  }

  private static final Validator validator = Validation.buildDefaultValidatorFactory().getValidator();

  private static ImmutableConfigurationProperty validate(ImmutableConfigurationProperty instance) {
    Set<ConstraintViolation<ImmutableConfigurationProperty>> constraintViolations = validator.validate(instance);
    if (!constraintViolations.isEmpty()) {
      throw new ConstraintViolationException(constraintViolations);
    }
    return instance;
  }

  /**
   * Creates an immutable copy of a {@link ConfigurationProperty} value.
   * Uses accessors to get values to initialize the new immutable instance.
   * If an instance is already immutable, it is returned as is.
   * @param instance The instance to copy
   * @return A copied immutable ConfigurationProperty instance
   */
  public static ImmutableConfigurationProperty copyOf(ConfigurationProperty instance) {
    if (instance instanceof ImmutableConfigurationProperty) {
      return (ImmutableConfigurationProperty) instance;
    }
    return new ConfigurationProperty.Builder()
        .createFrom(instance)
        .build();
  }

  /**
   * Builds instances of type {@link ImmutableConfigurationProperty ImmutableConfigurationProperty}.
   * Initialize attributes and then invoke the {@link #build()} method to create an
   * immutable instance.
   * <p><em>{@code Builder} is not thread-safe and generally should not be stored in a field or collection,
   * but instead used immediately to create instances.</em>
   */
  static class Builder {
    private Boolean componentProperty;
    private String defaultValue;
    private Boolean deprecated;
    private String description;
    private String displayName;
    private List<ConfigurationProperty.PropertyValue> getEnum = new ArrayList<ConfigurationProperty.PropertyValue>();
    private String group;
    private String javaType;
    private String kind;
    private String label;
    private Boolean required;
    private Boolean secret;
    private String type;
    private List<String> tags = new ArrayList<String>();

    /**
     * Creates a builder for {@link ImmutableConfigurationProperty ImmutableConfigurationProperty} instances.
     */
    Builder() {
      if (!(this instanceof ConfigurationProperty.Builder)) {
        throw new UnsupportedOperationException("Use: new ConfigurationProperty.Builder()");
      }
    }

    /**
     * Fill a builder with attribute values from the provided {@code io.syndesis.model.connection.ConfigurationProperty} instance.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    public final ConfigurationProperty.Builder createFrom(ConfigurationProperty instance) {
      Objects.requireNonNull(instance, "instance");
      from((Object) instance);
      return (ConfigurationProperty.Builder) this;
    }

    /**
     * Fill a builder with attribute values from the provided {@code io.syndesis.model.WithTags} instance.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    public final ConfigurationProperty.Builder createFrom(WithTags instance) {
      Objects.requireNonNull(instance, "instance");
      from((Object) instance);
      return (ConfigurationProperty.Builder) this;
    }

    private void from(Object object) {
      if (object instanceof ConfigurationProperty) {
        ConfigurationProperty instance = (ConfigurationProperty) object;
        String defaultValueValue = instance.getDefaultValue();
        if (defaultValueValue != null) {
          defaultValue(defaultValueValue);
        }
        String displayNameValue = instance.getDisplayName();
        if (displayNameValue != null) {
          displayName(displayNameValue);
        }
        String kindValue = instance.getKind();
        if (kindValue != null) {
          kind(kindValue);
        }
        Boolean deprecatedValue = instance.getDeprecated();
        if (deprecatedValue != null) {
          deprecated(deprecatedValue);
        }
        String descriptionValue = instance.getDescription();
        if (descriptionValue != null) {
          description(descriptionValue);
        }
        addAllEnum(instance.getEnum());
        String labelValue = instance.getLabel();
        if (labelValue != null) {
          label(labelValue);
        }
        Boolean secretValue = instance.getSecret();
        if (secretValue != null) {
          secret(secretValue);
        }
        String typeValue = instance.getType();
        if (typeValue != null) {
          type(typeValue);
        }
        Boolean requiredValue = instance.getRequired();
        if (requiredValue != null) {
          required(requiredValue);
        }
        Boolean componentPropertyValue = instance.getComponentProperty();
        if (componentPropertyValue != null) {
          componentProperty(componentPropertyValue);
        }
        String javaTypeValue = instance.getJavaType();
        if (javaTypeValue != null) {
          javaType(javaTypeValue);
        }
        String groupValue = instance.getGroup();
        if (groupValue != null) {
          group(groupValue);
        }
      }
      if (object instanceof WithTags) {
        WithTags instance = (WithTags) object;
        addAllTags(instance.getTags());
      }
    }

    /**
     * Initializes the value for the {@link ConfigurationProperty#getComponentProperty() componentProperty} attribute.
     * @param componentProperty The value for componentProperty (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("componentProperty")
    public final ConfigurationProperty.Builder componentProperty(Boolean componentProperty) {
      this.componentProperty = componentProperty;
      return (ConfigurationProperty.Builder) this;
    }

    /**
     * Initializes the value for the {@link ConfigurationProperty#getDefaultValue() defaultValue} attribute.
     * @param defaultValue The value for defaultValue (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("defaultValue")
    public final ConfigurationProperty.Builder defaultValue(String defaultValue) {
      this.defaultValue = defaultValue;
      return (ConfigurationProperty.Builder) this;
    }

    /**
     * Initializes the value for the {@link ConfigurationProperty#getDeprecated() deprecated} attribute.
     * @param deprecated The value for deprecated (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("deprecated")
    public final ConfigurationProperty.Builder deprecated(Boolean deprecated) {
      this.deprecated = deprecated;
      return (ConfigurationProperty.Builder) this;
    }

    /**
     * Initializes the value for the {@link ConfigurationProperty#getDescription() description} attribute.
     * @param description The value for description (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("description")
    public final ConfigurationProperty.Builder description(String description) {
      this.description = description;
      return (ConfigurationProperty.Builder) this;
    }

    /**
     * Initializes the value for the {@link ConfigurationProperty#getDisplayName() displayName} attribute.
     * @param displayName The value for displayName (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("displayName")
    public final ConfigurationProperty.Builder displayName(String displayName) {
      this.displayName = displayName;
      return (ConfigurationProperty.Builder) this;
    }

    /**
     * Adds one element to {@link ConfigurationProperty#getEnum() enum} list.
     * @param element A getEnum element
     * @return {@code this} builder for use in a chained invocation
     */
    public final ConfigurationProperty.Builder addEnum(ConfigurationProperty.PropertyValue element) {
      this.getEnum.add(Objects.requireNonNull(element, "getEnum element"));
      return (ConfigurationProperty.Builder) this;
    }

    /**
     * Adds elements to {@link ConfigurationProperty#getEnum() enum} list.
     * @param elements An array of getEnum elements
     * @return {@code this} builder for use in a chained invocation
     */
    public final ConfigurationProperty.Builder addEnum(ConfigurationProperty.PropertyValue... elements) {
      for (ConfigurationProperty.PropertyValue element : elements) {
        this.getEnum.add(Objects.requireNonNull(element, "getEnum element"));
      }
      return (ConfigurationProperty.Builder) this;
    }

    /**
     * Sets or replaces all elements for {@link ConfigurationProperty#getEnum() enum} list.
     * @param elements An iterable of getEnum elements
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("enum")
    public final ConfigurationProperty.Builder getEnum(Iterable<? extends ConfigurationProperty.PropertyValue> elements) {
      this.getEnum.clear();
      return addAllEnum(elements);
    }

    /**
     * Adds elements to {@link ConfigurationProperty#getEnum() enum} list.
     * @param elements An iterable of getEnum elements
     * @return {@code this} builder for use in a chained invocation
     */
    public final ConfigurationProperty.Builder addAllEnum(Iterable<? extends ConfigurationProperty.PropertyValue> elements) {
      for (ConfigurationProperty.PropertyValue element : elements) {
        this.getEnum.add(Objects.requireNonNull(element, "getEnum element"));
      }
      return (ConfigurationProperty.Builder) this;
    }

    /**
     * Initializes the value for the {@link ConfigurationProperty#getGroup() group} attribute.
     * @param group The value for group (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("group")
    public final ConfigurationProperty.Builder group(String group) {
      this.group = group;
      return (ConfigurationProperty.Builder) this;
    }

    /**
     * Initializes the value for the {@link ConfigurationProperty#getJavaType() javaType} attribute.
     * @param javaType The value for javaType (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("javaType")
    public final ConfigurationProperty.Builder javaType(String javaType) {
      this.javaType = javaType;
      return (ConfigurationProperty.Builder) this;
    }

    /**
     * Initializes the value for the {@link ConfigurationProperty#getKind() kind} attribute.
     * @param kind The value for kind (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("kind")
    public final ConfigurationProperty.Builder kind(String kind) {
      this.kind = kind;
      return (ConfigurationProperty.Builder) this;
    }

    /**
     * Initializes the value for the {@link ConfigurationProperty#getLabel() label} attribute.
     * @param label The value for label (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("label")
    public final ConfigurationProperty.Builder label(String label) {
      this.label = label;
      return (ConfigurationProperty.Builder) this;
    }

    /**
     * Initializes the value for the {@link ConfigurationProperty#getRequired() required} attribute.
     * @param required The value for required (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("required")
    public final ConfigurationProperty.Builder required(Boolean required) {
      this.required = required;
      return (ConfigurationProperty.Builder) this;
    }

    /**
     * Initializes the value for the {@link ConfigurationProperty#getSecret() secret} attribute.
     * @param secret The value for secret (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("secret")
    public final ConfigurationProperty.Builder secret(Boolean secret) {
      this.secret = secret;
      return (ConfigurationProperty.Builder) this;
    }

    /**
     * Initializes the value for the {@link ConfigurationProperty#getType() type} attribute.
     * @param type The value for type (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("type")
    public final ConfigurationProperty.Builder type(String type) {
      this.type = type;
      return (ConfigurationProperty.Builder) this;
    }

    /**
     * Adds one element to {@link ConfigurationProperty#getTags() tags} sortedSet.
     * @param element A tags element
     * @return {@code this} builder for use in a chained invocation
     */
    public final ConfigurationProperty.Builder addTag(String element) {
      if (element != null) this.tags.add(element);
      return (ConfigurationProperty.Builder) this;
    }

    /**
     * Adds elements to {@link ConfigurationProperty#getTags() tags} sortedSet.
     * @param elements An array of tags elements
     * @return {@code this} builder for use in a chained invocation
     */
    public final ConfigurationProperty.Builder addTag(String... elements) {
      for (String element : elements) {
        if (element != null) this.tags.add(element);
      }
      return (ConfigurationProperty.Builder) this;
    }

    /**
     * Sets or replaces all elements for {@link ConfigurationProperty#getTags() tags} sortedSet.
     * @param elements An iterable of tags elements
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("tags")
    public final ConfigurationProperty.Builder tags(Iterable<String> elements) {
      this.tags.clear();
      return addAllTags(elements);
    }

    /**
     * Adds elements to {@link ConfigurationProperty#getTags() tags} sortedSet.
     * @param elements An iterable of tags elements
     * @return {@code this} builder for use in a chained invocation
     */
    public final ConfigurationProperty.Builder addAllTags(Iterable<String> elements) {
      for (String element : elements) {
        if (element != null) this.tags.add(element);
      }
      return (ConfigurationProperty.Builder) this;
    }

    /**
     * Builds a new {@link ImmutableConfigurationProperty ImmutableConfigurationProperty}.
     * @return An immutable instance of ConfigurationProperty
     * @throws java.lang.IllegalStateException if any required attributes are missing
     */
    public ImmutableConfigurationProperty build() {
      return ImmutableConfigurationProperty.validate(new ImmutableConfigurationProperty(
          null,
          componentProperty,
          defaultValue,
          deprecated,
          description,
          displayName,
          createUnmodifiableList(true, getEnum),
          group,
          javaType,
          kind,
          label,
          required,
          secret,
          type,
          createUnmodifiableSortedSet(false, createSafeList(tags, false, false))));
    }
  }

  private static <T> List<T> createSafeList(Iterable<? extends T> iterable, boolean checkNulls, boolean skipNulls) {
    ArrayList<T> list;
    if (iterable instanceof Collection<?>) {
      int size = ((Collection<?>) iterable).size();
      if (size == 0) return Collections.emptyList();
      list = new ArrayList<T>();
    } else {
      list = new ArrayList<T>();
    }
    for (T element : iterable) {
      if (skipNulls && element == null) continue;
      if (checkNulls) Objects.requireNonNull(element, "element");
      list.add(element);
    }
    return list;
  }

  private static <T> List<T> createUnmodifiableList(boolean clone, List<T> list) {
    switch(list.size()) {
    case 0: return Collections.emptyList();
    case 1: return Collections.singletonList(list.get(0));
    default:
      if (clone) {
        return Collections.unmodifiableList(new ArrayList<T>(list));
      } else {
        if (list instanceof ArrayList<?>) {
          ((ArrayList<?>) list).trimToSize();
        }
        return Collections.unmodifiableList(list);
      }
    }
  }

  private static <T extends Comparable<T>> NavigableSet<T> createUnmodifiableSortedSet(boolean reverse, List<T> list) {
    TreeSet<T> set = reverse
        ? new TreeSet<T>(Collections.reverseOrder())
        : new TreeSet<T>();
    set.addAll(list);
    return Collections.unmodifiableNavigableSet(set);
  }
}
