/**
 * Copyright (C) 2016 Red Hat, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package io.syndesis.model.connection;

import com.fasterxml.jackson.annotation.JsonProperty;
import edu.umd.cs.findbugs.annotations.SuppressFBWarnings;
import io.syndesis.model.Kind;
import io.syndesis.model.WithConfigurationProperties;
import io.syndesis.model.WithKind;
import io.syndesis.model.WithName;
import io.syndesis.model.WithProperties;
import java.io.ObjectStreamException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Optional;
import java.util.Set;
import javax.annotation.Generated;
import javax.validation.ConstraintViolation;
import javax.validation.ConstraintViolationException;
import javax.validation.Validation;
import javax.validation.Validator;

/**
 * Immutable implementation of {@link Connector}.
 * <p>
 * Use the builder to create immutable instances:
 * {@code new Connector.Builder()}.
 * Use the static factory method to create immutable instances:
 * {@code ImmutableConnector.of()}.
 */
@SuppressWarnings({"all"})
@SuppressFBWarnings
@Generated({"Immutables.generator", "Connector"})
final class ImmutableConnector implements Connector {
  private final Kind kind;
  private final ConnectorGroup connectorGroup;
  private final String connectorGroupId;
  private final String icon;
  private final Map<String, ConfigurationProperty> properties;
  private final String description;
  private final List<Action> actions;
  private final Map<String, String> configuredProperties;
  private final String id;
  private final String name;

  private ImmutableConnector(
      Kind kind,
      Optional<ConnectorGroup> connectorGroup,
      Optional<String> connectorGroupId,
      String icon,
      Map<String, ? extends ConfigurationProperty> properties,
      String description,
      Iterable<? extends Action> actions,
      Map<String, ? extends String> configuredProperties,
      Optional<String> id,
      String name) {
    this.kind = Objects.requireNonNull(kind, "kind");
    this.connectorGroup = connectorGroup.orElse(null);
    this.connectorGroupId = connectorGroupId.orElse(null);
    this.icon = icon;
    this.properties = createUnmodifiableMap(true, false, properties);
    this.description = description;
    this.actions = createUnmodifiableList(false, createSafeList(actions, true, false));
    this.configuredProperties = createUnmodifiableMap(true, false, configuredProperties);
    this.id = id.orElse(null);
    this.name = name;
  }

  private ImmutableConnector(ImmutableConnector.Builder builder) {
    this.connectorGroup = builder.connectorGroup;
    this.connectorGroupId = builder.connectorGroupId;
    this.icon = builder.icon;
    this.properties = createUnmodifiableMap(false, false, builder.properties);
    this.description = builder.description;
    this.actions = createUnmodifiableList(true, builder.actions);
    this.configuredProperties = createUnmodifiableMap(false, false, builder.configuredProperties);
    this.id = builder.id;
    this.name = builder.name;
    this.kind = builder.kind != null
        ? builder.kind
        : Objects.requireNonNull(Connector.super.getKind(), "kind");
  }

  private ImmutableConnector(
      ImmutableConnector original,
      Kind kind,
      ConnectorGroup connectorGroup,
      String connectorGroupId,
      String icon,
      Map<String, ConfigurationProperty> properties,
      String description,
      List<Action> actions,
      Map<String, String> configuredProperties,
      String id,
      String name) {
    this.kind = kind;
    this.connectorGroup = connectorGroup;
    this.connectorGroupId = connectorGroupId;
    this.icon = icon;
    this.properties = properties;
    this.description = description;
    this.actions = actions;
    this.configuredProperties = configuredProperties;
    this.id = id;
    this.name = name;
  }

  /**
   * @return The value of the {@code kind} attribute
   */
  @JsonProperty("kind")
  @Override
  public Kind getKind() {
    return kind;
  }

  /**
   * @return The value of the {@code connectorGroup} attribute
   */
  @JsonProperty("connectorGroup")
  @Override
  public Optional<ConnectorGroup> getConnectorGroup() {
    return Optional.ofNullable(connectorGroup);
  }

  /**
   * @return The value of the {@code connectorGroupId} attribute
   */
  @JsonProperty("connectorGroupId")
  @Override
  public Optional<String> getConnectorGroupId() {
    return Optional.ofNullable(connectorGroupId);
  }

  /**
   * @return The value of the {@code icon} attribute
   */
  @JsonProperty("icon")
  @Override
  public String getIcon() {
    return icon;
  }

  /**
   * @return The value of the {@code properties} attribute
   */
  @JsonProperty("properties")
  @Override
  public Map<String, ConfigurationProperty> getProperties() {
    return properties;
  }

  /**
   * @return The value of the {@code description} attribute
   */
  @JsonProperty("description")
  @Override
  public String getDescription() {
    return description;
  }

  /**
   * @return The value of the {@code actions} attribute
   */
  @JsonProperty("actions")
  @Override
  public List<Action> getActions() {
    return actions;
  }

  /**
   * @return The value of the {@code configuredProperties} attribute
   */
  @JsonProperty("configuredProperties")
  @Override
  public Map<String, String> getConfiguredProperties() {
    return configuredProperties;
  }

  /**
   * @return The value of the {@code id} attribute
   */
  @JsonProperty("id")
  @Override
  public Optional<String> getId() {
    return Optional.ofNullable(id);
  }

  /**
   * @return The value of the {@code name} attribute
   */
  @JsonProperty("name")
  @Override
  public String getName() {
    return name;
  }

  /**
   * Copy the current immutable object by setting a value for the {@link Connector#getKind() kind} attribute.
   * A value equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for kind
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableConnector withKind(Kind value) {
    if (this.kind == value) return this;
    Kind newValue = Objects.requireNonNull(value, "kind");
    return validate(new ImmutableConnector(
        this,
        newValue,
        this.connectorGroup,
        this.connectorGroupId,
        this.icon,
        this.properties,
        this.description,
        this.actions,
        this.configuredProperties,
        this.id,
        this.name));
  }

  /**
   * Copy the current immutable object by setting a <i>present</i> value for the optional {@link Connector#getConnectorGroup() connectorGroup} attribute.
   * @param value The value for connectorGroup
   * @return A modified copy of {@code this} object
   */
  public final ImmutableConnector withConnectorGroup(ConnectorGroup value) {
    ConnectorGroup newValue = Objects.requireNonNull(value, "connectorGroup");
    if (this.connectorGroup == newValue) return this;
    return validate(new ImmutableConnector(
        this,
        this.kind,
        newValue,
        this.connectorGroupId,
        this.icon,
        this.properties,
        this.description,
        this.actions,
        this.configuredProperties,
        this.id,
        this.name));
  }

  /**
   * Copy the current immutable object by setting an optional value for the {@link Connector#getConnectorGroup() connectorGroup} attribute.
   * A shallow reference equality check is used on unboxed optional value to prevent copying of the same value by returning {@code this}.
   * @param optional A value for connectorGroup
   * @return A modified copy of {@code this} object
   */
  public final ImmutableConnector withConnectorGroup(Optional<? extends ConnectorGroup> optional) {
    ConnectorGroup value = optional.orElse(null);
    if (this.connectorGroup == value) return this;
    return validate(new ImmutableConnector(
        this,
        this.kind,
        value,
        this.connectorGroupId,
        this.icon,
        this.properties,
        this.description,
        this.actions,
        this.configuredProperties,
        this.id,
        this.name));
  }

  /**
   * Copy the current immutable object by setting a <i>present</i> value for the optional {@link Connector#getConnectorGroupId() connectorGroupId} attribute.
   * @param value The value for connectorGroupId
   * @return A modified copy of {@code this} object
   */
  public final ImmutableConnector withConnectorGroupId(String value) {
    String newValue = Objects.requireNonNull(value, "connectorGroupId");
    if (Objects.equals(this.connectorGroupId, newValue)) return this;
    return validate(new ImmutableConnector(
        this,
        this.kind,
        this.connectorGroup,
        newValue,
        this.icon,
        this.properties,
        this.description,
        this.actions,
        this.configuredProperties,
        this.id,
        this.name));
  }

  /**
   * Copy the current immutable object by setting an optional value for the {@link Connector#getConnectorGroupId() connectorGroupId} attribute.
   * An equality check is used on inner nullable value to prevent copying of the same value by returning {@code this}.
   * @param optional A value for connectorGroupId
   * @return A modified copy of {@code this} object
   */
  public final ImmutableConnector withConnectorGroupId(Optional<String> optional) {
    String value = optional.orElse(null);
    if (Objects.equals(this.connectorGroupId, value)) return this;
    return validate(new ImmutableConnector(
        this,
        this.kind,
        this.connectorGroup,
        value,
        this.icon,
        this.properties,
        this.description,
        this.actions,
        this.configuredProperties,
        this.id,
        this.name));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link Connector#getIcon() icon} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for icon (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableConnector withIcon(String value) {
    if (Objects.equals(this.icon, value)) return this;
    return validate(new ImmutableConnector(
        this,
        this.kind,
        this.connectorGroup,
        this.connectorGroupId,
        value,
        this.properties,
        this.description,
        this.actions,
        this.configuredProperties,
        this.id,
        this.name));
  }

  /**
   * Copy the current immutable object by replacing the {@link Connector#getProperties() properties} map with the specified map.
   * Nulls are not permitted as keys or values.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param entries The entries to be added to the properties map
   * @return A modified copy of {@code this} object
   */
  public final ImmutableConnector withProperties(Map<String, ? extends ConfigurationProperty> entries) {
    if (this.properties == entries) return this;
    Map<String, ConfigurationProperty> newValue = createUnmodifiableMap(true, false, entries);
    return validate(new ImmutableConnector(
        this,
        this.kind,
        this.connectorGroup,
        this.connectorGroupId,
        this.icon,
        newValue,
        this.description,
        this.actions,
        this.configuredProperties,
        this.id,
        this.name));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link Connector#getDescription() description} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for description (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableConnector withDescription(String value) {
    if (Objects.equals(this.description, value)) return this;
    return validate(new ImmutableConnector(
        this,
        this.kind,
        this.connectorGroup,
        this.connectorGroupId,
        this.icon,
        this.properties,
        value,
        this.actions,
        this.configuredProperties,
        this.id,
        this.name));
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link Connector#getActions() actions}.
   * @param elements The elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableConnector withActions(Action... elements) {
    List<Action> newValue = createUnmodifiableList(false, createSafeList(Arrays.asList(elements), true, false));
    return validate(new ImmutableConnector(
        this,
        this.kind,
        this.connectorGroup,
        this.connectorGroupId,
        this.icon,
        this.properties,
        this.description,
        newValue,
        this.configuredProperties,
        this.id,
        this.name));
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link Connector#getActions() actions}.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param elements An iterable of actions elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableConnector withActions(Iterable<? extends Action> elements) {
    if (this.actions == elements) return this;
    List<Action> newValue = createUnmodifiableList(false, createSafeList(elements, true, false));
    return validate(new ImmutableConnector(
        this,
        this.kind,
        this.connectorGroup,
        this.connectorGroupId,
        this.icon,
        this.properties,
        this.description,
        newValue,
        this.configuredProperties,
        this.id,
        this.name));
  }

  /**
   * Copy the current immutable object by replacing the {@link Connector#getConfiguredProperties() configuredProperties} map with the specified map.
   * Nulls are not permitted as keys or values.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param entries The entries to be added to the configuredProperties map
   * @return A modified copy of {@code this} object
   */
  public final ImmutableConnector withConfiguredProperties(Map<String, ? extends String> entries) {
    if (this.configuredProperties == entries) return this;
    Map<String, String> newValue = createUnmodifiableMap(true, false, entries);
    return validate(new ImmutableConnector(
        this,
        this.kind,
        this.connectorGroup,
        this.connectorGroupId,
        this.icon,
        this.properties,
        this.description,
        this.actions,
        newValue,
        this.id,
        this.name));
  }

  /**
   * Copy the current immutable object by setting a <i>present</i> value for the optional {@link Connector#getId() id} attribute.
   * @param value The value for id
   * @return A modified copy of {@code this} object
   */
  public final ImmutableConnector withId(String value) {
    String newValue = Objects.requireNonNull(value, "id");
    if (Objects.equals(this.id, newValue)) return this;
    return validate(new ImmutableConnector(
        this,
        this.kind,
        this.connectorGroup,
        this.connectorGroupId,
        this.icon,
        this.properties,
        this.description,
        this.actions,
        this.configuredProperties,
        newValue,
        this.name));
  }

  /**
   * Copy the current immutable object by setting an optional value for the {@link Connector#getId() id} attribute.
   * An equality check is used on inner nullable value to prevent copying of the same value by returning {@code this}.
   * @param optional A value for id
   * @return A modified copy of {@code this} object
   */
  public final ImmutableConnector withId(Optional<String> optional) {
    String value = optional.orElse(null);
    if (Objects.equals(this.id, value)) return this;
    return validate(new ImmutableConnector(
        this,
        this.kind,
        this.connectorGroup,
        this.connectorGroupId,
        this.icon,
        this.properties,
        this.description,
        this.actions,
        this.configuredProperties,
        value,
        this.name));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link Connector#getName() name} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for name (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableConnector withName(String value) {
    if (Objects.equals(this.name, value)) return this;
    return validate(new ImmutableConnector(
        this,
        this.kind,
        this.connectorGroup,
        this.connectorGroupId,
        this.icon,
        this.properties,
        this.description,
        this.actions,
        this.configuredProperties,
        this.id,
        value));
  }

  /**
   * This instance is equal to all instances of {@code ImmutableConnector} that have equal attribute values.
   * @return {@code true} if {@code this} is equal to {@code another} instance
   */
  @Override
  public boolean equals(Object another) {
    if (this == another) return true;
    return another instanceof ImmutableConnector
        && equalTo((ImmutableConnector) another);
  }

  private boolean equalTo(ImmutableConnector another) {
    return kind.equals(another.kind)
        && Objects.equals(connectorGroup, another.connectorGroup)
        && Objects.equals(connectorGroupId, another.connectorGroupId)
        && Objects.equals(icon, another.icon)
        && properties.equals(another.properties)
        && Objects.equals(description, another.description)
        && actions.equals(another.actions)
        && configuredProperties.equals(another.configuredProperties)
        && Objects.equals(id, another.id)
        && Objects.equals(name, another.name);
  }

  /**
   * Computes a hash code from attributes: {@code kind}, {@code connectorGroup}, {@code connectorGroupId}, {@code icon}, {@code properties}, {@code description}, {@code actions}, {@code configuredProperties}, {@code id}, {@code name}.
   * @return hashCode value
   */
  @Override
  public int hashCode() {
    int h = 5381;
    h += (h << 5) + kind.hashCode();
    h += (h << 5) + Objects.hashCode(connectorGroup);
    h += (h << 5) + Objects.hashCode(connectorGroupId);
    h += (h << 5) + Objects.hashCode(icon);
    h += (h << 5) + properties.hashCode();
    h += (h << 5) + Objects.hashCode(description);
    h += (h << 5) + actions.hashCode();
    h += (h << 5) + configuredProperties.hashCode();
    h += (h << 5) + Objects.hashCode(id);
    h += (h << 5) + Objects.hashCode(name);
    return h;
  }

  /**
   * Prints the immutable value {@code Connector} with attribute values.
   * @return A string representation of the value
   */
  @Override
  public String toString() {
    StringBuilder builder = new StringBuilder("Connector{");
    builder.append("kind=").append(kind);
    if (connectorGroup != null) {
      builder.append(", ");
      builder.append("connectorGroup=").append(connectorGroup);
    }
    if (connectorGroupId != null) {
      builder.append(", ");
      builder.append("connectorGroupId=").append(connectorGroupId);
    }
    if (icon != null) {
      builder.append(", ");
      builder.append("icon=").append(icon);
    }
    builder.append(", ");
    builder.append("properties=").append(properties);
    if (description != null) {
      builder.append(", ");
      builder.append("description=").append(description);
    }
    builder.append(", ");
    builder.append("actions=").append(actions);
    builder.append(", ");
    builder.append("configuredProperties=").append(configuredProperties);
    if (id != null) {
      builder.append(", ");
      builder.append("id=").append(id);
    }
    if (name != null) {
      builder.append(", ");
      builder.append("name=").append(name);
    }
    return builder.append("}").toString();
  }

  /**
   * Construct a new immutable {@code Connector} instance.
   * @param kind The value for the {@code kind} attribute
   * @param connectorGroup The value for the {@code connectorGroup} attribute
   * @param connectorGroupId The value for the {@code connectorGroupId} attribute
   * @param icon The value for the {@code icon} attribute
   * @param properties The value for the {@code properties} attribute
   * @param description The value for the {@code description} attribute
   * @param actions The value for the {@code actions} attribute
   * @param configuredProperties The value for the {@code configuredProperties} attribute
   * @param id The value for the {@code id} attribute
   * @param name The value for the {@code name} attribute
   * @return An immutable Connector instance
   */
  public static ImmutableConnector of(Kind kind, Optional<ConnectorGroup> connectorGroup, Optional<String> connectorGroupId, String icon, Map<String, ConfigurationProperty> properties, String description, List<Action> actions, Map<String, String> configuredProperties, Optional<String> id, String name) {
    return of(kind, connectorGroup, connectorGroupId, icon, properties, description, (Iterable<? extends Action>) actions, configuredProperties, id, name);
  }

  /**
   * Construct a new immutable {@code Connector} instance.
   * @param kind The value for the {@code kind} attribute
   * @param connectorGroup The value for the {@code connectorGroup} attribute
   * @param connectorGroupId The value for the {@code connectorGroupId} attribute
   * @param icon The value for the {@code icon} attribute
   * @param properties The value for the {@code properties} attribute
   * @param description The value for the {@code description} attribute
   * @param actions The value for the {@code actions} attribute
   * @param configuredProperties The value for the {@code configuredProperties} attribute
   * @param id The value for the {@code id} attribute
   * @param name The value for the {@code name} attribute
   * @return An immutable Connector instance
   */
  public static ImmutableConnector of(Kind kind, Optional<ConnectorGroup> connectorGroup, Optional<String> connectorGroupId, String icon, Map<String, ? extends ConfigurationProperty> properties, String description, Iterable<? extends Action> actions, Map<String, ? extends String> configuredProperties, Optional<String> id, String name) {
    return validate(new ImmutableConnector(kind, connectorGroup, connectorGroupId, icon, properties, description, actions, configuredProperties, id, name));
  }

  private static final Validator validator = Validation.buildDefaultValidatorFactory().getValidator();

  private static ImmutableConnector validate(ImmutableConnector instance) {
    Set<ConstraintViolation<ImmutableConnector>> constraintViolations = validator.validate(instance);
    if (!constraintViolations.isEmpty()) {
      throw new ConstraintViolationException(constraintViolations);
    }
    return instance;
  }

  /**
   * Creates an immutable copy of a {@link Connector} value.
   * Uses accessors to get values to initialize the new immutable instance.
   * If an instance is already immutable, it is returned as is.
   * @param instance The instance to copy
   * @return A copied immutable Connector instance
   */
  public static ImmutableConnector copyOf(Connector instance) {
    if (instance instanceof ImmutableConnector) {
      return (ImmutableConnector) instance;
    }
    return new Connector.Builder()
        .createFrom(instance)
        .build();
  }

  private Object readResolve() throws ObjectStreamException {
    return validate(this);
  }

  /**
   * Builds instances of type {@link ImmutableConnector ImmutableConnector}.
   * Initialize attributes and then invoke the {@link #build()} method to create an
   * immutable instance.
   * <p><em>{@code Builder} is not thread-safe and generally should not be stored in a field or collection,
   * but instead used immediately to create instances.</em>
   */
  static class Builder {
    private Kind kind;
    private ConnectorGroup connectorGroup;
    private String connectorGroupId;
    private String icon;
    private Map<String, ConfigurationProperty> properties = new LinkedHashMap<String, ConfigurationProperty>();
    private String description;
    private List<Action> actions = new ArrayList<Action>();
    private Map<String, String> configuredProperties = new LinkedHashMap<String, String>();
    private String id;
    private String name;

    /**
     * Creates a builder for {@link ImmutableConnector ImmutableConnector} instances.
     */
    Builder() {
      if (!(this instanceof Connector.Builder)) {
        throw new UnsupportedOperationException("Use: new Connector.Builder()");
      }
    }

    /**
     * Fill a builder with attribute values from the provided {@code io.syndesis.model.WithKind} instance.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    public final Connector.Builder createFrom(WithKind instance) {
      Objects.requireNonNull(instance, "instance");
      from((Object) instance);
      return (Connector.Builder) this;
    }

    /**
     * Fill a builder with attribute values from the provided {@code io.syndesis.model.WithConfigurationProperties} instance.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    public final Connector.Builder createFrom(WithConfigurationProperties instance) {
      Objects.requireNonNull(instance, "instance");
      from((Object) instance);
      return (Connector.Builder) this;
    }

    /**
     * Fill a builder with attribute values from the provided {@code io.syndesis.model.WithProperties} instance.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    public final Connector.Builder createFrom(WithProperties instance) {
      Objects.requireNonNull(instance, "instance");
      from((Object) instance);
      return (Connector.Builder) this;
    }

    /**
     * Fill a builder with attribute values from the provided {@code io.syndesis.model.connection.Connector} instance.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    public final Connector.Builder createFrom(Connector instance) {
      Objects.requireNonNull(instance, "instance");
      from((Object) instance);
      return (Connector.Builder) this;
    }

    /**
     * Fill a builder with attribute values from the provided {@code io.syndesis.model.WithName} instance.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    public final Connector.Builder createFrom(WithName instance) {
      Objects.requireNonNull(instance, "instance");
      from((Object) instance);
      return (Connector.Builder) this;
    }

    private void from(Object object) {
      long bits = 0;
      if (object instanceof WithKind) {
        WithKind instance = (WithKind) object;
        if ((bits & 0x2L) == 0) {
          kind(instance.getKind());
          bits |= 0x2L;
        }
      }
      if (object instanceof WithConfigurationProperties) {
        WithConfigurationProperties instance = (WithConfigurationProperties) object;
        if ((bits & 0x4L) == 0) {
          putAllProperties(instance.getProperties());
          bits |= 0x4L;
        }
      }
      if (object instanceof WithProperties) {
        WithProperties instance = (WithProperties) object;
        if ((bits & 0x1L) == 0) {
          putAllConfiguredProperties(instance.getConfiguredProperties());
          bits |= 0x1L;
        }
      }
      if (object instanceof Connector) {
        Connector instance = (Connector) object;
        if ((bits & 0x1L) == 0) {
          putAllConfiguredProperties(instance.getConfiguredProperties());
          bits |= 0x1L;
        }
        Optional<ConnectorGroup> connectorGroupOptional = instance.getConnectorGroup();
        if (connectorGroupOptional.isPresent()) {
          connectorGroup(connectorGroupOptional);
        }
        if ((bits & 0x2L) == 0) {
          kind(instance.getKind());
          bits |= 0x2L;
        }
        Optional<String> connectorGroupIdOptional = instance.getConnectorGroupId();
        if (connectorGroupIdOptional.isPresent()) {
          connectorGroupId(connectorGroupIdOptional);
        }
        String iconValue = instance.getIcon();
        if (iconValue != null) {
          icon(iconValue);
        }
        String descriptionValue = instance.getDescription();
        if (descriptionValue != null) {
          description(descriptionValue);
        }
        Optional<String> idOptional = instance.getId();
        if (idOptional.isPresent()) {
          id(idOptional);
        }
        addAllActions(instance.getActions());
        if ((bits & 0x4L) == 0) {
          putAllProperties(instance.getProperties());
          bits |= 0x4L;
        }
      }
      if (object instanceof WithName) {
        WithName instance = (WithName) object;
        String nameValue = instance.getName();
        if (nameValue != null) {
          name(nameValue);
        }
      }
    }

    /**
     * Initializes the value for the {@link Connector#getKind() kind} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link Connector#getKind() kind}.</em>
     * @param kind The value for kind 
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("kind")
    public final Connector.Builder kind(Kind kind) {
      this.kind = Objects.requireNonNull(kind, "kind");
      return (Connector.Builder) this;
    }

    /**
     * Initializes the optional value {@link Connector#getConnectorGroup() connectorGroup} to connectorGroup.
     * @param connectorGroup The value for connectorGroup
     * @return {@code this} builder for chained invocation
     */
    public final Connector.Builder connectorGroup(ConnectorGroup connectorGroup) {
      this.connectorGroup = Objects.requireNonNull(connectorGroup, "connectorGroup");
      return (Connector.Builder) this;
    }

    /**
     * Initializes the optional value {@link Connector#getConnectorGroup() connectorGroup} to connectorGroup.
     * @param connectorGroup The value for connectorGroup
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("connectorGroup")
    public final Connector.Builder connectorGroup(Optional<? extends ConnectorGroup> connectorGroup) {
      this.connectorGroup = connectorGroup.orElse(null);
      return (Connector.Builder) this;
    }

    /**
     * Initializes the optional value {@link Connector#getConnectorGroupId() connectorGroupId} to connectorGroupId.
     * @param connectorGroupId The value for connectorGroupId
     * @return {@code this} builder for chained invocation
     */
    public final Connector.Builder connectorGroupId(String connectorGroupId) {
      this.connectorGroupId = Objects.requireNonNull(connectorGroupId, "connectorGroupId");
      return (Connector.Builder) this;
    }

    /**
     * Initializes the optional value {@link Connector#getConnectorGroupId() connectorGroupId} to connectorGroupId.
     * @param connectorGroupId The value for connectorGroupId
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("connectorGroupId")
    public final Connector.Builder connectorGroupId(Optional<String> connectorGroupId) {
      this.connectorGroupId = connectorGroupId.orElse(null);
      return (Connector.Builder) this;
    }

    /**
     * Initializes the value for the {@link Connector#getIcon() icon} attribute.
     * @param icon The value for icon (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("icon")
    public final Connector.Builder icon(String icon) {
      this.icon = icon;
      return (Connector.Builder) this;
    }

    /**
     * Put one entry to the {@link Connector#getProperties() properties} map.
     * @param key The key in the properties map
     * @param value The associated value in the properties map
     * @return {@code this} builder for use in a chained invocation
     */
    public final Connector.Builder putProperty(String key, ConfigurationProperty value) {
      this.properties.put(
          Objects.requireNonNull(key, "properties key"),
          Objects.requireNonNull(value, "properties value"));
      return (Connector.Builder) this;
    }

    /**
     * Put one entry to the {@link Connector#getProperties() properties} map. Nulls are not permitted
     * @param entry The key and value entry
     * @return {@code this} builder for use in a chained invocation
     */
    public final Connector.Builder putProperty(Map.Entry<String, ? extends ConfigurationProperty> entry) {
      String k = entry.getKey();
      ConfigurationProperty v = entry.getValue();
      this.properties.put(
          Objects.requireNonNull(k, "properties key"),
          Objects.requireNonNull(v, "properties value"));
      return (Connector.Builder) this;
    }

    /**
     * Sets or replaces all mappings from the specified map as entries for the {@link Connector#getProperties() properties} map. Nulls are not permitted
     * @param properties The entries that will be added to the properties map
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("properties")
    public final Connector.Builder properties(Map<String, ? extends ConfigurationProperty> properties) {
      this.properties.clear();
      return putAllProperties(properties);
    }

    /**
     * Put all mappings from the specified map as entries to {@link Connector#getProperties() properties} map. Nulls are not permitted
     * @param properties The entries that will be added to the properties map
     * @return {@code this} builder for use in a chained invocation
     */
    public final Connector.Builder putAllProperties(Map<String, ? extends ConfigurationProperty> properties) {
      for (Map.Entry<String, ? extends ConfigurationProperty> entry : properties.entrySet()) {
        String k = entry.getKey();
        ConfigurationProperty v = entry.getValue();
        this.properties.put(
            Objects.requireNonNull(k, "properties key"),
            Objects.requireNonNull(v, "properties value"));
      }
      return (Connector.Builder) this;
    }

    /**
     * Initializes the value for the {@link Connector#getDescription() description} attribute.
     * @param description The value for description (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("description")
    public final Connector.Builder description(String description) {
      this.description = description;
      return (Connector.Builder) this;
    }

    /**
     * Adds one element to {@link Connector#getActions() actions} list.
     * @param element A actions element
     * @return {@code this} builder for use in a chained invocation
     */
    public final Connector.Builder addAction(Action element) {
      this.actions.add(Objects.requireNonNull(element, "actions element"));
      return (Connector.Builder) this;
    }

    /**
     * Adds elements to {@link Connector#getActions() actions} list.
     * @param elements An array of actions elements
     * @return {@code this} builder for use in a chained invocation
     */
    public final Connector.Builder addAction(Action... elements) {
      for (Action element : elements) {
        this.actions.add(Objects.requireNonNull(element, "actions element"));
      }
      return (Connector.Builder) this;
    }

    /**
     * Sets or replaces all elements for {@link Connector#getActions() actions} list.
     * @param elements An iterable of actions elements
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("actions")
    public final Connector.Builder actions(Iterable<? extends Action> elements) {
      this.actions.clear();
      return addAllActions(elements);
    }

    /**
     * Adds elements to {@link Connector#getActions() actions} list.
     * @param elements An iterable of actions elements
     * @return {@code this} builder for use in a chained invocation
     */
    public final Connector.Builder addAllActions(Iterable<? extends Action> elements) {
      for (Action element : elements) {
        this.actions.add(Objects.requireNonNull(element, "actions element"));
      }
      return (Connector.Builder) this;
    }

    /**
     * Put one entry to the {@link Connector#getConfiguredProperties() configuredProperties} map.
     * @param key The key in the configuredProperties map
     * @param value The associated value in the configuredProperties map
     * @return {@code this} builder for use in a chained invocation
     */
    public final Connector.Builder putConfiguredProperty(String key, String value) {
      this.configuredProperties.put(
          Objects.requireNonNull(key, "configuredProperties key"),
          Objects.requireNonNull(value, "configuredProperties value"));
      return (Connector.Builder) this;
    }

    /**
     * Put one entry to the {@link Connector#getConfiguredProperties() configuredProperties} map. Nulls are not permitted
     * @param entry The key and value entry
     * @return {@code this} builder for use in a chained invocation
     */
    public final Connector.Builder putConfiguredProperty(Map.Entry<String, ? extends String> entry) {
      String k = entry.getKey();
      String v = entry.getValue();
      this.configuredProperties.put(
          Objects.requireNonNull(k, "configuredProperties key"),
          Objects.requireNonNull(v, "configuredProperties value"));
      return (Connector.Builder) this;
    }

    /**
     * Sets or replaces all mappings from the specified map as entries for the {@link Connector#getConfiguredProperties() configuredProperties} map. Nulls are not permitted
     * @param configuredProperties The entries that will be added to the configuredProperties map
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("configuredProperties")
    public final Connector.Builder configuredProperties(Map<String, ? extends String> configuredProperties) {
      this.configuredProperties.clear();
      return putAllConfiguredProperties(configuredProperties);
    }

    /**
     * Put all mappings from the specified map as entries to {@link Connector#getConfiguredProperties() configuredProperties} map. Nulls are not permitted
     * @param configuredProperties The entries that will be added to the configuredProperties map
     * @return {@code this} builder for use in a chained invocation
     */
    public final Connector.Builder putAllConfiguredProperties(Map<String, ? extends String> configuredProperties) {
      for (Map.Entry<String, ? extends String> entry : configuredProperties.entrySet()) {
        String k = entry.getKey();
        String v = entry.getValue();
        this.configuredProperties.put(
            Objects.requireNonNull(k, "configuredProperties key"),
            Objects.requireNonNull(v, "configuredProperties value"));
      }
      return (Connector.Builder) this;
    }

    /**
     * Initializes the optional value {@link Connector#getId() id} to id.
     * @param id The value for id
     * @return {@code this} builder for chained invocation
     */
    public final Connector.Builder id(String id) {
      this.id = Objects.requireNonNull(id, "id");
      return (Connector.Builder) this;
    }

    /**
     * Initializes the optional value {@link Connector#getId() id} to id.
     * @param id The value for id
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("id")
    public final Connector.Builder id(Optional<String> id) {
      this.id = id.orElse(null);
      return (Connector.Builder) this;
    }

    /**
     * Initializes the value for the {@link Connector#getName() name} attribute.
     * @param name The value for name (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("name")
    public final Connector.Builder name(String name) {
      this.name = name;
      return (Connector.Builder) this;
    }

    /**
     * Builds a new {@link ImmutableConnector ImmutableConnector}.
     * @return An immutable instance of Connector
     * @throws java.lang.IllegalStateException if any required attributes are missing
     */
    public ImmutableConnector build() {
      return ImmutableConnector.validate(new ImmutableConnector(this));
    }
  }

  private static <T> List<T> createSafeList(Iterable<? extends T> iterable, boolean checkNulls, boolean skipNulls) {
    ArrayList<T> list;
    if (iterable instanceof Collection<?>) {
      int size = ((Collection<?>) iterable).size();
      if (size == 0) return Collections.emptyList();
      list = new ArrayList<T>();
    } else {
      list = new ArrayList<T>();
    }
    for (T element : iterable) {
      if (skipNulls && element == null) continue;
      if (checkNulls) Objects.requireNonNull(element, "element");
      list.add(element);
    }
    return list;
  }

  private static <T> List<T> createUnmodifiableList(boolean clone, List<T> list) {
    switch(list.size()) {
    case 0: return Collections.emptyList();
    case 1: return Collections.singletonList(list.get(0));
    default:
      if (clone) {
        return Collections.unmodifiableList(new ArrayList<T>(list));
      } else {
        if (list instanceof ArrayList<?>) {
          ((ArrayList<?>) list).trimToSize();
        }
        return Collections.unmodifiableList(list);
      }
    }
  }

  private static <K, V> Map<K, V> createUnmodifiableMap(boolean checkNulls, boolean skipNulls, Map<? extends K, ? extends V> map) {
    switch (map.size()) {
    case 0: return Collections.emptyMap();
    case 1: {
      Map.Entry<? extends K, ? extends V> e = map.entrySet().iterator().next();
      K k = e.getKey();
      V v = e.getValue();
      if (checkNulls) {
        Objects.requireNonNull(k, "key");
        Objects.requireNonNull(v, "value");
      }
      if (skipNulls && (k == null || v == null)) {
        return Collections.emptyMap();
      }
      return Collections.singletonMap(k, v);
    }
    default: {
      Map<K, V> linkedMap = new LinkedHashMap<K, V>(map.size());
      if (skipNulls || checkNulls) {
        for (Map.Entry<? extends K, ? extends V> e : map.entrySet()) {
          K k = e.getKey();
          V v = e.getValue();
          if (skipNulls) {
            if (k == null || v == null) continue;
          } else if (checkNulls) {
            Objects.requireNonNull(k, "key");
            Objects.requireNonNull(v, "value");
          }
          linkedMap.put(k, v);
        }
      } else {
        linkedMap.putAll(map);
      }
      return Collections.unmodifiableMap(linkedMap);
    }
    }
  }
}
