/**
 * Copyright (C) 2016 Red Hat, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package io.syndesis.model.connection;

import com.fasterxml.jackson.annotation.JsonProperty;
import edu.umd.cs.findbugs.annotations.SuppressFBWarnings;
import java.io.ObjectStreamException;
import java.util.Objects;
import java.util.Optional;
import java.util.Set;
import javax.annotation.Generated;
import javax.validation.ConstraintViolation;
import javax.validation.ConstraintViolationException;
import javax.validation.Validation;
import javax.validation.Validator;

/**
 * Immutable implementation of {@link DataShape}.
 * <p>
 * Use the builder to create immutable instances:
 * {@code new DataShape.Builder()}.
 * Use the static factory method to create immutable instances:
 * {@code ImmutableDataShape.of()}.
 */
@SuppressWarnings({"all"})
@SuppressFBWarnings
@Generated({"Immutables.generator", "DataShape"})
final class ImmutableDataShape implements DataShape {
  private final String kind;
  private final String type;
  private final String specification;
  private final byte[] exemplar;

  private ImmutableDataShape(
      String kind,
      String type,
      String specification,
      Optional<byte[]> exemplar) {
    this.kind = kind;
    this.type = type;
    this.specification = specification;
    this.exemplar = exemplar.orElse(null);
  }

  private ImmutableDataShape(
      ImmutableDataShape original,
      String kind,
      String type,
      String specification,
      byte[] exemplar) {
    this.kind = kind;
    this.type = type;
    this.specification = specification;
    this.exemplar = exemplar;
  }

  /**
   * @return The value of the {@code kind} attribute
   */
  @JsonProperty("kind")
  @Override
  public String getKind() {
    return kind;
  }

  /**
   * @return The value of the {@code type} attribute
   */
  @JsonProperty("type")
  @Override
  public String getType() {
    return type;
  }

  /**
   * @return The value of the {@code specification} attribute
   */
  @JsonProperty("specification")
  @Override
  public String getSpecification() {
    return specification;
  }

  /**
   * @return The value of the {@code exemplar} attribute
   */
  @JsonProperty("exemplar")
  @Override
  public Optional<byte[]> getExemplar() {
    return Optional.ofNullable(exemplar);
  }

  /**
   * Copy the current immutable object by setting a value for the {@link DataShape#getKind() kind} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for kind (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableDataShape withKind(String value) {
    if (Objects.equals(this.kind, value)) return this;
    return validate(new ImmutableDataShape(this, value, this.type, this.specification, this.exemplar));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link DataShape#getType() type} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for type (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableDataShape withType(String value) {
    if (Objects.equals(this.type, value)) return this;
    return validate(new ImmutableDataShape(this, this.kind, value, this.specification, this.exemplar));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link DataShape#getSpecification() specification} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for specification (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableDataShape withSpecification(String value) {
    if (Objects.equals(this.specification, value)) return this;
    return validate(new ImmutableDataShape(this, this.kind, this.type, value, this.exemplar));
  }

  /**
   * Copy the current immutable object by setting a <i>present</i> value for the optional {@link DataShape#getExemplar() exemplar} attribute.
   * @param value The value for exemplar
   * @return A modified copy of {@code this} object
   */
  public final ImmutableDataShape withExemplar(byte[] value) {
    byte[] newValue = Objects.requireNonNull(value, "exemplar");
    if (this.exemplar == newValue) return this;
    return validate(new ImmutableDataShape(this, this.kind, this.type, this.specification, newValue));
  }

  /**
   * Copy the current immutable object by setting an optional value for the {@link DataShape#getExemplar() exemplar} attribute.
   * A shallow reference equality check is used on unboxed optional value to prevent copying of the same value by returning {@code this}.
   * @param optional A value for exemplar
   * @return A modified copy of {@code this} object
   */
  public final ImmutableDataShape withExemplar(Optional<? extends byte[]> optional) {
    byte[] value = optional.orElse(null);
    if (this.exemplar == value) return this;
    return validate(new ImmutableDataShape(this, this.kind, this.type, this.specification, value));
  }

  /**
   * This instance is equal to all instances of {@code ImmutableDataShape} that have equal attribute values.
   * @return {@code true} if {@code this} is equal to {@code another} instance
   */
  @Override
  public boolean equals(Object another) {
    if (this == another) return true;
    return another instanceof ImmutableDataShape
        && equalTo((ImmutableDataShape) another);
  }

  private boolean equalTo(ImmutableDataShape another) {
    return Objects.equals(kind, another.kind)
        && Objects.equals(type, another.type)
        && Objects.equals(specification, another.specification)
        && Objects.equals(exemplar, another.exemplar);
  }

  /**
   * Computes a hash code from attributes: {@code kind}, {@code type}, {@code specification}, {@code exemplar}.
   * @return hashCode value
   */
  @Override
  public int hashCode() {
    int h = 5381;
    h += (h << 5) + Objects.hashCode(kind);
    h += (h << 5) + Objects.hashCode(type);
    h += (h << 5) + Objects.hashCode(specification);
    h += (h << 5) + Objects.hashCode(exemplar);
    return h;
  }

  /**
   * Prints the immutable value {@code DataShape} with attribute values.
   * @return A string representation of the value
   */
  @Override
  public String toString() {
    StringBuilder builder = new StringBuilder("DataShape{");
    if (kind != null) {
      builder.append("kind=").append(kind);
    }
    if (type != null) {
      if (builder.length() > 10) builder.append(", ");
      builder.append("type=").append(type);
    }
    if (specification != null) {
      if (builder.length() > 10) builder.append(", ");
      builder.append("specification=").append(specification);
    }
    if (exemplar != null) {
      if (builder.length() > 10) builder.append(", ");
      builder.append("exemplar=").append(exemplar);
    }
    return builder.append("}").toString();
  }

  /**
   * Construct a new immutable {@code DataShape} instance.
   * @param kind The value for the {@code kind} attribute
   * @param type The value for the {@code type} attribute
   * @param specification The value for the {@code specification} attribute
   * @param exemplar The value for the {@code exemplar} attribute
   * @return An immutable DataShape instance
   */
  public static ImmutableDataShape of(String kind, String type, String specification, Optional<byte[]> exemplar) {
    return validate(new ImmutableDataShape(kind, type, specification, exemplar));
  }

  private static final Validator validator = Validation.buildDefaultValidatorFactory().getValidator();

  private static ImmutableDataShape validate(ImmutableDataShape instance) {
    Set<ConstraintViolation<ImmutableDataShape>> constraintViolations = validator.validate(instance);
    if (!constraintViolations.isEmpty()) {
      throw new ConstraintViolationException(constraintViolations);
    }
    return instance;
  }

  /**
   * Creates an immutable copy of a {@link DataShape} value.
   * Uses accessors to get values to initialize the new immutable instance.
   * If an instance is already immutable, it is returned as is.
   * @param instance The instance to copy
   * @return A copied immutable DataShape instance
   */
  public static ImmutableDataShape copyOf(DataShape instance) {
    if (instance instanceof ImmutableDataShape) {
      return (ImmutableDataShape) instance;
    }
    return new DataShape.Builder()
        .createFrom(instance)
        .build();
  }

  private Object readResolve() throws ObjectStreamException {
    return validate(this);
  }

  /**
   * Builds instances of type {@link ImmutableDataShape ImmutableDataShape}.
   * Initialize attributes and then invoke the {@link #build()} method to create an
   * immutable instance.
   * <p><em>{@code Builder} is not thread-safe and generally should not be stored in a field or collection,
   * but instead used immediately to create instances.</em>
   */
  static class Builder {
    private String kind;
    private String type;
    private String specification;
    private byte[] exemplar;

    /**
     * Creates a builder for {@link ImmutableDataShape ImmutableDataShape} instances.
     */
    Builder() {
      if (!(this instanceof DataShape.Builder)) {
        throw new UnsupportedOperationException("Use: new DataShape.Builder()");
      }
    }

    /**
     * Fill a builder with attribute values from the provided {@code DataShape} instance.
     * Regular attribute values will be replaced with those from the given instance.
     * Absent optional values will not replace present values.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    public final DataShape.Builder createFrom(DataShape instance) {
      Objects.requireNonNull(instance, "instance");
      String kindValue = instance.getKind();
      if (kindValue != null) {
        kind(kindValue);
      }
      String typeValue = instance.getType();
      if (typeValue != null) {
        type(typeValue);
      }
      String specificationValue = instance.getSpecification();
      if (specificationValue != null) {
        specification(specificationValue);
      }
      Optional<byte[]> exemplarOptional = instance.getExemplar();
      if (exemplarOptional.isPresent()) {
        exemplar(exemplarOptional);
      }
      return (DataShape.Builder) this;
    }

    /**
     * Initializes the value for the {@link DataShape#getKind() kind} attribute.
     * @param kind The value for kind (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("kind")
    public final DataShape.Builder kind(String kind) {
      this.kind = kind;
      return (DataShape.Builder) this;
    }

    /**
     * Initializes the value for the {@link DataShape#getType() type} attribute.
     * @param type The value for type (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("type")
    public final DataShape.Builder type(String type) {
      this.type = type;
      return (DataShape.Builder) this;
    }

    /**
     * Initializes the value for the {@link DataShape#getSpecification() specification} attribute.
     * @param specification The value for specification (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("specification")
    public final DataShape.Builder specification(String specification) {
      this.specification = specification;
      return (DataShape.Builder) this;
    }

    /**
     * Initializes the optional value {@link DataShape#getExemplar() exemplar} to exemplar.
     * @param exemplar The value for exemplar
     * @return {@code this} builder for chained invocation
     */
    public final DataShape.Builder exemplar(byte[] exemplar) {
      this.exemplar = Objects.requireNonNull(exemplar, "exemplar");
      return (DataShape.Builder) this;
    }

    /**
     * Initializes the optional value {@link DataShape#getExemplar() exemplar} to exemplar.
     * @param exemplar The value for exemplar
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("exemplar")
    public final DataShape.Builder exemplar(Optional<? extends byte[]> exemplar) {
      this.exemplar = exemplar.orElse(null);
      return (DataShape.Builder) this;
    }

    /**
     * Builds a new {@link ImmutableDataShape ImmutableDataShape}.
     * @return An immutable instance of DataShape
     * @throws java.lang.IllegalStateException if any required attributes are missing
     */
    public ImmutableDataShape build() {
      return ImmutableDataShape.validate(new ImmutableDataShape(null, kind, type, specification, exemplar));
    }
  }
}
