/**
 * Copyright (C) 2016 Red Hat, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package io.syndesis.model.connection;

import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
import edu.umd.cs.findbugs.annotations.SuppressFBWarnings;
import java.util.Collections;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Set;
import javax.annotation.Generated;
import javax.validation.ConstraintViolation;
import javax.validation.ConstraintViolationException;
import javax.validation.Validation;
import javax.validation.Validator;

/**
 * Immutable implementation of {@link DynamicActionMetadata}.
 * <p>
 * Use the builder to create immutable instances:
 * {@code new DynamicActionMetadata.Builder()}.
 * Use the static factory method to create immutable instances:
 * {@code ImmutableDynamicActionMetadata.of()}.
 */
@SuppressWarnings({"all"})
@SuppressFBWarnings
@Generated({"Immutables.generator", "DynamicActionMetadata"})
final class ImmutableDynamicActionMetadata
    implements DynamicActionMetadata {
  private final Map<String, List<DynamicActionMetadata.ActionPropertySuggestion>> properties;
  private final Object inputSchema;
  private final Object outputSchema;

  private ImmutableDynamicActionMetadata(
      Map<String, ? extends List<DynamicActionMetadata.ActionPropertySuggestion>> properties,
      Object inputSchema,
      Object outputSchema) {
    this.properties = createUnmodifiableMap(true, false, properties);
    this.inputSchema = inputSchema;
    this.outputSchema = outputSchema;
  }

  private ImmutableDynamicActionMetadata(
      ImmutableDynamicActionMetadata original,
      Map<String, List<DynamicActionMetadata.ActionPropertySuggestion>> properties,
      Object inputSchema,
      Object outputSchema) {
    this.properties = properties;
    this.inputSchema = inputSchema;
    this.outputSchema = outputSchema;
  }

  /**
   * @return The value of the {@code properties} attribute
   */
  @JsonProperty("properties")
  @Override
  public Map<String, List<DynamicActionMetadata.ActionPropertySuggestion>> properties() {
    return properties;
  }

  /**
   * @return The value of the {@code inputSchema} attribute
   */
  @JsonProperty("inputSchema")
  @JsonDeserialize(as = JsonNode.class)
  @Override
  public Object inputSchema() {
    return inputSchema;
  }

  /**
   * @return The value of the {@code outputSchema} attribute
   */
  @JsonProperty("outputSchema")
  @JsonDeserialize(as = JsonNode.class)
  @Override
  public Object outputSchema() {
    return outputSchema;
  }

  /**
   * Copy the current immutable object by replacing the {@link DynamicActionMetadata#properties() properties} map with the specified map.
   * Nulls are not permitted as keys or values.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param entries The entries to be added to the properties map
   * @return A modified copy of {@code this} object
   */
  public final ImmutableDynamicActionMetadata withProperties(Map<String, ? extends List<DynamicActionMetadata.ActionPropertySuggestion>> entries) {
    if (this.properties == entries) return this;
    Map<String, List<DynamicActionMetadata.ActionPropertySuggestion>> newValue = createUnmodifiableMap(true, false, entries);
    return validate(new ImmutableDynamicActionMetadata(this, newValue, this.inputSchema, this.outputSchema));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link DynamicActionMetadata#inputSchema() inputSchema} attribute.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for inputSchema (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableDynamicActionMetadata withInputSchema(Object value) {
    if (this.inputSchema == value) return this;
    return validate(new ImmutableDynamicActionMetadata(this, this.properties, value, this.outputSchema));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link DynamicActionMetadata#outputSchema() outputSchema} attribute.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for outputSchema (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableDynamicActionMetadata withOutputSchema(Object value) {
    if (this.outputSchema == value) return this;
    return validate(new ImmutableDynamicActionMetadata(this, this.properties, this.inputSchema, value));
  }

  /**
   * This instance is equal to all instances of {@code ImmutableDynamicActionMetadata} that have equal attribute values.
   * @return {@code true} if {@code this} is equal to {@code another} instance
   */
  @Override
  public boolean equals(Object another) {
    if (this == another) return true;
    return another instanceof ImmutableDynamicActionMetadata
        && equalTo((ImmutableDynamicActionMetadata) another);
  }

  private boolean equalTo(ImmutableDynamicActionMetadata another) {
    return properties.equals(another.properties)
        && Objects.equals(inputSchema, another.inputSchema)
        && Objects.equals(outputSchema, another.outputSchema);
  }

  /**
   * Computes a hash code from attributes: {@code properties}, {@code inputSchema}, {@code outputSchema}.
   * @return hashCode value
   */
  @Override
  public int hashCode() {
    int h = 5381;
    h += (h << 5) + properties.hashCode();
    h += (h << 5) + Objects.hashCode(inputSchema);
    h += (h << 5) + Objects.hashCode(outputSchema);
    return h;
  }

  /**
   * Prints the immutable value {@code DynamicActionMetadata} with attribute values.
   * @return A string representation of the value
   */
  @Override
  public String toString() {
    return "DynamicActionMetadata{"
        + "properties=" + properties
        + ", inputSchema=" + inputSchema
        + ", outputSchema=" + outputSchema
        + "}";
  }

  /**
   * Construct a new immutable {@code DynamicActionMetadata} instance.
   * @param properties The value for the {@code properties} attribute
   * @param inputSchema The value for the {@code inputSchema} attribute
   * @param outputSchema The value for the {@code outputSchema} attribute
   * @return An immutable DynamicActionMetadata instance
   */
  public static ImmutableDynamicActionMetadata of(Map<String, ? extends List<DynamicActionMetadata.ActionPropertySuggestion>> properties, Object inputSchema, Object outputSchema) {
    return validate(new ImmutableDynamicActionMetadata(properties, inputSchema, outputSchema));
  }

  private static final Validator validator = Validation.buildDefaultValidatorFactory().getValidator();

  private static ImmutableDynamicActionMetadata validate(ImmutableDynamicActionMetadata instance) {
    Set<ConstraintViolation<ImmutableDynamicActionMetadata>> constraintViolations = validator.validate(instance);
    if (!constraintViolations.isEmpty()) {
      throw new ConstraintViolationException(constraintViolations);
    }
    return instance;
  }

  /**
   * Creates an immutable copy of a {@link DynamicActionMetadata} value.
   * Uses accessors to get values to initialize the new immutable instance.
   * If an instance is already immutable, it is returned as is.
   * @param instance The instance to copy
   * @return A copied immutable DynamicActionMetadata instance
   */
  public static ImmutableDynamicActionMetadata copyOf(DynamicActionMetadata instance) {
    if (instance instanceof ImmutableDynamicActionMetadata) {
      return (ImmutableDynamicActionMetadata) instance;
    }
    return new DynamicActionMetadata.Builder()
        .createFrom(instance)
        .build();
  }

  /**
   * Builds instances of type {@link ImmutableDynamicActionMetadata ImmutableDynamicActionMetadata}.
   * Initialize attributes and then invoke the {@link #build()} method to create an
   * immutable instance.
   * <p><em>{@code Builder} is not thread-safe and generally should not be stored in a field or collection,
   * but instead used immediately to create instances.</em>
   */
  static class Builder {
    private Map<String, List<DynamicActionMetadata.ActionPropertySuggestion>> properties = new LinkedHashMap<String, List<DynamicActionMetadata.ActionPropertySuggestion>>();
    private Object inputSchema;
    private Object outputSchema;

    /**
     * Creates a builder for {@link ImmutableDynamicActionMetadata ImmutableDynamicActionMetadata} instances.
     */
    Builder() {
      if (!(this instanceof DynamicActionMetadata.Builder)) {
        throw new UnsupportedOperationException("Use: new DynamicActionMetadata.Builder()");
      }
    }

    /**
     * Fill a builder with attribute values from the provided {@code DynamicActionMetadata} instance.
     * Regular attribute values will be replaced with those from the given instance.
     * Absent optional values will not replace present values.
     * Collection elements and entries will be added, not replaced.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    public final DynamicActionMetadata.Builder createFrom(DynamicActionMetadata instance) {
      Objects.requireNonNull(instance, "instance");
      putAllProperties(instance.properties());
      Object inputSchemaValue = instance.inputSchema();
      if (inputSchemaValue != null) {
        inputSchema(inputSchemaValue);
      }
      Object outputSchemaValue = instance.outputSchema();
      if (outputSchemaValue != null) {
        outputSchema(outputSchemaValue);
      }
      return (DynamicActionMetadata.Builder) this;
    }

    /**
     * Put one entry to the {@link DynamicActionMetadata#properties() properties} map.
     * @param key The key in the properties map
     * @param value The associated value in the properties map
     * @return {@code this} builder for use in a chained invocation
     */
    public final DynamicActionMetadata.Builder putProperty(String key, List<DynamicActionMetadata.ActionPropertySuggestion> value) {
      this.properties.put(
          Objects.requireNonNull(key, "properties key"),
          Objects.requireNonNull(value, "properties value"));
      return (DynamicActionMetadata.Builder) this;
    }

    /**
     * Put one entry to the {@link DynamicActionMetadata#properties() properties} map. Nulls are not permitted
     * @param entry The key and value entry
     * @return {@code this} builder for use in a chained invocation
     */
    public final DynamicActionMetadata.Builder putProperty(Map.Entry<String, ? extends List<DynamicActionMetadata.ActionPropertySuggestion>> entry) {
      String k = entry.getKey();
      List<DynamicActionMetadata.ActionPropertySuggestion> v = entry.getValue();
      this.properties.put(
          Objects.requireNonNull(k, "properties key"),
          Objects.requireNonNull(v, "properties value"));
      return (DynamicActionMetadata.Builder) this;
    }

    /**
     * Sets or replaces all mappings from the specified map as entries for the {@link DynamicActionMetadata#properties() properties} map. Nulls are not permitted
     * @param properties The entries that will be added to the properties map
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("properties")
    public final DynamicActionMetadata.Builder properties(Map<String, ? extends List<DynamicActionMetadata.ActionPropertySuggestion>> properties) {
      this.properties.clear();
      return putAllProperties(properties);
    }

    /**
     * Put all mappings from the specified map as entries to {@link DynamicActionMetadata#properties() properties} map. Nulls are not permitted
     * @param properties The entries that will be added to the properties map
     * @return {@code this} builder for use in a chained invocation
     */
    public final DynamicActionMetadata.Builder putAllProperties(Map<String, ? extends List<DynamicActionMetadata.ActionPropertySuggestion>> properties) {
      for (Map.Entry<String, ? extends List<DynamicActionMetadata.ActionPropertySuggestion>> entry : properties.entrySet()) {
        String k = entry.getKey();
        List<DynamicActionMetadata.ActionPropertySuggestion> v = entry.getValue();
        this.properties.put(
            Objects.requireNonNull(k, "properties key"),
            Objects.requireNonNull(v, "properties value"));
      }
      return (DynamicActionMetadata.Builder) this;
    }

    /**
     * Initializes the value for the {@link DynamicActionMetadata#inputSchema() inputSchema} attribute.
     * @param inputSchema The value for inputSchema (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("inputSchema")
    @JsonDeserialize(as = JsonNode.class)
    public final DynamicActionMetadata.Builder inputSchema(Object inputSchema) {
      this.inputSchema = inputSchema;
      return (DynamicActionMetadata.Builder) this;
    }

    /**
     * Initializes the value for the {@link DynamicActionMetadata#outputSchema() outputSchema} attribute.
     * @param outputSchema The value for outputSchema (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("outputSchema")
    @JsonDeserialize(as = JsonNode.class)
    public final DynamicActionMetadata.Builder outputSchema(Object outputSchema) {
      this.outputSchema = outputSchema;
      return (DynamicActionMetadata.Builder) this;
    }

    /**
     * Builds a new {@link ImmutableDynamicActionMetadata ImmutableDynamicActionMetadata}.
     * @return An immutable instance of DynamicActionMetadata
     * @throws java.lang.IllegalStateException if any required attributes are missing
     */
    public ImmutableDynamicActionMetadata build() {
      return ImmutableDynamicActionMetadata.validate(new ImmutableDynamicActionMetadata(null, createUnmodifiableMap(false, false, properties), inputSchema, outputSchema));
    }
  }

  private static <K, V> Map<K, V> createUnmodifiableMap(boolean checkNulls, boolean skipNulls, Map<? extends K, ? extends V> map) {
    switch (map.size()) {
    case 0: return Collections.emptyMap();
    case 1: {
      Map.Entry<? extends K, ? extends V> e = map.entrySet().iterator().next();
      K k = e.getKey();
      V v = e.getValue();
      if (checkNulls) {
        Objects.requireNonNull(k, "key");
        Objects.requireNonNull(v, "value");
      }
      if (skipNulls && (k == null || v == null)) {
        return Collections.emptyMap();
      }
      return Collections.singletonMap(k, v);
    }
    default: {
      Map<K, V> linkedMap = new LinkedHashMap<K, V>(map.size());
      if (skipNulls || checkNulls) {
        for (Map.Entry<? extends K, ? extends V> e : map.entrySet()) {
          K k = e.getKey();
          V v = e.getValue();
          if (skipNulls) {
            if (k == null || v == null) continue;
          } else if (checkNulls) {
            Objects.requireNonNull(k, "key");
            Objects.requireNonNull(v, "value");
          }
          linkedMap.put(k, v);
        }
      } else {
        linkedMap.putAll(map);
      }
      return Collections.unmodifiableMap(linkedMap);
    }
    }
  }
}
