/**
 * Copyright (C) 2016 Red Hat, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package io.syndesis.model.environment;

import com.fasterxml.jackson.annotation.JsonProperty;
import edu.umd.cs.findbugs.annotations.SuppressFBWarnings;
import io.syndesis.model.WithName;
import java.io.ObjectStreamException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.Objects;
import java.util.Optional;
import java.util.Set;
import javax.annotation.Generated;
import javax.validation.ConstraintViolation;
import javax.validation.ConstraintViolationException;
import javax.validation.Validation;
import javax.validation.Validator;

/**
 * Immutable implementation of {@link Environment}.
 * <p>
 * Use the builder to create immutable instances:
 * {@code new Environment.Builder()}.
 * Use the static factory method to create immutable instances:
 * {@code ImmutableEnvironment.of()}.
 */
@SuppressWarnings({"all"})
@SuppressFBWarnings
@Generated({"Immutables.generator", "Environment"})
final class ImmutableEnvironment implements Environment {
  private final EnvironmentType environmentType;
  private final List<Organization> organizations;
  private final String id;
  private final String name;

  private ImmutableEnvironment(
      EnvironmentType environmentType,
      Iterable<? extends Organization> organizations,
      Optional<String> id,
      String name) {
    this.environmentType = environmentType;
    this.organizations = createUnmodifiableList(false, createSafeList(organizations, true, false));
    this.id = id.orElse(null);
    this.name = name;
  }

  private ImmutableEnvironment(
      ImmutableEnvironment original,
      EnvironmentType environmentType,
      List<Organization> organizations,
      String id,
      String name) {
    this.environmentType = environmentType;
    this.organizations = organizations;
    this.id = id;
    this.name = name;
  }

  /**
   * @return The value of the {@code environmentType} attribute
   */
  @JsonProperty("environmentType")
  @Override
  public EnvironmentType environmentType() {
    return environmentType;
  }

  /**
   * @return The value of the {@code organizations} attribute
   */
  @JsonProperty("organizations")
  @Override
  public List<Organization> organizations() {
    return organizations;
  }

  /**
   * @return The value of the {@code id} attribute
   */
  @JsonProperty("id")
  @Override
  public Optional<String> getId() {
    return Optional.ofNullable(id);
  }

  /**
   * @return The value of the {@code name} attribute
   */
  @JsonProperty("name")
  @Override
  public String getName() {
    return name;
  }

  /**
   * Copy the current immutable object by setting a value for the {@link Environment#environmentType() environmentType} attribute.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for environmentType (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableEnvironment withEnvironmentType(EnvironmentType value) {
    if (this.environmentType == value) return this;
    return validate(new ImmutableEnvironment(this, value, this.organizations, this.id, this.name));
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link Environment#organizations() organizations}.
   * @param elements The elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableEnvironment withOrganizations(Organization... elements) {
    List<Organization> newValue = createUnmodifiableList(false, createSafeList(Arrays.asList(elements), true, false));
    return validate(new ImmutableEnvironment(this, this.environmentType, newValue, this.id, this.name));
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link Environment#organizations() organizations}.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param elements An iterable of organizations elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableEnvironment withOrganizations(Iterable<? extends Organization> elements) {
    if (this.organizations == elements) return this;
    List<Organization> newValue = createUnmodifiableList(false, createSafeList(elements, true, false));
    return validate(new ImmutableEnvironment(this, this.environmentType, newValue, this.id, this.name));
  }

  /**
   * Copy the current immutable object by setting a <i>present</i> value for the optional {@link Environment#getId() id} attribute.
   * @param value The value for id
   * @return A modified copy of {@code this} object
   */
  public final ImmutableEnvironment withId(String value) {
    String newValue = Objects.requireNonNull(value, "id");
    if (Objects.equals(this.id, newValue)) return this;
    return validate(new ImmutableEnvironment(this, this.environmentType, this.organizations, newValue, this.name));
  }

  /**
   * Copy the current immutable object by setting an optional value for the {@link Environment#getId() id} attribute.
   * An equality check is used on inner nullable value to prevent copying of the same value by returning {@code this}.
   * @param optional A value for id
   * @return A modified copy of {@code this} object
   */
  public final ImmutableEnvironment withId(Optional<String> optional) {
    String value = optional.orElse(null);
    if (Objects.equals(this.id, value)) return this;
    return validate(new ImmutableEnvironment(this, this.environmentType, this.organizations, value, this.name));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link Environment#getName() name} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for name (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableEnvironment withName(String value) {
    if (Objects.equals(this.name, value)) return this;
    return validate(new ImmutableEnvironment(this, this.environmentType, this.organizations, this.id, value));
  }

  /**
   * This instance is equal to all instances of {@code ImmutableEnvironment} that have equal attribute values.
   * @return {@code true} if {@code this} is equal to {@code another} instance
   */
  @Override
  public boolean equals(Object another) {
    if (this == another) return true;
    return another instanceof ImmutableEnvironment
        && equalTo((ImmutableEnvironment) another);
  }

  private boolean equalTo(ImmutableEnvironment another) {
    return Objects.equals(environmentType, another.environmentType)
        && organizations.equals(another.organizations)
        && Objects.equals(id, another.id)
        && Objects.equals(name, another.name);
  }

  /**
   * Computes a hash code from attributes: {@code environmentType}, {@code organizations}, {@code id}, {@code name}.
   * @return hashCode value
   */
  @Override
  public int hashCode() {
    int h = 5381;
    h += (h << 5) + Objects.hashCode(environmentType);
    h += (h << 5) + organizations.hashCode();
    h += (h << 5) + Objects.hashCode(id);
    h += (h << 5) + Objects.hashCode(name);
    return h;
  }

  /**
   * Prints the immutable value {@code Environment} with attribute values.
   * @return A string representation of the value
   */
  @Override
  public String toString() {
    StringBuilder builder = new StringBuilder("Environment{");
    if (environmentType != null) {
      builder.append("environmentType=").append(environmentType);
    }
    if (builder.length() > 12) builder.append(", ");
    builder.append("organizations=").append(organizations);
    if (id != null) {
      builder.append(", ");
      builder.append("id=").append(id);
    }
    if (name != null) {
      builder.append(", ");
      builder.append("name=").append(name);
    }
    return builder.append("}").toString();
  }

  /**
   * Construct a new immutable {@code Environment} instance.
   * @param environmentType The value for the {@code environmentType} attribute
   * @param organizations The value for the {@code organizations} attribute
   * @param id The value for the {@code id} attribute
   * @param name The value for the {@code name} attribute
   * @return An immutable Environment instance
   */
  public static ImmutableEnvironment of(EnvironmentType environmentType, List<Organization> organizations, Optional<String> id, String name) {
    return of(environmentType, (Iterable<? extends Organization>) organizations, id, name);
  }

  /**
   * Construct a new immutable {@code Environment} instance.
   * @param environmentType The value for the {@code environmentType} attribute
   * @param organizations The value for the {@code organizations} attribute
   * @param id The value for the {@code id} attribute
   * @param name The value for the {@code name} attribute
   * @return An immutable Environment instance
   */
  public static ImmutableEnvironment of(EnvironmentType environmentType, Iterable<? extends Organization> organizations, Optional<String> id, String name) {
    return validate(new ImmutableEnvironment(environmentType, organizations, id, name));
  }

  private static final Validator validator = Validation.buildDefaultValidatorFactory().getValidator();

  private static ImmutableEnvironment validate(ImmutableEnvironment instance) {
    Set<ConstraintViolation<ImmutableEnvironment>> constraintViolations = validator.validate(instance);
    if (!constraintViolations.isEmpty()) {
      throw new ConstraintViolationException(constraintViolations);
    }
    return instance;
  }

  /**
   * Creates an immutable copy of a {@link Environment} value.
   * Uses accessors to get values to initialize the new immutable instance.
   * If an instance is already immutable, it is returned as is.
   * @param instance The instance to copy
   * @return A copied immutable Environment instance
   */
  public static ImmutableEnvironment copyOf(Environment instance) {
    if (instance instanceof ImmutableEnvironment) {
      return (ImmutableEnvironment) instance;
    }
    return new Environment.Builder()
        .createFrom(instance)
        .build();
  }

  private Object readResolve() throws ObjectStreamException {
    return validate(this);
  }

  /**
   * Builds instances of type {@link ImmutableEnvironment ImmutableEnvironment}.
   * Initialize attributes and then invoke the {@link #build()} method to create an
   * immutable instance.
   * <p><em>{@code Builder} is not thread-safe and generally should not be stored in a field or collection,
   * but instead used immediately to create instances.</em>
   */
  static class Builder {
    private EnvironmentType environmentType;
    private List<Organization> organizations = new ArrayList<Organization>();
    private String id;
    private String name;

    /**
     * Creates a builder for {@link ImmutableEnvironment ImmutableEnvironment} instances.
     */
    Builder() {
      if (!(this instanceof Environment.Builder)) {
        throw new UnsupportedOperationException("Use: new Environment.Builder()");
      }
    }

    /**
     * Fill a builder with attribute values from the provided {@code io.syndesis.model.environment.Environment} instance.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    public final Environment.Builder createFrom(Environment instance) {
      Objects.requireNonNull(instance, "instance");
      from((Object) instance);
      return (Environment.Builder) this;
    }

    /**
     * Fill a builder with attribute values from the provided {@code io.syndesis.model.WithName} instance.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    public final Environment.Builder createFrom(WithName instance) {
      Objects.requireNonNull(instance, "instance");
      from((Object) instance);
      return (Environment.Builder) this;
    }

    private void from(Object object) {
      if (object instanceof Environment) {
        Environment instance = (Environment) object;
        addAllOrganizations(instance.organizations());
        EnvironmentType environmentTypeValue = instance.environmentType();
        if (environmentTypeValue != null) {
          environmentType(environmentTypeValue);
        }
        Optional<String> idOptional = instance.getId();
        if (idOptional.isPresent()) {
          id(idOptional);
        }
      }
      if (object instanceof WithName) {
        WithName instance = (WithName) object;
        String nameValue = instance.getName();
        if (nameValue != null) {
          name(nameValue);
        }
      }
    }

    /**
     * Initializes the value for the {@link Environment#environmentType() environmentType} attribute.
     * @param environmentType The value for environmentType (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("environmentType")
    public final Environment.Builder environmentType(EnvironmentType environmentType) {
      this.environmentType = environmentType;
      return (Environment.Builder) this;
    }

    /**
     * Adds one element to {@link Environment#organizations() organizations} list.
     * @param element A organizations element
     * @return {@code this} builder for use in a chained invocation
     */
    public final Environment.Builder addOrganization(Organization element) {
      this.organizations.add(Objects.requireNonNull(element, "organizations element"));
      return (Environment.Builder) this;
    }

    /**
     * Adds elements to {@link Environment#organizations() organizations} list.
     * @param elements An array of organizations elements
     * @return {@code this} builder for use in a chained invocation
     */
    public final Environment.Builder addOrganization(Organization... elements) {
      for (Organization element : elements) {
        this.organizations.add(Objects.requireNonNull(element, "organizations element"));
      }
      return (Environment.Builder) this;
    }

    /**
     * Sets or replaces all elements for {@link Environment#organizations() organizations} list.
     * @param elements An iterable of organizations elements
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("organizations")
    public final Environment.Builder organizations(Iterable<? extends Organization> elements) {
      this.organizations.clear();
      return addAllOrganizations(elements);
    }

    /**
     * Adds elements to {@link Environment#organizations() organizations} list.
     * @param elements An iterable of organizations elements
     * @return {@code this} builder for use in a chained invocation
     */
    public final Environment.Builder addAllOrganizations(Iterable<? extends Organization> elements) {
      for (Organization element : elements) {
        this.organizations.add(Objects.requireNonNull(element, "organizations element"));
      }
      return (Environment.Builder) this;
    }

    /**
     * Initializes the optional value {@link Environment#getId() id} to id.
     * @param id The value for id
     * @return {@code this} builder for chained invocation
     */
    public final Environment.Builder id(String id) {
      this.id = Objects.requireNonNull(id, "id");
      return (Environment.Builder) this;
    }

    /**
     * Initializes the optional value {@link Environment#getId() id} to id.
     * @param id The value for id
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("id")
    public final Environment.Builder id(Optional<String> id) {
      this.id = id.orElse(null);
      return (Environment.Builder) this;
    }

    /**
     * Initializes the value for the {@link Environment#getName() name} attribute.
     * @param name The value for name (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("name")
    public final Environment.Builder name(String name) {
      this.name = name;
      return (Environment.Builder) this;
    }

    /**
     * Builds a new {@link ImmutableEnvironment ImmutableEnvironment}.
     * @return An immutable instance of Environment
     * @throws java.lang.IllegalStateException if any required attributes are missing
     */
    public ImmutableEnvironment build() {
      return ImmutableEnvironment.validate(new ImmutableEnvironment(null, environmentType, createUnmodifiableList(true, organizations), id, name));
    }
  }

  private static <T> List<T> createSafeList(Iterable<? extends T> iterable, boolean checkNulls, boolean skipNulls) {
    ArrayList<T> list;
    if (iterable instanceof Collection<?>) {
      int size = ((Collection<?>) iterable).size();
      if (size == 0) return Collections.emptyList();
      list = new ArrayList<T>();
    } else {
      list = new ArrayList<T>();
    }
    for (T element : iterable) {
      if (skipNulls && element == null) continue;
      if (checkNulls) Objects.requireNonNull(element, "element");
      list.add(element);
    }
    return list;
  }

  private static <T> List<T> createUnmodifiableList(boolean clone, List<T> list) {
    switch(list.size()) {
    case 0: return Collections.emptyList();
    case 1: return Collections.singletonList(list.get(0));
    default:
      if (clone) {
        return Collections.unmodifiableList(new ArrayList<T>(list));
      } else {
        if (list instanceof ArrayList<?>) {
          ((ArrayList<?>) list).trimToSize();
        }
        return Collections.unmodifiableList(list);
      }
    }
  }
}
