/**
 * Copyright (C) 2016 Red Hat, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package io.syndesis.model.environment;

import com.fasterxml.jackson.annotation.JsonProperty;
import edu.umd.cs.findbugs.annotations.SuppressFBWarnings;
import io.syndesis.model.WithName;
import io.syndesis.model.user.User;
import java.io.ObjectStreamException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.Objects;
import java.util.Optional;
import java.util.Set;
import javax.annotation.Generated;
import javax.validation.ConstraintViolation;
import javax.validation.ConstraintViolationException;
import javax.validation.Validation;
import javax.validation.Validator;

/**
 * Immutable implementation of {@link Organization}.
 * <p>
 * Use the builder to create immutable instances:
 * {@code new Organization.Builder()}.
 * Use the static factory method to create immutable instances:
 * {@code ImmutableOrganization.of()}.
 */
@SuppressWarnings({"all"})
@SuppressFBWarnings
@Generated({"Immutables.generator", "Organization"})
final class ImmutableOrganization implements Organization {
  private final List<Environment> environments;
  private final List<User> users;
  private final String id;
  private final String name;

  private ImmutableOrganization(
      Iterable<? extends Environment> environments,
      Iterable<? extends User> users,
      Optional<String> id,
      String name) {
    this.environments = createUnmodifiableList(false, createSafeList(environments, true, false));
    this.users = createUnmodifiableList(false, createSafeList(users, true, false));
    this.id = id.orElse(null);
    this.name = name;
  }

  private ImmutableOrganization(
      ImmutableOrganization original,
      List<Environment> environments,
      List<User> users,
      String id,
      String name) {
    this.environments = environments;
    this.users = users;
    this.id = id;
    this.name = name;
  }

  /**
   * @return The value of the {@code environments} attribute
   */
  @JsonProperty("environments")
  @Override
  public List<Environment> getEnvironments() {
    return environments;
  }

  /**
   * @return The value of the {@code users} attribute
   */
  @JsonProperty("users")
  @Override
  public List<User> getUsers() {
    return users;
  }

  /**
   * @return The value of the {@code id} attribute
   */
  @JsonProperty("id")
  @Override
  public Optional<String> getId() {
    return Optional.ofNullable(id);
  }

  /**
   * @return The value of the {@code name} attribute
   */
  @JsonProperty("name")
  @Override
  public String getName() {
    return name;
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link Organization#getEnvironments() environments}.
   * @param elements The elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableOrganization withEnvironments(Environment... elements) {
    List<Environment> newValue = createUnmodifiableList(false, createSafeList(Arrays.asList(elements), true, false));
    return validate(new ImmutableOrganization(this, newValue, this.users, this.id, this.name));
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link Organization#getEnvironments() environments}.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param elements An iterable of environments elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableOrganization withEnvironments(Iterable<? extends Environment> elements) {
    if (this.environments == elements) return this;
    List<Environment> newValue = createUnmodifiableList(false, createSafeList(elements, true, false));
    return validate(new ImmutableOrganization(this, newValue, this.users, this.id, this.name));
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link Organization#getUsers() users}.
   * @param elements The elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableOrganization withUsers(User... elements) {
    List<User> newValue = createUnmodifiableList(false, createSafeList(Arrays.asList(elements), true, false));
    return validate(new ImmutableOrganization(this, this.environments, newValue, this.id, this.name));
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link Organization#getUsers() users}.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param elements An iterable of users elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableOrganization withUsers(Iterable<? extends User> elements) {
    if (this.users == elements) return this;
    List<User> newValue = createUnmodifiableList(false, createSafeList(elements, true, false));
    return validate(new ImmutableOrganization(this, this.environments, newValue, this.id, this.name));
  }

  /**
   * Copy the current immutable object by setting a <i>present</i> value for the optional {@link Organization#getId() id} attribute.
   * @param value The value for id
   * @return A modified copy of {@code this} object
   */
  public final ImmutableOrganization withId(String value) {
    String newValue = Objects.requireNonNull(value, "id");
    if (Objects.equals(this.id, newValue)) return this;
    return validate(new ImmutableOrganization(this, this.environments, this.users, newValue, this.name));
  }

  /**
   * Copy the current immutable object by setting an optional value for the {@link Organization#getId() id} attribute.
   * An equality check is used on inner nullable value to prevent copying of the same value by returning {@code this}.
   * @param optional A value for id
   * @return A modified copy of {@code this} object
   */
  public final ImmutableOrganization withId(Optional<String> optional) {
    String value = optional.orElse(null);
    if (Objects.equals(this.id, value)) return this;
    return validate(new ImmutableOrganization(this, this.environments, this.users, value, this.name));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link Organization#getName() name} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for name (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableOrganization withName(String value) {
    if (Objects.equals(this.name, value)) return this;
    return validate(new ImmutableOrganization(this, this.environments, this.users, this.id, value));
  }

  /**
   * This instance is equal to all instances of {@code ImmutableOrganization} that have equal attribute values.
   * @return {@code true} if {@code this} is equal to {@code another} instance
   */
  @Override
  public boolean equals(Object another) {
    if (this == another) return true;
    return another instanceof ImmutableOrganization
        && equalTo((ImmutableOrganization) another);
  }

  private boolean equalTo(ImmutableOrganization another) {
    return environments.equals(another.environments)
        && users.equals(another.users)
        && Objects.equals(id, another.id)
        && Objects.equals(name, another.name);
  }

  /**
   * Computes a hash code from attributes: {@code environments}, {@code users}, {@code id}, {@code name}.
   * @return hashCode value
   */
  @Override
  public int hashCode() {
    int h = 5381;
    h += (h << 5) + environments.hashCode();
    h += (h << 5) + users.hashCode();
    h += (h << 5) + Objects.hashCode(id);
    h += (h << 5) + Objects.hashCode(name);
    return h;
  }

  /**
   * Prints the immutable value {@code Organization} with attribute values.
   * @return A string representation of the value
   */
  @Override
  public String toString() {
    StringBuilder builder = new StringBuilder("Organization{");
    builder.append("environments=").append(environments);
    builder.append(", ");
    builder.append("users=").append(users);
    if (id != null) {
      builder.append(", ");
      builder.append("id=").append(id);
    }
    if (name != null) {
      builder.append(", ");
      builder.append("name=").append(name);
    }
    return builder.append("}").toString();
  }

  /**
   * Construct a new immutable {@code Organization} instance.
   * @param environments The value for the {@code environments} attribute
   * @param users The value for the {@code users} attribute
   * @param id The value for the {@code id} attribute
   * @param name The value for the {@code name} attribute
   * @return An immutable Organization instance
   */
  public static ImmutableOrganization of(List<Environment> environments, List<User> users, Optional<String> id, String name) {
    return of((Iterable<? extends Environment>) environments, (Iterable<? extends User>) users, id, name);
  }

  /**
   * Construct a new immutable {@code Organization} instance.
   * @param environments The value for the {@code environments} attribute
   * @param users The value for the {@code users} attribute
   * @param id The value for the {@code id} attribute
   * @param name The value for the {@code name} attribute
   * @return An immutable Organization instance
   */
  public static ImmutableOrganization of(Iterable<? extends Environment> environments, Iterable<? extends User> users, Optional<String> id, String name) {
    return validate(new ImmutableOrganization(environments, users, id, name));
  }

  private static final Validator validator = Validation.buildDefaultValidatorFactory().getValidator();

  private static ImmutableOrganization validate(ImmutableOrganization instance) {
    Set<ConstraintViolation<ImmutableOrganization>> constraintViolations = validator.validate(instance);
    if (!constraintViolations.isEmpty()) {
      throw new ConstraintViolationException(constraintViolations);
    }
    return instance;
  }

  /**
   * Creates an immutable copy of a {@link Organization} value.
   * Uses accessors to get values to initialize the new immutable instance.
   * If an instance is already immutable, it is returned as is.
   * @param instance The instance to copy
   * @return A copied immutable Organization instance
   */
  public static ImmutableOrganization copyOf(Organization instance) {
    if (instance instanceof ImmutableOrganization) {
      return (ImmutableOrganization) instance;
    }
    return new Organization.Builder()
        .createFrom(instance)
        .build();
  }

  private Object readResolve() throws ObjectStreamException {
    return validate(this);
  }

  /**
   * Builds instances of type {@link ImmutableOrganization ImmutableOrganization}.
   * Initialize attributes and then invoke the {@link #build()} method to create an
   * immutable instance.
   * <p><em>{@code Builder} is not thread-safe and generally should not be stored in a field or collection,
   * but instead used immediately to create instances.</em>
   */
  static class Builder {
    private List<Environment> environments = new ArrayList<Environment>();
    private List<User> users = new ArrayList<User>();
    private String id;
    private String name;

    /**
     * Creates a builder for {@link ImmutableOrganization ImmutableOrganization} instances.
     */
    Builder() {
      if (!(this instanceof Organization.Builder)) {
        throw new UnsupportedOperationException("Use: new Organization.Builder()");
      }
    }

    /**
     * Fill a builder with attribute values from the provided {@code io.syndesis.model.environment.Organization} instance.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    public final Organization.Builder createFrom(Organization instance) {
      Objects.requireNonNull(instance, "instance");
      from((Object) instance);
      return (Organization.Builder) this;
    }

    /**
     * Fill a builder with attribute values from the provided {@code io.syndesis.model.WithName} instance.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    public final Organization.Builder createFrom(WithName instance) {
      Objects.requireNonNull(instance, "instance");
      from((Object) instance);
      return (Organization.Builder) this;
    }

    private void from(Object object) {
      if (object instanceof Organization) {
        Organization instance = (Organization) object;
        addAllEnvironments(instance.getEnvironments());
        addAllUsers(instance.getUsers());
        Optional<String> idOptional = instance.getId();
        if (idOptional.isPresent()) {
          id(idOptional);
        }
      }
      if (object instanceof WithName) {
        WithName instance = (WithName) object;
        String nameValue = instance.getName();
        if (nameValue != null) {
          name(nameValue);
        }
      }
    }

    /**
     * Adds one element to {@link Organization#getEnvironments() environments} list.
     * @param element A environments element
     * @return {@code this} builder for use in a chained invocation
     */
    public final Organization.Builder addEnvironment(Environment element) {
      this.environments.add(Objects.requireNonNull(element, "environments element"));
      return (Organization.Builder) this;
    }

    /**
     * Adds elements to {@link Organization#getEnvironments() environments} list.
     * @param elements An array of environments elements
     * @return {@code this} builder for use in a chained invocation
     */
    public final Organization.Builder addEnvironment(Environment... elements) {
      for (Environment element : elements) {
        this.environments.add(Objects.requireNonNull(element, "environments element"));
      }
      return (Organization.Builder) this;
    }

    /**
     * Sets or replaces all elements for {@link Organization#getEnvironments() environments} list.
     * @param elements An iterable of environments elements
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("environments")
    public final Organization.Builder environments(Iterable<? extends Environment> elements) {
      this.environments.clear();
      return addAllEnvironments(elements);
    }

    /**
     * Adds elements to {@link Organization#getEnvironments() environments} list.
     * @param elements An iterable of environments elements
     * @return {@code this} builder for use in a chained invocation
     */
    public final Organization.Builder addAllEnvironments(Iterable<? extends Environment> elements) {
      for (Environment element : elements) {
        this.environments.add(Objects.requireNonNull(element, "environments element"));
      }
      return (Organization.Builder) this;
    }

    /**
     * Adds one element to {@link Organization#getUsers() users} list.
     * @param element A users element
     * @return {@code this} builder for use in a chained invocation
     */
    public final Organization.Builder addUser(User element) {
      this.users.add(Objects.requireNonNull(element, "users element"));
      return (Organization.Builder) this;
    }

    /**
     * Adds elements to {@link Organization#getUsers() users} list.
     * @param elements An array of users elements
     * @return {@code this} builder for use in a chained invocation
     */
    public final Organization.Builder addUser(User... elements) {
      for (User element : elements) {
        this.users.add(Objects.requireNonNull(element, "users element"));
      }
      return (Organization.Builder) this;
    }

    /**
     * Sets or replaces all elements for {@link Organization#getUsers() users} list.
     * @param elements An iterable of users elements
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("users")
    public final Organization.Builder users(Iterable<? extends User> elements) {
      this.users.clear();
      return addAllUsers(elements);
    }

    /**
     * Adds elements to {@link Organization#getUsers() users} list.
     * @param elements An iterable of users elements
     * @return {@code this} builder for use in a chained invocation
     */
    public final Organization.Builder addAllUsers(Iterable<? extends User> elements) {
      for (User element : elements) {
        this.users.add(Objects.requireNonNull(element, "users element"));
      }
      return (Organization.Builder) this;
    }

    /**
     * Initializes the optional value {@link Organization#getId() id} to id.
     * @param id The value for id
     * @return {@code this} builder for chained invocation
     */
    public final Organization.Builder id(String id) {
      this.id = Objects.requireNonNull(id, "id");
      return (Organization.Builder) this;
    }

    /**
     * Initializes the optional value {@link Organization#getId() id} to id.
     * @param id The value for id
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("id")
    public final Organization.Builder id(Optional<String> id) {
      this.id = id.orElse(null);
      return (Organization.Builder) this;
    }

    /**
     * Initializes the value for the {@link Organization#getName() name} attribute.
     * @param name The value for name (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("name")
    public final Organization.Builder name(String name) {
      this.name = name;
      return (Organization.Builder) this;
    }

    /**
     * Builds a new {@link ImmutableOrganization ImmutableOrganization}.
     * @return An immutable instance of Organization
     * @throws java.lang.IllegalStateException if any required attributes are missing
     */
    public ImmutableOrganization build() {
      return ImmutableOrganization.validate(new ImmutableOrganization(
          null,
          createUnmodifiableList(true, environments),
          createUnmodifiableList(true, users),
          id,
          name));
    }
  }

  private static <T> List<T> createSafeList(Iterable<? extends T> iterable, boolean checkNulls, boolean skipNulls) {
    ArrayList<T> list;
    if (iterable instanceof Collection<?>) {
      int size = ((Collection<?>) iterable).size();
      if (size == 0) return Collections.emptyList();
      list = new ArrayList<T>();
    } else {
      list = new ArrayList<T>();
    }
    for (T element : iterable) {
      if (skipNulls && element == null) continue;
      if (checkNulls) Objects.requireNonNull(element, "element");
      list.add(element);
    }
    return list;
  }

  private static <T> List<T> createUnmodifiableList(boolean clone, List<T> list) {
    switch(list.size()) {
    case 0: return Collections.emptyList();
    case 1: return Collections.singletonList(list.get(0));
    default:
      if (clone) {
        return Collections.unmodifiableList(new ArrayList<T>(list));
      } else {
        if (list instanceof ArrayList<?>) {
          ((ArrayList<?>) list).trimToSize();
        }
        return Collections.unmodifiableList(list);
      }
    }
  }
}
