/**
 * Copyright (C) 2016 Red Hat, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package io.syndesis.model.filter;

import com.fasterxml.jackson.annotation.JsonProperty;
import edu.umd.cs.findbugs.annotations.SuppressFBWarnings;
import java.io.ObjectStreamException;
import java.util.Objects;
import java.util.Set;
import javax.annotation.Generated;
import javax.validation.ConstraintViolation;
import javax.validation.ConstraintViolationException;
import javax.validation.Validation;
import javax.validation.Validator;

/**
 * Immutable implementation of {@link Op}.
 * <p>
 * Use the builder to create immutable instances:
 * {@code new Op.Builder()}.
 * Use the static factory method to create immutable instances:
 * {@code ImmutableOp.of()}.
 */
@SuppressWarnings({"PMD.ShortClassName", "all"})
@SuppressFBWarnings
@Generated({"Immutables.generator", "Op"})
final class ImmutableOp implements Op {
  private final String label;
  private final String operator;

  private ImmutableOp(String label, String operator) {
    this.label = label;
    this.operator = operator;
  }

  /**
   * @return The value of the {@code label} attribute
   */
  @JsonProperty("label")
  @Override
  public String getLabel() {
    return label;
  }

  /**
   * @return The value of the {@code operator} attribute
   */
  @JsonProperty("operator")
  @Override
  public String getOperator() {
    return operator;
  }

  /**
   * Copy the current immutable object by setting a value for the {@link Op#getLabel() label} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for label (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableOp withLabel(String value) {
    if (Objects.equals(this.label, value)) return this;
    return validate(new ImmutableOp(value, this.operator));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link Op#getOperator() operator} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for operator (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableOp withOperator(String value) {
    if (Objects.equals(this.operator, value)) return this;
    return validate(new ImmutableOp(this.label, value));
  }

  /**
   * This instance is equal to all instances of {@code ImmutableOp} that have equal attribute values.
   * @return {@code true} if {@code this} is equal to {@code another} instance
   */
  @Override
  public boolean equals(Object another) {
    if (this == another) return true;
    return another instanceof ImmutableOp
        && equalTo((ImmutableOp) another);
  }

  private boolean equalTo(ImmutableOp another) {
    return Objects.equals(label, another.label)
        && Objects.equals(operator, another.operator);
  }

  /**
   * Computes a hash code from attributes: {@code label}, {@code operator}.
   * @return hashCode value
   */
  @Override
  public int hashCode() {
    int h = 5381;
    h += (h << 5) + Objects.hashCode(label);
    h += (h << 5) + Objects.hashCode(operator);
    return h;
  }

  /**
   * Prints the immutable value {@code Op} with attribute values.
   * @return A string representation of the value
   */
  @Override
  public String toString() {
    return "Op{"
        + "label=" + label
        + ", operator=" + operator
        + "}";
  }

  /**
   * Construct a new immutable {@code Op} instance.
   * @param label The value for the {@code label} attribute
   * @param operator The value for the {@code operator} attribute
   * @return An immutable Op instance
   */
  public static ImmutableOp of(String label, String operator) {
    return validate(new ImmutableOp(label, operator));
  }

  private static final Validator validator = Validation.buildDefaultValidatorFactory().getValidator();

  private static ImmutableOp validate(ImmutableOp instance) {
    Set<ConstraintViolation<ImmutableOp>> constraintViolations = validator.validate(instance);
    if (!constraintViolations.isEmpty()) {
      throw new ConstraintViolationException(constraintViolations);
    }
    return instance;
  }

  /**
   * Creates an immutable copy of a {@link Op} value.
   * Uses accessors to get values to initialize the new immutable instance.
   * If an instance is already immutable, it is returned as is.
   * @param instance The instance to copy
   * @return A copied immutable Op instance
   */
  public static ImmutableOp copyOf(Op instance) {
    if (instance instanceof ImmutableOp) {
      return (ImmutableOp) instance;
    }
    return new Op.Builder()
        .createFrom(instance)
        .build();
  }

  private Object readResolve() throws ObjectStreamException {
    return validate(this);
  }

  /**
   * Builds instances of type {@link ImmutableOp ImmutableOp}.
   * Initialize attributes and then invoke the {@link #build()} method to create an
   * immutable instance.
   * <p><em>{@code Builder} is not thread-safe and generally should not be stored in a field or collection,
   * but instead used immediately to create instances.</em>
   */
  static class Builder {
    private String label;
    private String operator;

    /**
     * Creates a builder for {@link ImmutableOp ImmutableOp} instances.
     */
    Builder() {
      if (!(this instanceof Op.Builder)) {
        throw new UnsupportedOperationException("Use: new Op.Builder()");
      }
    }

    /**
     * Fill a builder with attribute values from the provided {@code Op} instance.
     * Regular attribute values will be replaced with those from the given instance.
     * Absent optional values will not replace present values.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    public final Op.Builder createFrom(Op instance) {
      Objects.requireNonNull(instance, "instance");
      String labelValue = instance.getLabel();
      if (labelValue != null) {
        label(labelValue);
      }
      String operatorValue = instance.getOperator();
      if (operatorValue != null) {
        operator(operatorValue);
      }
      return (Op.Builder) this;
    }

    /**
     * Initializes the value for the {@link Op#getLabel() label} attribute.
     * @param label The value for label (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("label")
    public final Op.Builder label(String label) {
      this.label = label;
      return (Op.Builder) this;
    }

    /**
     * Initializes the value for the {@link Op#getOperator() operator} attribute.
     * @param operator The value for operator (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("operator")
    public final Op.Builder operator(String operator) {
      this.operator = operator;
      return (Op.Builder) this;
    }

    /**
     * Builds a new {@link ImmutableOp ImmutableOp}.
     * @return An immutable instance of Op
     * @throws java.lang.IllegalStateException if any required attributes are missing
     */
    public ImmutableOp build() {
      return ImmutableOp.validate(new ImmutableOp(label, operator));
    }
  }
}
