/**
 * Copyright (C) 2016 Red Hat, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package io.syndesis.model.integration;

import com.fasterxml.jackson.annotation.JsonProperty;
import edu.umd.cs.findbugs.annotations.SuppressFBWarnings;
import io.syndesis.model.WithName;
import io.syndesis.model.WithTags;
import io.syndesis.model.connection.Connection;
import io.syndesis.model.user.User;
import java.io.ObjectStreamException;
import java.math.BigInteger;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.Date;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.NavigableSet;
import java.util.Objects;
import java.util.Optional;
import java.util.Set;
import java.util.SortedSet;
import java.util.TreeSet;
import javax.annotation.Generated;
import javax.validation.ConstraintViolation;
import javax.validation.ConstraintViolationException;
import javax.validation.Validation;
import javax.validation.Validator;

/**
 * Immutable implementation of {@link Integration}.
 * <p>
 * Use the builder to create immutable instances:
 * {@code new Integration.Builder()}.
 * Use the static factory method to create immutable instances:
 * {@code ImmutableIntegration.of()}.
 */
@SuppressWarnings({"all"})
@SuppressFBWarnings
@Generated({"Immutables.generator", "Integration"})
final class ImmutableIntegration implements Integration {
  private final Set<IntegrationRevision> revisions;
  private final IntegrationRevision draftRevision;
  private final Integer deployedRevisionId;
  private final String configuration;
  private final String integrationTemplateId;
  private final String userId;
  private final List<User> users;
  private final List<Connection> connections;
  private final List<? extends Step> steps;
  private final String description;
  private final Integration.Status desiredStatus;
  private final Integration.Status currentStatus;
  private final List<String> stepsDone;
  private final String statusMessage;
  private final Date lastUpdated;
  private final Date createdDate;
  private final BigInteger timesUsed;
  private final String id;
  private final SortedSet<String> tags;
  private final String name;

  private ImmutableIntegration(
      Iterable<? extends IntegrationRevision> revisions,
      Optional<IntegrationRevision> draftRevision,
      Optional<Integer> deployedRevisionId,
      Optional<String> configuration,
      Optional<String> integrationTemplateId,
      Optional<String> userId,
      Iterable<? extends User> users,
      Iterable<? extends Connection> connections,
      List<? extends Step> steps,
      Optional<String> description,
      Optional<Integration.Status> desiredStatus,
      Optional<Integration.Status> currentStatus,
      Iterable<String> stepsDone,
      Optional<String> statusMessage,
      Optional<Date> lastUpdated,
      Optional<Date> createdDate,
      Optional<BigInteger> timesUsed,
      Optional<String> id,
      Iterable<String> tags,
      String name) {
    this.revisions = createUnmodifiableSet(createSafeList(revisions, true, false));
    this.draftRevision = draftRevision.orElse(null);
    this.deployedRevisionId = deployedRevisionId.orElse(null);
    this.configuration = configuration.orElse(null);
    this.integrationTemplateId = integrationTemplateId.orElse(null);
    this.userId = userId.orElse(null);
    this.users = createUnmodifiableList(false, createSafeList(users, true, false));
    this.connections = createUnmodifiableList(false, createSafeList(connections, true, false));
    this.steps = Objects.requireNonNull(steps, "steps");
    this.description = description.orElse(null);
    this.desiredStatus = desiredStatus.orElse(null);
    this.currentStatus = currentStatus.orElse(null);
    this.stepsDone = createUnmodifiableList(false, createSafeList(stepsDone, true, false));
    this.statusMessage = statusMessage.orElse(null);
    this.lastUpdated = lastUpdated.orElse(null);
    this.createdDate = createdDate.orElse(null);
    this.timesUsed = timesUsed.orElse(null);
    this.id = id.orElse(null);
    this.tags = createUnmodifiableSortedSet(false, createSafeList(tags, false, true));
    this.name = name;
    initShim.connections(this.connections);
    initShim.steps(this.steps);
    initShim.stepsDone(this.stepsDone);
    this.initShim = null;
  }

  private ImmutableIntegration(ImmutableIntegration.Builder builder) {
    this.revisions = createUnmodifiableSet(builder.revisions);
    this.draftRevision = builder.draftRevision;
    this.deployedRevisionId = builder.deployedRevisionId;
    this.configuration = builder.configuration;
    this.integrationTemplateId = builder.integrationTemplateId;
    this.userId = builder.userId;
    this.users = createUnmodifiableList(true, builder.users);
    this.description = builder.description;
    this.desiredStatus = builder.desiredStatus;
    this.currentStatus = builder.currentStatus;
    this.statusMessage = builder.statusMessage;
    this.lastUpdated = builder.lastUpdated;
    this.createdDate = builder.createdDate;
    this.timesUsed = builder.timesUsed;
    this.id = builder.id;
    this.tags = createUnmodifiableSortedSet(false, createSafeList(builder.tags, false, false));
    this.name = builder.name;
    if (builder.connectionsIsSet()) {
      initShim.connections(createUnmodifiableList(true, builder.connections));
    }
    if (builder.steps != null) {
      initShim.steps(builder.steps);
    }
    if (builder.stepsDoneIsSet()) {
      initShim.stepsDone(createUnmodifiableList(true, builder.stepsDone));
    }
    this.connections = initShim.getConnections();
    this.steps = initShim.getSteps();
    this.stepsDone = initShim.getStepsDone();
    this.initShim = null;
  }

  private ImmutableIntegration(
      ImmutableIntegration original,
      Set<IntegrationRevision> revisions,
      IntegrationRevision draftRevision,
      Integer deployedRevisionId,
      String configuration,
      String integrationTemplateId,
      String userId,
      List<User> users,
      List<Connection> connections,
      List<? extends Step> steps,
      String description,
      Integration.Status desiredStatus,
      Integration.Status currentStatus,
      List<String> stepsDone,
      String statusMessage,
      Date lastUpdated,
      Date createdDate,
      BigInteger timesUsed,
      String id,
      SortedSet<String> tags,
      String name) {
    this.revisions = revisions;
    this.draftRevision = draftRevision;
    this.deployedRevisionId = deployedRevisionId;
    this.configuration = configuration;
    this.integrationTemplateId = integrationTemplateId;
    this.userId = userId;
    this.users = users;
    this.connections = connections;
    this.steps = steps;
    this.description = description;
    this.desiredStatus = desiredStatus;
    this.currentStatus = currentStatus;
    this.stepsDone = stepsDone;
    this.statusMessage = statusMessage;
    this.lastUpdated = lastUpdated;
    this.createdDate = createdDate;
    this.timesUsed = timesUsed;
    this.id = id;
    this.tags = tags;
    this.name = name;
    this.initShim = null;
  }

  private static final int STAGE_INITIALIZING = -1;
  private static final int STAGE_UNINITIALIZED = 0;
  private static final int STAGE_INITIALIZED = 1;
  private transient volatile InitShim initShim = new InitShim();

  private final class InitShim {
    private List<Connection> connections;
    private int connectionsBuildStage;

    List<Connection> getConnections() {
      if (connectionsBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (connectionsBuildStage == STAGE_UNINITIALIZED) {
        connectionsBuildStage = STAGE_INITIALIZING;
        this.connections = createUnmodifiableList(false, createSafeList(getConnectionsInitialize(), true, false));
        connectionsBuildStage = STAGE_INITIALIZED;
      }
      return this.connections;
    }

    void connections(List<Connection> connections) {
      this.connections = connections;
      connectionsBuildStage = STAGE_INITIALIZED;
    }
    private List<? extends Step> steps;
    private int stepsBuildStage;

    List<? extends Step> getSteps() {
      if (stepsBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (stepsBuildStage == STAGE_UNINITIALIZED) {
        stepsBuildStage = STAGE_INITIALIZING;
        this.steps = Objects.requireNonNull(getStepsInitialize(), "steps");
        stepsBuildStage = STAGE_INITIALIZED;
      }
      return this.steps;
    }

    void steps(List<? extends Step> steps) {
      this.steps = steps;
      stepsBuildStage = STAGE_INITIALIZED;
    }
    private List<String> stepsDone;
    private int stepsDoneBuildStage;

    List<String> getStepsDone() {
      if (stepsDoneBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (stepsDoneBuildStage == STAGE_UNINITIALIZED) {
        stepsDoneBuildStage = STAGE_INITIALIZING;
        this.stepsDone = createUnmodifiableList(false, createSafeList(getStepsDoneInitialize(), true, false));
        stepsDoneBuildStage = STAGE_INITIALIZED;
      }
      return this.stepsDone;
    }

    void stepsDone(List<String> stepsDone) {
      this.stepsDone = stepsDone;
      stepsDoneBuildStage = STAGE_INITIALIZED;
    }

    private String formatInitCycleMessage() {
      ArrayList<String> attributes = new ArrayList<String>();
      if (connectionsBuildStage == STAGE_INITIALIZING) attributes.add("connections");
      if (stepsBuildStage == STAGE_INITIALIZING) attributes.add("steps");
      if (stepsDoneBuildStage == STAGE_INITIALIZING) attributes.add("stepsDone");
      return "Cannot build Integration, attribute initializers form cycle" + attributes;
    }
  }

  private List<Connection> getConnectionsInitialize() {
    return Integration.super.getConnections();
  }

  private List<? extends Step> getStepsInitialize() {
    return Integration.super.getSteps();
  }

  private List<String> getStepsDoneInitialize() {
    return Integration.super.getStepsDone();
  }

  /**
   * @return The value of the {@code revisions} attribute
   */
  @JsonProperty("revisions")
  @Override
  public Set<IntegrationRevision> getRevisions() {
    return revisions;
  }

  /**
   * @return The value of the {@code draftRevision} attribute
   */
  @JsonProperty("draftRevision")
  @Override
  public Optional<IntegrationRevision> getDraftRevision() {
    return Optional.ofNullable(draftRevision);
  }

  /**
   * @return The value of the {@code deployedRevisionId} attribute
   */
  @JsonProperty("deployedRevisionId")
  @Override
  public Optional<Integer> getDeployedRevisionId() {
    return Optional.ofNullable(deployedRevisionId);
  }

  /**
   * @return The value of the {@code configuration} attribute
   */
  @JsonProperty("configuration")
  @Override
  public Optional<String> getConfiguration() {
    return Optional.ofNullable(configuration);
  }

  /**
   * @return The value of the {@code integrationTemplateId} attribute
   */
  @JsonProperty("integrationTemplateId")
  @Override
  public Optional<String> getIntegrationTemplateId() {
    return Optional.ofNullable(integrationTemplateId);
  }

  /**
   * @return The value of the {@code userId} attribute
   */
  @JsonProperty("userId")
  @Override
  public Optional<String> getUserId() {
    return Optional.ofNullable(userId);
  }

  /**
   * @return The value of the {@code users} attribute
   */
  @JsonProperty("users")
  @Override
  public List<User> getUsers() {
    return users;
  }

  /**
   * @return The value of the {@code connections} attribute
   */
  @JsonProperty("connections")
  @Override
  public List<Connection> getConnections() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getConnections()
        : this.connections;
  }

  /**
   * @return The value of the {@code steps} attribute
   */
  @JsonProperty("steps")
  @Override
  public List<? extends Step> getSteps() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getSteps()
        : this.steps;
  }

  /**
   * @return The value of the {@code description} attribute
   */
  @JsonProperty("description")
  @Override
  public Optional<String> getDescription() {
    return Optional.ofNullable(description);
  }

  /**
   * @return The value of the {@code desiredStatus} attribute
   */
  @JsonProperty("desiredStatus")
  @Override
  public Optional<Integration.Status> getDesiredStatus() {
    return Optional.ofNullable(desiredStatus);
  }

  /**
   * @return The value of the {@code currentStatus} attribute
   */
  @JsonProperty("currentStatus")
  @Override
  public Optional<Integration.Status> getCurrentStatus() {
    return Optional.ofNullable(currentStatus);
  }

  /**
   * @return The value of the {@code stepsDone} attribute
   */
  @JsonProperty("stepsDone")
  @Override
  public List<String> getStepsDone() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getStepsDone()
        : this.stepsDone;
  }

  /**
   * @return The value of the {@code statusMessage} attribute
   */
  @JsonProperty("statusMessage")
  @Override
  public Optional<String> getStatusMessage() {
    return Optional.ofNullable(statusMessage);
  }

  /**
   * @return The value of the {@code lastUpdated} attribute
   */
  @JsonProperty("lastUpdated")
  @Override
  public Optional<Date> getLastUpdated() {
    return Optional.ofNullable(lastUpdated);
  }

  /**
   * @return The value of the {@code createdDate} attribute
   */
  @JsonProperty("createdDate")
  @Override
  public Optional<Date> getCreatedDate() {
    return Optional.ofNullable(createdDate);
  }

  /**
   * @return The value of the {@code timesUsed} attribute
   */
  @JsonProperty("timesUsed")
  @Override
  public Optional<BigInteger> getTimesUsed() {
    return Optional.ofNullable(timesUsed);
  }

  /**
   * @return The value of the {@code id} attribute
   */
  @JsonProperty("id")
  @Override
  public Optional<String> getId() {
    return Optional.ofNullable(id);
  }

  /**
   * @return The value of the {@code tags} attribute
   */
  @JsonProperty("tags")
  @Override
  public SortedSet<String> getTags() {
    return tags;
  }

  /**
   * @return The value of the {@code name} attribute
   */
  @JsonProperty("name")
  @Override
  public String getName() {
    return name;
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link Integration#getRevisions() revisions}.
   * @param elements The elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegration withRevisions(IntegrationRevision... elements) {
    Set<IntegrationRevision> newValue = createUnmodifiableSet(createSafeList(Arrays.asList(elements), true, false));
    return validate(new ImmutableIntegration(
        this,
        newValue,
        this.draftRevision,
        this.deployedRevisionId,
        this.configuration,
        this.integrationTemplateId,
        this.userId,
        this.users,
        this.connections,
        this.steps,
        this.description,
        this.desiredStatus,
        this.currentStatus,
        this.stepsDone,
        this.statusMessage,
        this.lastUpdated,
        this.createdDate,
        this.timesUsed,
        this.id,
        this.tags,
        this.name));
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link Integration#getRevisions() revisions}.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param elements An iterable of revisions elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegration withRevisions(Iterable<? extends IntegrationRevision> elements) {
    if (this.revisions == elements) return this;
    Set<IntegrationRevision> newValue = createUnmodifiableSet(createSafeList(elements, true, false));
    return validate(new ImmutableIntegration(
        this,
        newValue,
        this.draftRevision,
        this.deployedRevisionId,
        this.configuration,
        this.integrationTemplateId,
        this.userId,
        this.users,
        this.connections,
        this.steps,
        this.description,
        this.desiredStatus,
        this.currentStatus,
        this.stepsDone,
        this.statusMessage,
        this.lastUpdated,
        this.createdDate,
        this.timesUsed,
        this.id,
        this.tags,
        this.name));
  }

  /**
   * Copy the current immutable object by setting a <i>present</i> value for the optional {@link Integration#getDraftRevision() draftRevision} attribute.
   * @param value The value for draftRevision
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegration withDraftRevision(IntegrationRevision value) {
    IntegrationRevision newValue = Objects.requireNonNull(value, "draftRevision");
    if (this.draftRevision == newValue) return this;
    return validate(new ImmutableIntegration(
        this,
        this.revisions,
        newValue,
        this.deployedRevisionId,
        this.configuration,
        this.integrationTemplateId,
        this.userId,
        this.users,
        this.connections,
        this.steps,
        this.description,
        this.desiredStatus,
        this.currentStatus,
        this.stepsDone,
        this.statusMessage,
        this.lastUpdated,
        this.createdDate,
        this.timesUsed,
        this.id,
        this.tags,
        this.name));
  }

  /**
   * Copy the current immutable object by setting an optional value for the {@link Integration#getDraftRevision() draftRevision} attribute.
   * A shallow reference equality check is used on unboxed optional value to prevent copying of the same value by returning {@code this}.
   * @param optional A value for draftRevision
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegration withDraftRevision(Optional<? extends IntegrationRevision> optional) {
    IntegrationRevision value = optional.orElse(null);
    if (this.draftRevision == value) return this;
    return validate(new ImmutableIntegration(
        this,
        this.revisions,
        value,
        this.deployedRevisionId,
        this.configuration,
        this.integrationTemplateId,
        this.userId,
        this.users,
        this.connections,
        this.steps,
        this.description,
        this.desiredStatus,
        this.currentStatus,
        this.stepsDone,
        this.statusMessage,
        this.lastUpdated,
        this.createdDate,
        this.timesUsed,
        this.id,
        this.tags,
        this.name));
  }

  /**
   * Copy the current immutable object by setting a <i>present</i> value for the optional {@link Integration#getDeployedRevisionId() deployedRevisionId} attribute.
   * @param value The value for deployedRevisionId
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegration withDeployedRevisionId(int value) {
    Integer newValue = value;
    if (Objects.equals(this.deployedRevisionId, newValue)) return this;
    return validate(new ImmutableIntegration(
        this,
        this.revisions,
        this.draftRevision,
        newValue,
        this.configuration,
        this.integrationTemplateId,
        this.userId,
        this.users,
        this.connections,
        this.steps,
        this.description,
        this.desiredStatus,
        this.currentStatus,
        this.stepsDone,
        this.statusMessage,
        this.lastUpdated,
        this.createdDate,
        this.timesUsed,
        this.id,
        this.tags,
        this.name));
  }

  /**
   * Copy the current immutable object by setting an optional value for the {@link Integration#getDeployedRevisionId() deployedRevisionId} attribute.
   * An equality check is used on inner nullable value to prevent copying of the same value by returning {@code this}.
   * @param optional A value for deployedRevisionId
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegration withDeployedRevisionId(Optional<Integer> optional) {
    Integer value = optional.orElse(null);
    if (Objects.equals(this.deployedRevisionId, value)) return this;
    return validate(new ImmutableIntegration(
        this,
        this.revisions,
        this.draftRevision,
        value,
        this.configuration,
        this.integrationTemplateId,
        this.userId,
        this.users,
        this.connections,
        this.steps,
        this.description,
        this.desiredStatus,
        this.currentStatus,
        this.stepsDone,
        this.statusMessage,
        this.lastUpdated,
        this.createdDate,
        this.timesUsed,
        this.id,
        this.tags,
        this.name));
  }

  /**
   * Copy the current immutable object by setting a <i>present</i> value for the optional {@link Integration#getConfiguration() configuration} attribute.
   * @param value The value for configuration
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegration withConfiguration(String value) {
    String newValue = Objects.requireNonNull(value, "configuration");
    if (Objects.equals(this.configuration, newValue)) return this;
    return validate(new ImmutableIntegration(
        this,
        this.revisions,
        this.draftRevision,
        this.deployedRevisionId,
        newValue,
        this.integrationTemplateId,
        this.userId,
        this.users,
        this.connections,
        this.steps,
        this.description,
        this.desiredStatus,
        this.currentStatus,
        this.stepsDone,
        this.statusMessage,
        this.lastUpdated,
        this.createdDate,
        this.timesUsed,
        this.id,
        this.tags,
        this.name));
  }

  /**
   * Copy the current immutable object by setting an optional value for the {@link Integration#getConfiguration() configuration} attribute.
   * An equality check is used on inner nullable value to prevent copying of the same value by returning {@code this}.
   * @param optional A value for configuration
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegration withConfiguration(Optional<String> optional) {
    String value = optional.orElse(null);
    if (Objects.equals(this.configuration, value)) return this;
    return validate(new ImmutableIntegration(
        this,
        this.revisions,
        this.draftRevision,
        this.deployedRevisionId,
        value,
        this.integrationTemplateId,
        this.userId,
        this.users,
        this.connections,
        this.steps,
        this.description,
        this.desiredStatus,
        this.currentStatus,
        this.stepsDone,
        this.statusMessage,
        this.lastUpdated,
        this.createdDate,
        this.timesUsed,
        this.id,
        this.tags,
        this.name));
  }

  /**
   * Copy the current immutable object by setting a <i>present</i> value for the optional {@link Integration#getIntegrationTemplateId() integrationTemplateId} attribute.
   * @param value The value for integrationTemplateId
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegration withIntegrationTemplateId(String value) {
    String newValue = Objects.requireNonNull(value, "integrationTemplateId");
    if (Objects.equals(this.integrationTemplateId, newValue)) return this;
    return validate(new ImmutableIntegration(
        this,
        this.revisions,
        this.draftRevision,
        this.deployedRevisionId,
        this.configuration,
        newValue,
        this.userId,
        this.users,
        this.connections,
        this.steps,
        this.description,
        this.desiredStatus,
        this.currentStatus,
        this.stepsDone,
        this.statusMessage,
        this.lastUpdated,
        this.createdDate,
        this.timesUsed,
        this.id,
        this.tags,
        this.name));
  }

  /**
   * Copy the current immutable object by setting an optional value for the {@link Integration#getIntegrationTemplateId() integrationTemplateId} attribute.
   * An equality check is used on inner nullable value to prevent copying of the same value by returning {@code this}.
   * @param optional A value for integrationTemplateId
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegration withIntegrationTemplateId(Optional<String> optional) {
    String value = optional.orElse(null);
    if (Objects.equals(this.integrationTemplateId, value)) return this;
    return validate(new ImmutableIntegration(
        this,
        this.revisions,
        this.draftRevision,
        this.deployedRevisionId,
        this.configuration,
        value,
        this.userId,
        this.users,
        this.connections,
        this.steps,
        this.description,
        this.desiredStatus,
        this.currentStatus,
        this.stepsDone,
        this.statusMessage,
        this.lastUpdated,
        this.createdDate,
        this.timesUsed,
        this.id,
        this.tags,
        this.name));
  }

  /**
   * Copy the current immutable object by setting a <i>present</i> value for the optional {@link Integration#getUserId() userId} attribute.
   * @param value The value for userId
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegration withUserId(String value) {
    String newValue = Objects.requireNonNull(value, "userId");
    if (Objects.equals(this.userId, newValue)) return this;
    return validate(new ImmutableIntegration(
        this,
        this.revisions,
        this.draftRevision,
        this.deployedRevisionId,
        this.configuration,
        this.integrationTemplateId,
        newValue,
        this.users,
        this.connections,
        this.steps,
        this.description,
        this.desiredStatus,
        this.currentStatus,
        this.stepsDone,
        this.statusMessage,
        this.lastUpdated,
        this.createdDate,
        this.timesUsed,
        this.id,
        this.tags,
        this.name));
  }

  /**
   * Copy the current immutable object by setting an optional value for the {@link Integration#getUserId() userId} attribute.
   * An equality check is used on inner nullable value to prevent copying of the same value by returning {@code this}.
   * @param optional A value for userId
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegration withUserId(Optional<String> optional) {
    String value = optional.orElse(null);
    if (Objects.equals(this.userId, value)) return this;
    return validate(new ImmutableIntegration(
        this,
        this.revisions,
        this.draftRevision,
        this.deployedRevisionId,
        this.configuration,
        this.integrationTemplateId,
        value,
        this.users,
        this.connections,
        this.steps,
        this.description,
        this.desiredStatus,
        this.currentStatus,
        this.stepsDone,
        this.statusMessage,
        this.lastUpdated,
        this.createdDate,
        this.timesUsed,
        this.id,
        this.tags,
        this.name));
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link Integration#getUsers() users}.
   * @param elements The elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegration withUsers(User... elements) {
    List<User> newValue = createUnmodifiableList(false, createSafeList(Arrays.asList(elements), true, false));
    return validate(new ImmutableIntegration(
        this,
        this.revisions,
        this.draftRevision,
        this.deployedRevisionId,
        this.configuration,
        this.integrationTemplateId,
        this.userId,
        newValue,
        this.connections,
        this.steps,
        this.description,
        this.desiredStatus,
        this.currentStatus,
        this.stepsDone,
        this.statusMessage,
        this.lastUpdated,
        this.createdDate,
        this.timesUsed,
        this.id,
        this.tags,
        this.name));
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link Integration#getUsers() users}.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param elements An iterable of users elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegration withUsers(Iterable<? extends User> elements) {
    if (this.users == elements) return this;
    List<User> newValue = createUnmodifiableList(false, createSafeList(elements, true, false));
    return validate(new ImmutableIntegration(
        this,
        this.revisions,
        this.draftRevision,
        this.deployedRevisionId,
        this.configuration,
        this.integrationTemplateId,
        this.userId,
        newValue,
        this.connections,
        this.steps,
        this.description,
        this.desiredStatus,
        this.currentStatus,
        this.stepsDone,
        this.statusMessage,
        this.lastUpdated,
        this.createdDate,
        this.timesUsed,
        this.id,
        this.tags,
        this.name));
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link Integration#getConnections() connections}.
   * @param elements The elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegration withConnections(Connection... elements) {
    List<Connection> newValue = createUnmodifiableList(false, createSafeList(Arrays.asList(elements), true, false));
    return validate(new ImmutableIntegration(
        this,
        this.revisions,
        this.draftRevision,
        this.deployedRevisionId,
        this.configuration,
        this.integrationTemplateId,
        this.userId,
        this.users,
        newValue,
        this.steps,
        this.description,
        this.desiredStatus,
        this.currentStatus,
        this.stepsDone,
        this.statusMessage,
        this.lastUpdated,
        this.createdDate,
        this.timesUsed,
        this.id,
        this.tags,
        this.name));
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link Integration#getConnections() connections}.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param elements An iterable of connections elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegration withConnections(Iterable<? extends Connection> elements) {
    if (this.connections == elements) return this;
    List<Connection> newValue = createUnmodifiableList(false, createSafeList(elements, true, false));
    return validate(new ImmutableIntegration(
        this,
        this.revisions,
        this.draftRevision,
        this.deployedRevisionId,
        this.configuration,
        this.integrationTemplateId,
        this.userId,
        this.users,
        newValue,
        this.steps,
        this.description,
        this.desiredStatus,
        this.currentStatus,
        this.stepsDone,
        this.statusMessage,
        this.lastUpdated,
        this.createdDate,
        this.timesUsed,
        this.id,
        this.tags,
        this.name));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link Integration#getSteps() steps} attribute.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for steps
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIntegration withSteps(List<? extends Step> value) {
    if (this.steps == value) return this;
    List<? extends Step> newValue = Objects.requireNonNull(value, "steps");
    return validate(new ImmutableIntegration(
        this,
        this.revisions,
        this.draftRevision,
        this.deployedRevisionId,
        this.configuration,
        this.integrationTemplateId,
        this.userId,
        this.users,
        this.connections,
        newValue,
        this.description,
        this.desiredStatus,
        this.currentStatus,
        this.stepsDone,
        this.statusMessage,
        this.lastUpdated,
        this.createdDate,
        this.timesUsed,
        this.id,
        this.tags,
        this.name));
  }

  /**
   * Copy the current immutable object by setting a <i>present</i> value for the optional {@link Integration#getDescription() description} attribute.
   * @param value The value for description
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegration withDescription(String value) {
    String newValue = Objects.requireNonNull(value, "description");
    if (Objects.equals(this.description, newValue)) return this;
    return validate(new ImmutableIntegration(
        this,
        this.revisions,
        this.draftRevision,
        this.deployedRevisionId,
        this.configuration,
        this.integrationTemplateId,
        this.userId,
        this.users,
        this.connections,
        this.steps,
        newValue,
        this.desiredStatus,
        this.currentStatus,
        this.stepsDone,
        this.statusMessage,
        this.lastUpdated,
        this.createdDate,
        this.timesUsed,
        this.id,
        this.tags,
        this.name));
  }

  /**
   * Copy the current immutable object by setting an optional value for the {@link Integration#getDescription() description} attribute.
   * An equality check is used on inner nullable value to prevent copying of the same value by returning {@code this}.
   * @param optional A value for description
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegration withDescription(Optional<String> optional) {
    String value = optional.orElse(null);
    if (Objects.equals(this.description, value)) return this;
    return validate(new ImmutableIntegration(
        this,
        this.revisions,
        this.draftRevision,
        this.deployedRevisionId,
        this.configuration,
        this.integrationTemplateId,
        this.userId,
        this.users,
        this.connections,
        this.steps,
        value,
        this.desiredStatus,
        this.currentStatus,
        this.stepsDone,
        this.statusMessage,
        this.lastUpdated,
        this.createdDate,
        this.timesUsed,
        this.id,
        this.tags,
        this.name));
  }

  /**
   * Copy the current immutable object by setting a <i>present</i> value for the optional {@link Integration#getDesiredStatus() desiredStatus} attribute.
   * @param value The value for desiredStatus
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegration withDesiredStatus(Integration.Status value) {
    Integration.Status newValue = Objects.requireNonNull(value, "desiredStatus");
    if (this.desiredStatus == newValue) return this;
    return validate(new ImmutableIntegration(
        this,
        this.revisions,
        this.draftRevision,
        this.deployedRevisionId,
        this.configuration,
        this.integrationTemplateId,
        this.userId,
        this.users,
        this.connections,
        this.steps,
        this.description,
        newValue,
        this.currentStatus,
        this.stepsDone,
        this.statusMessage,
        this.lastUpdated,
        this.createdDate,
        this.timesUsed,
        this.id,
        this.tags,
        this.name));
  }

  /**
   * Copy the current immutable object by setting an optional value for the {@link Integration#getDesiredStatus() desiredStatus} attribute.
   * An equality check is used on inner nullable value to prevent copying of the same value by returning {@code this}.
   * @param optional A value for desiredStatus
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegration withDesiredStatus(Optional<? extends Integration.Status> optional) {
    Integration.Status value = optional.orElse(null);
    if (Objects.equals(this.desiredStatus, value)) return this;
    return validate(new ImmutableIntegration(
        this,
        this.revisions,
        this.draftRevision,
        this.deployedRevisionId,
        this.configuration,
        this.integrationTemplateId,
        this.userId,
        this.users,
        this.connections,
        this.steps,
        this.description,
        value,
        this.currentStatus,
        this.stepsDone,
        this.statusMessage,
        this.lastUpdated,
        this.createdDate,
        this.timesUsed,
        this.id,
        this.tags,
        this.name));
  }

  /**
   * Copy the current immutable object by setting a <i>present</i> value for the optional {@link Integration#getCurrentStatus() currentStatus} attribute.
   * @param value The value for currentStatus
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegration withCurrentStatus(Integration.Status value) {
    Integration.Status newValue = Objects.requireNonNull(value, "currentStatus");
    if (this.currentStatus == newValue) return this;
    return validate(new ImmutableIntegration(
        this,
        this.revisions,
        this.draftRevision,
        this.deployedRevisionId,
        this.configuration,
        this.integrationTemplateId,
        this.userId,
        this.users,
        this.connections,
        this.steps,
        this.description,
        this.desiredStatus,
        newValue,
        this.stepsDone,
        this.statusMessage,
        this.lastUpdated,
        this.createdDate,
        this.timesUsed,
        this.id,
        this.tags,
        this.name));
  }

  /**
   * Copy the current immutable object by setting an optional value for the {@link Integration#getCurrentStatus() currentStatus} attribute.
   * An equality check is used on inner nullable value to prevent copying of the same value by returning {@code this}.
   * @param optional A value for currentStatus
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegration withCurrentStatus(Optional<? extends Integration.Status> optional) {
    Integration.Status value = optional.orElse(null);
    if (Objects.equals(this.currentStatus, value)) return this;
    return validate(new ImmutableIntegration(
        this,
        this.revisions,
        this.draftRevision,
        this.deployedRevisionId,
        this.configuration,
        this.integrationTemplateId,
        this.userId,
        this.users,
        this.connections,
        this.steps,
        this.description,
        this.desiredStatus,
        value,
        this.stepsDone,
        this.statusMessage,
        this.lastUpdated,
        this.createdDate,
        this.timesUsed,
        this.id,
        this.tags,
        this.name));
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link Integration#getStepsDone() stepsDone}.
   * @param elements The elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegration withStepsDone(String... elements) {
    List<String> newValue = createUnmodifiableList(false, createSafeList(Arrays.asList(elements), true, false));
    return validate(new ImmutableIntegration(
        this,
        this.revisions,
        this.draftRevision,
        this.deployedRevisionId,
        this.configuration,
        this.integrationTemplateId,
        this.userId,
        this.users,
        this.connections,
        this.steps,
        this.description,
        this.desiredStatus,
        this.currentStatus,
        newValue,
        this.statusMessage,
        this.lastUpdated,
        this.createdDate,
        this.timesUsed,
        this.id,
        this.tags,
        this.name));
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link Integration#getStepsDone() stepsDone}.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param elements An iterable of stepsDone elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegration withStepsDone(Iterable<String> elements) {
    if (this.stepsDone == elements) return this;
    List<String> newValue = createUnmodifiableList(false, createSafeList(elements, true, false));
    return validate(new ImmutableIntegration(
        this,
        this.revisions,
        this.draftRevision,
        this.deployedRevisionId,
        this.configuration,
        this.integrationTemplateId,
        this.userId,
        this.users,
        this.connections,
        this.steps,
        this.description,
        this.desiredStatus,
        this.currentStatus,
        newValue,
        this.statusMessage,
        this.lastUpdated,
        this.createdDate,
        this.timesUsed,
        this.id,
        this.tags,
        this.name));
  }

  /**
   * Copy the current immutable object by setting a <i>present</i> value for the optional {@link Integration#getStatusMessage() statusMessage} attribute.
   * @param value The value for statusMessage
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegration withStatusMessage(String value) {
    String newValue = Objects.requireNonNull(value, "statusMessage");
    if (Objects.equals(this.statusMessage, newValue)) return this;
    return validate(new ImmutableIntegration(
        this,
        this.revisions,
        this.draftRevision,
        this.deployedRevisionId,
        this.configuration,
        this.integrationTemplateId,
        this.userId,
        this.users,
        this.connections,
        this.steps,
        this.description,
        this.desiredStatus,
        this.currentStatus,
        this.stepsDone,
        newValue,
        this.lastUpdated,
        this.createdDate,
        this.timesUsed,
        this.id,
        this.tags,
        this.name));
  }

  /**
   * Copy the current immutable object by setting an optional value for the {@link Integration#getStatusMessage() statusMessage} attribute.
   * An equality check is used on inner nullable value to prevent copying of the same value by returning {@code this}.
   * @param optional A value for statusMessage
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegration withStatusMessage(Optional<String> optional) {
    String value = optional.orElse(null);
    if (Objects.equals(this.statusMessage, value)) return this;
    return validate(new ImmutableIntegration(
        this,
        this.revisions,
        this.draftRevision,
        this.deployedRevisionId,
        this.configuration,
        this.integrationTemplateId,
        this.userId,
        this.users,
        this.connections,
        this.steps,
        this.description,
        this.desiredStatus,
        this.currentStatus,
        this.stepsDone,
        value,
        this.lastUpdated,
        this.createdDate,
        this.timesUsed,
        this.id,
        this.tags,
        this.name));
  }

  /**
   * Copy the current immutable object by setting a <i>present</i> value for the optional {@link Integration#getLastUpdated() lastUpdated} attribute.
   * @param value The value for lastUpdated
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegration withLastUpdated(Date value) {
    Date newValue = Objects.requireNonNull(value, "lastUpdated");
    if (this.lastUpdated == newValue) return this;
    return validate(new ImmutableIntegration(
        this,
        this.revisions,
        this.draftRevision,
        this.deployedRevisionId,
        this.configuration,
        this.integrationTemplateId,
        this.userId,
        this.users,
        this.connections,
        this.steps,
        this.description,
        this.desiredStatus,
        this.currentStatus,
        this.stepsDone,
        this.statusMessage,
        newValue,
        this.createdDate,
        this.timesUsed,
        this.id,
        this.tags,
        this.name));
  }

  /**
   * Copy the current immutable object by setting an optional value for the {@link Integration#getLastUpdated() lastUpdated} attribute.
   * A shallow reference equality check is used on unboxed optional value to prevent copying of the same value by returning {@code this}.
   * @param optional A value for lastUpdated
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegration withLastUpdated(Optional<? extends Date> optional) {
    Date value = optional.orElse(null);
    if (this.lastUpdated == value) return this;
    return validate(new ImmutableIntegration(
        this,
        this.revisions,
        this.draftRevision,
        this.deployedRevisionId,
        this.configuration,
        this.integrationTemplateId,
        this.userId,
        this.users,
        this.connections,
        this.steps,
        this.description,
        this.desiredStatus,
        this.currentStatus,
        this.stepsDone,
        this.statusMessage,
        value,
        this.createdDate,
        this.timesUsed,
        this.id,
        this.tags,
        this.name));
  }

  /**
   * Copy the current immutable object by setting a <i>present</i> value for the optional {@link Integration#getCreatedDate() createdDate} attribute.
   * @param value The value for createdDate
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegration withCreatedDate(Date value) {
    Date newValue = Objects.requireNonNull(value, "createdDate");
    if (this.createdDate == newValue) return this;
    return validate(new ImmutableIntegration(
        this,
        this.revisions,
        this.draftRevision,
        this.deployedRevisionId,
        this.configuration,
        this.integrationTemplateId,
        this.userId,
        this.users,
        this.connections,
        this.steps,
        this.description,
        this.desiredStatus,
        this.currentStatus,
        this.stepsDone,
        this.statusMessage,
        this.lastUpdated,
        newValue,
        this.timesUsed,
        this.id,
        this.tags,
        this.name));
  }

  /**
   * Copy the current immutable object by setting an optional value for the {@link Integration#getCreatedDate() createdDate} attribute.
   * A shallow reference equality check is used on unboxed optional value to prevent copying of the same value by returning {@code this}.
   * @param optional A value for createdDate
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegration withCreatedDate(Optional<? extends Date> optional) {
    Date value = optional.orElse(null);
    if (this.createdDate == value) return this;
    return validate(new ImmutableIntegration(
        this,
        this.revisions,
        this.draftRevision,
        this.deployedRevisionId,
        this.configuration,
        this.integrationTemplateId,
        this.userId,
        this.users,
        this.connections,
        this.steps,
        this.description,
        this.desiredStatus,
        this.currentStatus,
        this.stepsDone,
        this.statusMessage,
        this.lastUpdated,
        value,
        this.timesUsed,
        this.id,
        this.tags,
        this.name));
  }

  /**
   * Copy the current immutable object by setting a <i>present</i> value for the optional {@link Integration#getTimesUsed() timesUsed} attribute.
   * @param value The value for timesUsed
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegration withTimesUsed(BigInteger value) {
    BigInteger newValue = Objects.requireNonNull(value, "timesUsed");
    if (Objects.equals(this.timesUsed, newValue)) return this;
    return validate(new ImmutableIntegration(
        this,
        this.revisions,
        this.draftRevision,
        this.deployedRevisionId,
        this.configuration,
        this.integrationTemplateId,
        this.userId,
        this.users,
        this.connections,
        this.steps,
        this.description,
        this.desiredStatus,
        this.currentStatus,
        this.stepsDone,
        this.statusMessage,
        this.lastUpdated,
        this.createdDate,
        newValue,
        this.id,
        this.tags,
        this.name));
  }

  /**
   * Copy the current immutable object by setting an optional value for the {@link Integration#getTimesUsed() timesUsed} attribute.
   * An equality check is used on inner nullable value to prevent copying of the same value by returning {@code this}.
   * @param optional A value for timesUsed
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegration withTimesUsed(Optional<? extends BigInteger> optional) {
    BigInteger value = optional.orElse(null);
    if (Objects.equals(this.timesUsed, value)) return this;
    return validate(new ImmutableIntegration(
        this,
        this.revisions,
        this.draftRevision,
        this.deployedRevisionId,
        this.configuration,
        this.integrationTemplateId,
        this.userId,
        this.users,
        this.connections,
        this.steps,
        this.description,
        this.desiredStatus,
        this.currentStatus,
        this.stepsDone,
        this.statusMessage,
        this.lastUpdated,
        this.createdDate,
        value,
        this.id,
        this.tags,
        this.name));
  }

  /**
   * Copy the current immutable object by setting a <i>present</i> value for the optional {@link Integration#getId() id} attribute.
   * @param value The value for id
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegration withId(String value) {
    String newValue = Objects.requireNonNull(value, "id");
    if (Objects.equals(this.id, newValue)) return this;
    return validate(new ImmutableIntegration(
        this,
        this.revisions,
        this.draftRevision,
        this.deployedRevisionId,
        this.configuration,
        this.integrationTemplateId,
        this.userId,
        this.users,
        this.connections,
        this.steps,
        this.description,
        this.desiredStatus,
        this.currentStatus,
        this.stepsDone,
        this.statusMessage,
        this.lastUpdated,
        this.createdDate,
        this.timesUsed,
        newValue,
        this.tags,
        this.name));
  }

  /**
   * Copy the current immutable object by setting an optional value for the {@link Integration#getId() id} attribute.
   * An equality check is used on inner nullable value to prevent copying of the same value by returning {@code this}.
   * @param optional A value for id
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegration withId(Optional<String> optional) {
    String value = optional.orElse(null);
    if (Objects.equals(this.id, value)) return this;
    return validate(new ImmutableIntegration(
        this,
        this.revisions,
        this.draftRevision,
        this.deployedRevisionId,
        this.configuration,
        this.integrationTemplateId,
        this.userId,
        this.users,
        this.connections,
        this.steps,
        this.description,
        this.desiredStatus,
        this.currentStatus,
        this.stepsDone,
        this.statusMessage,
        this.lastUpdated,
        this.createdDate,
        this.timesUsed,
        value,
        this.tags,
        this.name));
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link Integration#getTags() tags}.
   * @param elements The elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegration withTags(String... elements) {
    SortedSet<String> newValue = createUnmodifiableSortedSet(false, createSafeList(Arrays.asList(elements), false, true));
    return validate(new ImmutableIntegration(
        this,
        this.revisions,
        this.draftRevision,
        this.deployedRevisionId,
        this.configuration,
        this.integrationTemplateId,
        this.userId,
        this.users,
        this.connections,
        this.steps,
        this.description,
        this.desiredStatus,
        this.currentStatus,
        this.stepsDone,
        this.statusMessage,
        this.lastUpdated,
        this.createdDate,
        this.timesUsed,
        this.id,
        newValue,
        this.name));
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link Integration#getTags() tags}.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param elements An iterable of tags elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegration withTags(Iterable<String> elements) {
    if (this.tags == elements) return this;
    SortedSet<String> newValue = createUnmodifiableSortedSet(false, createSafeList(elements, false, true));
    return validate(new ImmutableIntegration(
        this,
        this.revisions,
        this.draftRevision,
        this.deployedRevisionId,
        this.configuration,
        this.integrationTemplateId,
        this.userId,
        this.users,
        this.connections,
        this.steps,
        this.description,
        this.desiredStatus,
        this.currentStatus,
        this.stepsDone,
        this.statusMessage,
        this.lastUpdated,
        this.createdDate,
        this.timesUsed,
        this.id,
        newValue,
        this.name));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link Integration#getName() name} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for name (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIntegration withName(String value) {
    if (Objects.equals(this.name, value)) return this;
    return validate(new ImmutableIntegration(
        this,
        this.revisions,
        this.draftRevision,
        this.deployedRevisionId,
        this.configuration,
        this.integrationTemplateId,
        this.userId,
        this.users,
        this.connections,
        this.steps,
        this.description,
        this.desiredStatus,
        this.currentStatus,
        this.stepsDone,
        this.statusMessage,
        this.lastUpdated,
        this.createdDate,
        this.timesUsed,
        this.id,
        this.tags,
        value));
  }

  /**
   * This instance is equal to all instances of {@code ImmutableIntegration} that have equal attribute values.
   * @return {@code true} if {@code this} is equal to {@code another} instance
   */
  @Override
  public boolean equals(Object another) {
    if (this == another) return true;
    return another instanceof ImmutableIntegration
        && equalTo((ImmutableIntegration) another);
  }

  private boolean equalTo(ImmutableIntegration another) {
    return revisions.equals(another.revisions)
        && Objects.equals(draftRevision, another.draftRevision)
        && Objects.equals(deployedRevisionId, another.deployedRevisionId)
        && Objects.equals(configuration, another.configuration)
        && Objects.equals(integrationTemplateId, another.integrationTemplateId)
        && Objects.equals(userId, another.userId)
        && users.equals(another.users)
        && connections.equals(another.connections)
        && steps.equals(another.steps)
        && Objects.equals(description, another.description)
        && Objects.equals(desiredStatus, another.desiredStatus)
        && Objects.equals(currentStatus, another.currentStatus)
        && stepsDone.equals(another.stepsDone)
        && Objects.equals(statusMessage, another.statusMessage)
        && Objects.equals(lastUpdated, another.lastUpdated)
        && Objects.equals(createdDate, another.createdDate)
        && Objects.equals(timesUsed, another.timesUsed)
        && Objects.equals(id, another.id)
        && tags.equals(another.tags)
        && Objects.equals(name, another.name);
  }

  /**
   * Computes a hash code from attributes: {@code revisions}, {@code draftRevision}, {@code deployedRevisionId}, {@code configuration}, {@code integrationTemplateId}, {@code userId}, {@code users}, {@code connections}, {@code steps}, {@code description}, {@code desiredStatus}, {@code currentStatus}, {@code stepsDone}, {@code statusMessage}, {@code lastUpdated}, {@code createdDate}, {@code timesUsed}, {@code id}, {@code tags}, {@code name}.
   * @return hashCode value
   */
  @Override
  public int hashCode() {
    int h = 5381;
    h += (h << 5) + revisions.hashCode();
    h += (h << 5) + Objects.hashCode(draftRevision);
    h += (h << 5) + Objects.hashCode(deployedRevisionId);
    h += (h << 5) + Objects.hashCode(configuration);
    h += (h << 5) + Objects.hashCode(integrationTemplateId);
    h += (h << 5) + Objects.hashCode(userId);
    h += (h << 5) + users.hashCode();
    h += (h << 5) + connections.hashCode();
    h += (h << 5) + steps.hashCode();
    h += (h << 5) + Objects.hashCode(description);
    h += (h << 5) + Objects.hashCode(desiredStatus);
    h += (h << 5) + Objects.hashCode(currentStatus);
    h += (h << 5) + stepsDone.hashCode();
    h += (h << 5) + Objects.hashCode(statusMessage);
    h += (h << 5) + Objects.hashCode(lastUpdated);
    h += (h << 5) + Objects.hashCode(createdDate);
    h += (h << 5) + Objects.hashCode(timesUsed);
    h += (h << 5) + Objects.hashCode(id);
    h += (h << 5) + tags.hashCode();
    h += (h << 5) + Objects.hashCode(name);
    return h;
  }

  /**
   * Prints the immutable value {@code Integration} with attribute values.
   * @return A string representation of the value
   */
  @Override
  public String toString() {
    StringBuilder builder = new StringBuilder("Integration{");
    builder.append("revisions=").append(revisions);
    if (draftRevision != null) {
      builder.append(", ");
      builder.append("draftRevision=").append(draftRevision);
    }
    if (deployedRevisionId != null) {
      builder.append(", ");
      builder.append("deployedRevisionId=").append(deployedRevisionId);
    }
    if (configuration != null) {
      builder.append(", ");
      builder.append("configuration=").append(configuration);
    }
    if (integrationTemplateId != null) {
      builder.append(", ");
      builder.append("integrationTemplateId=").append(integrationTemplateId);
    }
    if (userId != null) {
      builder.append(", ");
      builder.append("userId=").append(userId);
    }
    builder.append(", ");
    builder.append("users=").append(users);
    builder.append(", ");
    builder.append("connections=").append(connections);
    builder.append(", ");
    builder.append("steps=").append(steps);
    if (description != null) {
      builder.append(", ");
      builder.append("description=").append(description);
    }
    if (desiredStatus != null) {
      builder.append(", ");
      builder.append("desiredStatus=").append(desiredStatus);
    }
    if (currentStatus != null) {
      builder.append(", ");
      builder.append("currentStatus=").append(currentStatus);
    }
    builder.append(", ");
    builder.append("stepsDone=").append(stepsDone);
    if (statusMessage != null) {
      builder.append(", ");
      builder.append("statusMessage=").append(statusMessage);
    }
    if (lastUpdated != null) {
      builder.append(", ");
      builder.append("lastUpdated=").append(lastUpdated);
    }
    if (createdDate != null) {
      builder.append(", ");
      builder.append("createdDate=").append(createdDate);
    }
    if (timesUsed != null) {
      builder.append(", ");
      builder.append("timesUsed=").append(timesUsed);
    }
    if (id != null) {
      builder.append(", ");
      builder.append("id=").append(id);
    }
    builder.append(", ");
    builder.append("tags=").append(tags);
    if (name != null) {
      builder.append(", ");
      builder.append("name=").append(name);
    }
    return builder.append("}").toString();
  }

  /**
   * Construct a new immutable {@code Integration} instance.
   * @param revisions The value for the {@code revisions} attribute
   * @param draftRevision The value for the {@code draftRevision} attribute
   * @param deployedRevisionId The value for the {@code deployedRevisionId} attribute
   * @param configuration The value for the {@code configuration} attribute
   * @param integrationTemplateId The value for the {@code integrationTemplateId} attribute
   * @param userId The value for the {@code userId} attribute
   * @param users The value for the {@code users} attribute
   * @param connections The value for the {@code connections} attribute
   * @param steps The value for the {@code steps} attribute
   * @param description The value for the {@code description} attribute
   * @param desiredStatus The value for the {@code desiredStatus} attribute
   * @param currentStatus The value for the {@code currentStatus} attribute
   * @param stepsDone The value for the {@code stepsDone} attribute
   * @param statusMessage The value for the {@code statusMessage} attribute
   * @param lastUpdated The value for the {@code lastUpdated} attribute
   * @param createdDate The value for the {@code createdDate} attribute
   * @param timesUsed The value for the {@code timesUsed} attribute
   * @param id The value for the {@code id} attribute
   * @param tags The value for the {@code tags} attribute
   * @param name The value for the {@code name} attribute
   * @return An immutable Integration instance
   */
  public static ImmutableIntegration of(Set<IntegrationRevision> revisions, Optional<IntegrationRevision> draftRevision, Optional<Integer> deployedRevisionId, Optional<String> configuration, Optional<String> integrationTemplateId, Optional<String> userId, List<User> users, List<Connection> connections, List<? extends Step> steps, Optional<String> description, Optional<Integration.Status> desiredStatus, Optional<Integration.Status> currentStatus, List<String> stepsDone, Optional<String> statusMessage, Optional<Date> lastUpdated, Optional<Date> createdDate, Optional<BigInteger> timesUsed, Optional<String> id, SortedSet<String> tags, String name) {
    return of((Iterable<? extends IntegrationRevision>) revisions, draftRevision, deployedRevisionId, configuration, integrationTemplateId, userId, (Iterable<? extends User>) users, (Iterable<? extends Connection>) connections, steps, description, desiredStatus, currentStatus, (Iterable<String>) stepsDone, statusMessage, lastUpdated, createdDate, timesUsed, id, (Iterable<String>) tags, name);
  }

  /**
   * Construct a new immutable {@code Integration} instance.
   * @param revisions The value for the {@code revisions} attribute
   * @param draftRevision The value for the {@code draftRevision} attribute
   * @param deployedRevisionId The value for the {@code deployedRevisionId} attribute
   * @param configuration The value for the {@code configuration} attribute
   * @param integrationTemplateId The value for the {@code integrationTemplateId} attribute
   * @param userId The value for the {@code userId} attribute
   * @param users The value for the {@code users} attribute
   * @param connections The value for the {@code connections} attribute
   * @param steps The value for the {@code steps} attribute
   * @param description The value for the {@code description} attribute
   * @param desiredStatus The value for the {@code desiredStatus} attribute
   * @param currentStatus The value for the {@code currentStatus} attribute
   * @param stepsDone The value for the {@code stepsDone} attribute
   * @param statusMessage The value for the {@code statusMessage} attribute
   * @param lastUpdated The value for the {@code lastUpdated} attribute
   * @param createdDate The value for the {@code createdDate} attribute
   * @param timesUsed The value for the {@code timesUsed} attribute
   * @param id The value for the {@code id} attribute
   * @param tags The value for the {@code tags} attribute
   * @param name The value for the {@code name} attribute
   * @return An immutable Integration instance
   */
  public static ImmutableIntegration of(Iterable<? extends IntegrationRevision> revisions, Optional<IntegrationRevision> draftRevision, Optional<Integer> deployedRevisionId, Optional<String> configuration, Optional<String> integrationTemplateId, Optional<String> userId, Iterable<? extends User> users, Iterable<? extends Connection> connections, List<? extends Step> steps, Optional<String> description, Optional<Integration.Status> desiredStatus, Optional<Integration.Status> currentStatus, Iterable<String> stepsDone, Optional<String> statusMessage, Optional<Date> lastUpdated, Optional<Date> createdDate, Optional<BigInteger> timesUsed, Optional<String> id, Iterable<String> tags, String name) {
    return validate(new ImmutableIntegration(revisions, draftRevision, deployedRevisionId, configuration, integrationTemplateId, userId, users, connections, steps, description, desiredStatus, currentStatus, stepsDone, statusMessage, lastUpdated, createdDate, timesUsed, id, tags, name));
  }

  private static final Validator validator = Validation.buildDefaultValidatorFactory().getValidator();

  private static ImmutableIntegration validate(ImmutableIntegration instance) {
    Set<ConstraintViolation<ImmutableIntegration>> constraintViolations = validator.validate(instance);
    if (!constraintViolations.isEmpty()) {
      throw new ConstraintViolationException(constraintViolations);
    }
    return instance;
  }

  /**
   * Creates an immutable copy of a {@link Integration} value.
   * Uses accessors to get values to initialize the new immutable instance.
   * If an instance is already immutable, it is returned as is.
   * @param instance The instance to copy
   * @return A copied immutable Integration instance
   */
  public static ImmutableIntegration copyOf(Integration instance) {
    if (instance instanceof ImmutableIntegration) {
      return (ImmutableIntegration) instance;
    }
    return new Integration.Builder()
        .createFrom(instance)
        .build();
  }

  private Object readResolve() throws ObjectStreamException {
    return validate(this);
  }

  /**
   * Builds instances of type {@link ImmutableIntegration ImmutableIntegration}.
   * Initialize attributes and then invoke the {@link #build()} method to create an
   * immutable instance.
   * <p><em>{@code Builder} is not thread-safe and generally should not be stored in a field or collection,
   * but instead used immediately to create instances.</em>
   */
  static class Builder {
    private static final long OPT_BIT_CONNECTIONS = 0x1L;
    private static final long OPT_BIT_STEPS_DONE = 0x2L;
    private long optBits;

    private List<IntegrationRevision> revisions = new ArrayList<IntegrationRevision>();
    private IntegrationRevision draftRevision;
    private Integer deployedRevisionId;
    private String configuration;
    private String integrationTemplateId;
    private String userId;
    private List<User> users = new ArrayList<User>();
    private List<Connection> connections = new ArrayList<Connection>();
    private List<? extends Step> steps;
    private String description;
    private Integration.Status desiredStatus;
    private Integration.Status currentStatus;
    private List<String> stepsDone = new ArrayList<String>();
    private String statusMessage;
    private Date lastUpdated;
    private Date createdDate;
    private BigInteger timesUsed;
    private String id;
    private List<String> tags = new ArrayList<String>();
    private String name;

    /**
     * Creates a builder for {@link ImmutableIntegration ImmutableIntegration} instances.
     */
    Builder() {
      if (!(this instanceof Integration.Builder)) {
        throw new UnsupportedOperationException("Use: new Integration.Builder()");
      }
    }

    /**
     * Fill a builder with attribute values from the provided {@code io.syndesis.model.integration.Integration} instance.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    public final Integration.Builder createFrom(Integration instance) {
      Objects.requireNonNull(instance, "instance");
      from((Object) instance);
      return (Integration.Builder) this;
    }

    /**
     * Fill a builder with attribute values from the provided {@code io.syndesis.model.WithTags} instance.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    public final Integration.Builder createFrom(WithTags instance) {
      Objects.requireNonNull(instance, "instance");
      from((Object) instance);
      return (Integration.Builder) this;
    }

    /**
     * Fill a builder with attribute values from the provided {@code io.syndesis.model.WithName} instance.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    public final Integration.Builder createFrom(WithName instance) {
      Objects.requireNonNull(instance, "instance");
      from((Object) instance);
      return (Integration.Builder) this;
    }

    private void from(Object object) {
      if (object instanceof Integration) {
        Integration instance = (Integration) object;
        Optional<Integer> deployedRevisionIdOptional = instance.getDeployedRevisionId();
        if (deployedRevisionIdOptional.isPresent()) {
          deployedRevisionId(deployedRevisionIdOptional);
        }
        Optional<String> integrationTemplateIdOptional = instance.getIntegrationTemplateId();
        if (integrationTemplateIdOptional.isPresent()) {
          integrationTemplateId(integrationTemplateIdOptional);
        }
        Optional<String> configurationOptional = instance.getConfiguration();
        if (configurationOptional.isPresent()) {
          configuration(configurationOptional);
        }
        Optional<Integration.Status> currentStatusOptional = instance.getCurrentStatus();
        if (currentStatusOptional.isPresent()) {
          currentStatus(currentStatusOptional);
        }
        addAllStepsDone(instance.getStepsDone());
        Optional<String> descriptionOptional = instance.getDescription();
        if (descriptionOptional.isPresent()) {
          description(descriptionOptional);
        }
        Optional<String> userIdOptional = instance.getUserId();
        if (userIdOptional.isPresent()) {
          userId(userIdOptional);
        }
        steps(instance.getSteps());
        addAllUsers(instance.getUsers());
        Optional<String> statusMessageOptional = instance.getStatusMessage();
        if (statusMessageOptional.isPresent()) {
          statusMessage(statusMessageOptional);
        }
        Optional<Date> lastUpdatedOptional = instance.getLastUpdated();
        if (lastUpdatedOptional.isPresent()) {
          lastUpdated(lastUpdatedOptional);
        }
        Optional<Date> createdDateOptional = instance.getCreatedDate();
        if (createdDateOptional.isPresent()) {
          createdDate(createdDateOptional);
        }
        Optional<BigInteger> timesUsedOptional = instance.getTimesUsed();
        if (timesUsedOptional.isPresent()) {
          timesUsed(timesUsedOptional);
        }
        addAllRevisions(instance.getRevisions());
        Optional<IntegrationRevision> draftRevisionOptional = instance.getDraftRevision();
        if (draftRevisionOptional.isPresent()) {
          draftRevision(draftRevisionOptional);
        }
        Optional<String> idOptional = instance.getId();
        if (idOptional.isPresent()) {
          id(idOptional);
        }
        addAllConnections(instance.getConnections());
        Optional<Integration.Status> desiredStatusOptional = instance.getDesiredStatus();
        if (desiredStatusOptional.isPresent()) {
          desiredStatus(desiredStatusOptional);
        }
      }
      if (object instanceof WithTags) {
        WithTags instance = (WithTags) object;
        addAllTags(instance.getTags());
      }
      if (object instanceof WithName) {
        WithName instance = (WithName) object;
        String nameValue = instance.getName();
        if (nameValue != null) {
          name(nameValue);
        }
      }
    }

    /**
     * Adds one element to {@link Integration#getRevisions() revisions} set.
     * @param element A revisions element
     * @return {@code this} builder for use in a chained invocation
     */
    public final Integration.Builder addRevision(IntegrationRevision element) {
      this.revisions.add(Objects.requireNonNull(element, "revisions element"));
      return (Integration.Builder) this;
    }

    /**
     * Adds elements to {@link Integration#getRevisions() revisions} set.
     * @param elements An array of revisions elements
     * @return {@code this} builder for use in a chained invocation
     */
    public final Integration.Builder addRevision(IntegrationRevision... elements) {
      for (IntegrationRevision element : elements) {
        this.revisions.add(Objects.requireNonNull(element, "revisions element"));
      }
      return (Integration.Builder) this;
    }

    /**
     * Sets or replaces all elements for {@link Integration#getRevisions() revisions} set.
     * @param elements An iterable of revisions elements
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("revisions")
    public final Integration.Builder revisions(Iterable<? extends IntegrationRevision> elements) {
      this.revisions.clear();
      return addAllRevisions(elements);
    }

    /**
     * Adds elements to {@link Integration#getRevisions() revisions} set.
     * @param elements An iterable of revisions elements
     * @return {@code this} builder for use in a chained invocation
     */
    public final Integration.Builder addAllRevisions(Iterable<? extends IntegrationRevision> elements) {
      for (IntegrationRevision element : elements) {
        this.revisions.add(Objects.requireNonNull(element, "revisions element"));
      }
      return (Integration.Builder) this;
    }

    /**
     * Initializes the optional value {@link Integration#getDraftRevision() draftRevision} to draftRevision.
     * @param draftRevision The value for draftRevision
     * @return {@code this} builder for chained invocation
     */
    public final Integration.Builder draftRevision(IntegrationRevision draftRevision) {
      this.draftRevision = Objects.requireNonNull(draftRevision, "draftRevision");
      return (Integration.Builder) this;
    }

    /**
     * Initializes the optional value {@link Integration#getDraftRevision() draftRevision} to draftRevision.
     * @param draftRevision The value for draftRevision
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("draftRevision")
    public final Integration.Builder draftRevision(Optional<? extends IntegrationRevision> draftRevision) {
      this.draftRevision = draftRevision.orElse(null);
      return (Integration.Builder) this;
    }

    /**
     * Initializes the optional value {@link Integration#getDeployedRevisionId() deployedRevisionId} to deployedRevisionId.
     * @param deployedRevisionId The value for deployedRevisionId
     * @return {@code this} builder for chained invocation
     */
    public final Integration.Builder deployedRevisionId(int deployedRevisionId) {
      this.deployedRevisionId = deployedRevisionId;
      return (Integration.Builder) this;
    }

    /**
     * Initializes the optional value {@link Integration#getDeployedRevisionId() deployedRevisionId} to deployedRevisionId.
     * @param deployedRevisionId The value for deployedRevisionId
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("deployedRevisionId")
    public final Integration.Builder deployedRevisionId(Optional<Integer> deployedRevisionId) {
      this.deployedRevisionId = deployedRevisionId.orElse(null);
      return (Integration.Builder) this;
    }

    /**
     * Initializes the optional value {@link Integration#getConfiguration() configuration} to configuration.
     * @param configuration The value for configuration
     * @return {@code this} builder for chained invocation
     */
    public final Integration.Builder configuration(String configuration) {
      this.configuration = Objects.requireNonNull(configuration, "configuration");
      return (Integration.Builder) this;
    }

    /**
     * Initializes the optional value {@link Integration#getConfiguration() configuration} to configuration.
     * @param configuration The value for configuration
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("configuration")
    public final Integration.Builder configuration(Optional<String> configuration) {
      this.configuration = configuration.orElse(null);
      return (Integration.Builder) this;
    }

    /**
     * Initializes the optional value {@link Integration#getIntegrationTemplateId() integrationTemplateId} to integrationTemplateId.
     * @param integrationTemplateId The value for integrationTemplateId
     * @return {@code this} builder for chained invocation
     */
    public final Integration.Builder integrationTemplateId(String integrationTemplateId) {
      this.integrationTemplateId = Objects.requireNonNull(integrationTemplateId, "integrationTemplateId");
      return (Integration.Builder) this;
    }

    /**
     * Initializes the optional value {@link Integration#getIntegrationTemplateId() integrationTemplateId} to integrationTemplateId.
     * @param integrationTemplateId The value for integrationTemplateId
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("integrationTemplateId")
    public final Integration.Builder integrationTemplateId(Optional<String> integrationTemplateId) {
      this.integrationTemplateId = integrationTemplateId.orElse(null);
      return (Integration.Builder) this;
    }

    /**
     * Initializes the optional value {@link Integration#getUserId() userId} to userId.
     * @param userId The value for userId
     * @return {@code this} builder for chained invocation
     */
    public final Integration.Builder userId(String userId) {
      this.userId = Objects.requireNonNull(userId, "userId");
      return (Integration.Builder) this;
    }

    /**
     * Initializes the optional value {@link Integration#getUserId() userId} to userId.
     * @param userId The value for userId
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("userId")
    public final Integration.Builder userId(Optional<String> userId) {
      this.userId = userId.orElse(null);
      return (Integration.Builder) this;
    }

    /**
     * Adds one element to {@link Integration#getUsers() users} list.
     * @param element A users element
     * @return {@code this} builder for use in a chained invocation
     */
    public final Integration.Builder addUser(User element) {
      this.users.add(Objects.requireNonNull(element, "users element"));
      return (Integration.Builder) this;
    }

    /**
     * Adds elements to {@link Integration#getUsers() users} list.
     * @param elements An array of users elements
     * @return {@code this} builder for use in a chained invocation
     */
    public final Integration.Builder addUser(User... elements) {
      for (User element : elements) {
        this.users.add(Objects.requireNonNull(element, "users element"));
      }
      return (Integration.Builder) this;
    }

    /**
     * Sets or replaces all elements for {@link Integration#getUsers() users} list.
     * @param elements An iterable of users elements
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("users")
    public final Integration.Builder users(Iterable<? extends User> elements) {
      this.users.clear();
      return addAllUsers(elements);
    }

    /**
     * Adds elements to {@link Integration#getUsers() users} list.
     * @param elements An iterable of users elements
     * @return {@code this} builder for use in a chained invocation
     */
    public final Integration.Builder addAllUsers(Iterable<? extends User> elements) {
      for (User element : elements) {
        this.users.add(Objects.requireNonNull(element, "users element"));
      }
      return (Integration.Builder) this;
    }

    /**
     * Adds one element to {@link Integration#getConnections() connections} list.
     * @param element A connections element
     * @return {@code this} builder for use in a chained invocation
     */
    public final Integration.Builder addConnection(Connection element) {
      this.connections.add(Objects.requireNonNull(element, "connections element"));
      optBits |= OPT_BIT_CONNECTIONS;
      return (Integration.Builder) this;
    }

    /**
     * Adds elements to {@link Integration#getConnections() connections} list.
     * @param elements An array of connections elements
     * @return {@code this} builder for use in a chained invocation
     */
    public final Integration.Builder addConnection(Connection... elements) {
      for (Connection element : elements) {
        this.connections.add(Objects.requireNonNull(element, "connections element"));
      }
      optBits |= OPT_BIT_CONNECTIONS;
      return (Integration.Builder) this;
    }

    /**
     * Sets or replaces all elements for {@link Integration#getConnections() connections} list.
     * @param elements An iterable of connections elements
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("connections")
    public final Integration.Builder connections(Iterable<? extends Connection> elements) {
      this.connections.clear();
      return addAllConnections(elements);
    }

    /**
     * Adds elements to {@link Integration#getConnections() connections} list.
     * @param elements An iterable of connections elements
     * @return {@code this} builder for use in a chained invocation
     */
    public final Integration.Builder addAllConnections(Iterable<? extends Connection> elements) {
      for (Connection element : elements) {
        this.connections.add(Objects.requireNonNull(element, "connections element"));
      }
      optBits |= OPT_BIT_CONNECTIONS;
      return (Integration.Builder) this;
    }

    /**
     * Initializes the value for the {@link Integration#getSteps() steps} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link Integration#getSteps() steps}.</em>
     * @param steps The value for steps 
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("steps")
    public final Integration.Builder steps(List<? extends Step> steps) {
      this.steps = Objects.requireNonNull(steps, "steps");
      return (Integration.Builder) this;
    }

    /**
     * Initializes the optional value {@link Integration#getDescription() description} to description.
     * @param description The value for description
     * @return {@code this} builder for chained invocation
     */
    public final Integration.Builder description(String description) {
      this.description = Objects.requireNonNull(description, "description");
      return (Integration.Builder) this;
    }

    /**
     * Initializes the optional value {@link Integration#getDescription() description} to description.
     * @param description The value for description
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("description")
    public final Integration.Builder description(Optional<String> description) {
      this.description = description.orElse(null);
      return (Integration.Builder) this;
    }

    /**
     * Initializes the optional value {@link Integration#getDesiredStatus() desiredStatus} to desiredStatus.
     * @param desiredStatus The value for desiredStatus
     * @return {@code this} builder for chained invocation
     */
    public final Integration.Builder desiredStatus(Integration.Status desiredStatus) {
      this.desiredStatus = Objects.requireNonNull(desiredStatus, "desiredStatus");
      return (Integration.Builder) this;
    }

    /**
     * Initializes the optional value {@link Integration#getDesiredStatus() desiredStatus} to desiredStatus.
     * @param desiredStatus The value for desiredStatus
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("desiredStatus")
    public final Integration.Builder desiredStatus(Optional<? extends Integration.Status> desiredStatus) {
      this.desiredStatus = desiredStatus.orElse(null);
      return (Integration.Builder) this;
    }

    /**
     * Initializes the optional value {@link Integration#getCurrentStatus() currentStatus} to currentStatus.
     * @param currentStatus The value for currentStatus
     * @return {@code this} builder for chained invocation
     */
    public final Integration.Builder currentStatus(Integration.Status currentStatus) {
      this.currentStatus = Objects.requireNonNull(currentStatus, "currentStatus");
      return (Integration.Builder) this;
    }

    /**
     * Initializes the optional value {@link Integration#getCurrentStatus() currentStatus} to currentStatus.
     * @param currentStatus The value for currentStatus
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("currentStatus")
    public final Integration.Builder currentStatus(Optional<? extends Integration.Status> currentStatus) {
      this.currentStatus = currentStatus.orElse(null);
      return (Integration.Builder) this;
    }

    /**
     * Adds one element to {@link Integration#getStepsDone() stepsDone} list.
     * @param element A stepsDone element
     * @return {@code this} builder for use in a chained invocation
     */
    public final Integration.Builder addStepsDone(String element) {
      this.stepsDone.add(Objects.requireNonNull(element, "stepsDone element"));
      optBits |= OPT_BIT_STEPS_DONE;
      return (Integration.Builder) this;
    }

    /**
     * Adds elements to {@link Integration#getStepsDone() stepsDone} list.
     * @param elements An array of stepsDone elements
     * @return {@code this} builder for use in a chained invocation
     */
    public final Integration.Builder addStepsDone(String... elements) {
      for (String element : elements) {
        this.stepsDone.add(Objects.requireNonNull(element, "stepsDone element"));
      }
      optBits |= OPT_BIT_STEPS_DONE;
      return (Integration.Builder) this;
    }

    /**
     * Sets or replaces all elements for {@link Integration#getStepsDone() stepsDone} list.
     * @param elements An iterable of stepsDone elements
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("stepsDone")
    public final Integration.Builder stepsDone(Iterable<String> elements) {
      this.stepsDone.clear();
      return addAllStepsDone(elements);
    }

    /**
     * Adds elements to {@link Integration#getStepsDone() stepsDone} list.
     * @param elements An iterable of stepsDone elements
     * @return {@code this} builder for use in a chained invocation
     */
    public final Integration.Builder addAllStepsDone(Iterable<String> elements) {
      for (String element : elements) {
        this.stepsDone.add(Objects.requireNonNull(element, "stepsDone element"));
      }
      optBits |= OPT_BIT_STEPS_DONE;
      return (Integration.Builder) this;
    }

    /**
     * Initializes the optional value {@link Integration#getStatusMessage() statusMessage} to statusMessage.
     * @param statusMessage The value for statusMessage
     * @return {@code this} builder for chained invocation
     */
    public final Integration.Builder statusMessage(String statusMessage) {
      this.statusMessage = Objects.requireNonNull(statusMessage, "statusMessage");
      return (Integration.Builder) this;
    }

    /**
     * Initializes the optional value {@link Integration#getStatusMessage() statusMessage} to statusMessage.
     * @param statusMessage The value for statusMessage
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("statusMessage")
    public final Integration.Builder statusMessage(Optional<String> statusMessage) {
      this.statusMessage = statusMessage.orElse(null);
      return (Integration.Builder) this;
    }

    /**
     * Initializes the optional value {@link Integration#getLastUpdated() lastUpdated} to lastUpdated.
     * @param lastUpdated The value for lastUpdated
     * @return {@code this} builder for chained invocation
     */
    public final Integration.Builder lastUpdated(Date lastUpdated) {
      this.lastUpdated = Objects.requireNonNull(lastUpdated, "lastUpdated");
      return (Integration.Builder) this;
    }

    /**
     * Initializes the optional value {@link Integration#getLastUpdated() lastUpdated} to lastUpdated.
     * @param lastUpdated The value for lastUpdated
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("lastUpdated")
    public final Integration.Builder lastUpdated(Optional<? extends Date> lastUpdated) {
      this.lastUpdated = lastUpdated.orElse(null);
      return (Integration.Builder) this;
    }

    /**
     * Initializes the optional value {@link Integration#getCreatedDate() createdDate} to createdDate.
     * @param createdDate The value for createdDate
     * @return {@code this} builder for chained invocation
     */
    public final Integration.Builder createdDate(Date createdDate) {
      this.createdDate = Objects.requireNonNull(createdDate, "createdDate");
      return (Integration.Builder) this;
    }

    /**
     * Initializes the optional value {@link Integration#getCreatedDate() createdDate} to createdDate.
     * @param createdDate The value for createdDate
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("createdDate")
    public final Integration.Builder createdDate(Optional<? extends Date> createdDate) {
      this.createdDate = createdDate.orElse(null);
      return (Integration.Builder) this;
    }

    /**
     * Initializes the optional value {@link Integration#getTimesUsed() timesUsed} to timesUsed.
     * @param timesUsed The value for timesUsed
     * @return {@code this} builder for chained invocation
     */
    public final Integration.Builder timesUsed(BigInteger timesUsed) {
      this.timesUsed = Objects.requireNonNull(timesUsed, "timesUsed");
      return (Integration.Builder) this;
    }

    /**
     * Initializes the optional value {@link Integration#getTimesUsed() timesUsed} to timesUsed.
     * @param timesUsed The value for timesUsed
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("timesUsed")
    public final Integration.Builder timesUsed(Optional<? extends BigInteger> timesUsed) {
      this.timesUsed = timesUsed.orElse(null);
      return (Integration.Builder) this;
    }

    /**
     * Initializes the optional value {@link Integration#getId() id} to id.
     * @param id The value for id
     * @return {@code this} builder for chained invocation
     */
    public final Integration.Builder id(String id) {
      this.id = Objects.requireNonNull(id, "id");
      return (Integration.Builder) this;
    }

    /**
     * Initializes the optional value {@link Integration#getId() id} to id.
     * @param id The value for id
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("id")
    public final Integration.Builder id(Optional<String> id) {
      this.id = id.orElse(null);
      return (Integration.Builder) this;
    }

    /**
     * Adds one element to {@link Integration#getTags() tags} sortedSet.
     * @param element A tags element
     * @return {@code this} builder for use in a chained invocation
     */
    public final Integration.Builder addTag(String element) {
      if (element != null) this.tags.add(element);
      return (Integration.Builder) this;
    }

    /**
     * Adds elements to {@link Integration#getTags() tags} sortedSet.
     * @param elements An array of tags elements
     * @return {@code this} builder for use in a chained invocation
     */
    public final Integration.Builder addTag(String... elements) {
      for (String element : elements) {
        if (element != null) this.tags.add(element);
      }
      return (Integration.Builder) this;
    }

    /**
     * Sets or replaces all elements for {@link Integration#getTags() tags} sortedSet.
     * @param elements An iterable of tags elements
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("tags")
    public final Integration.Builder tags(Iterable<String> elements) {
      this.tags.clear();
      return addAllTags(elements);
    }

    /**
     * Adds elements to {@link Integration#getTags() tags} sortedSet.
     * @param elements An iterable of tags elements
     * @return {@code this} builder for use in a chained invocation
     */
    public final Integration.Builder addAllTags(Iterable<String> elements) {
      for (String element : elements) {
        if (element != null) this.tags.add(element);
      }
      return (Integration.Builder) this;
    }

    /**
     * Initializes the value for the {@link Integration#getName() name} attribute.
     * @param name The value for name (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("name")
    public final Integration.Builder name(String name) {
      this.name = name;
      return (Integration.Builder) this;
    }

    /**
     * Builds a new {@link ImmutableIntegration ImmutableIntegration}.
     * @return An immutable instance of Integration
     * @throws java.lang.IllegalStateException if any required attributes are missing
     */
    public ImmutableIntegration build() {
      return ImmutableIntegration.validate(new ImmutableIntegration(this));
    }

    private boolean connectionsIsSet() {
      return (optBits & OPT_BIT_CONNECTIONS) != 0;
    }

    private boolean stepsDoneIsSet() {
      return (optBits & OPT_BIT_STEPS_DONE) != 0;
    }
  }

  private static <T> List<T> createSafeList(Iterable<? extends T> iterable, boolean checkNulls, boolean skipNulls) {
    ArrayList<T> list;
    if (iterable instanceof Collection<?>) {
      int size = ((Collection<?>) iterable).size();
      if (size == 0) return Collections.emptyList();
      list = new ArrayList<T>();
    } else {
      list = new ArrayList<T>();
    }
    for (T element : iterable) {
      if (skipNulls && element == null) continue;
      if (checkNulls) Objects.requireNonNull(element, "element");
      list.add(element);
    }
    return list;
  }

  private static <T> List<T> createUnmodifiableList(boolean clone, List<T> list) {
    switch(list.size()) {
    case 0: return Collections.emptyList();
    case 1: return Collections.singletonList(list.get(0));
    default:
      if (clone) {
        return Collections.unmodifiableList(new ArrayList<T>(list));
      } else {
        if (list instanceof ArrayList<?>) {
          ((ArrayList<?>) list).trimToSize();
        }
        return Collections.unmodifiableList(list);
      }
    }
  }

  /** Unmodifiable set constructed from list to avoid rehashing. */
  private static <T> Set<T> createUnmodifiableSet(List<T> list) {
    switch(list.size()) {
    case 0: return Collections.emptySet();
    case 1: return Collections.singleton(list.get(0));
    default:
      Set<T> set = new LinkedHashSet<T>(list.size());
      set.addAll(list);
      return Collections.unmodifiableSet(set);
    }
  }

  private static <T extends Comparable<T>> NavigableSet<T> createUnmodifiableSortedSet(boolean reverse, List<T> list) {
    TreeSet<T> set = reverse
        ? new TreeSet<T>(Collections.reverseOrder())
        : new TreeSet<T>();
    set.addAll(list);
    return Collections.unmodifiableNavigableSet(set);
  }
}
