/*
 * Copyright (C) 2016 Red Hat, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package io.syndesis.model.integration;

import com.fasterxml.jackson.annotation.JsonProperty;
import edu.umd.cs.findbugs.annotations.SuppressFBWarnings;
import java.math.BigInteger;
import java.util.Date;
import java.util.Objects;
import java.util.Optional;
import java.util.Set;
import javax.annotation.Generated;
import javax.validation.ConstraintViolation;
import javax.validation.ConstraintViolationException;
import javax.validation.Validation;
import javax.validation.Validator;

/**
 * Immutable implementation of {@link IntegrationRevision}.
 * <p>
 * Use the builder to create immutable instances:
 * {@code new IntegrationRevision.Builder()}.
 * Use the static factory method to create immutable instances:
 * {@code ImmutableIntegrationRevision.of()}.
 */
@SuppressWarnings({"all"})
@SuppressFBWarnings
@Generated({"Immutables.generator", "IntegrationRevision"})
final class ImmutableIntegrationRevision extends IntegrationRevision {
  private final Integer version;
  private final Integer parentVersion;
  private final IntegrationRevisionSpec spec;
  private final IntegrationRevisionState targetState;
  private final IntegrationRevisionState currentState;
  private final String currentMessage;
  private final String targetMessage;
  private final BigInteger timesUsed;
  private final Date lastUpdated;
  private final Date createdDate;

  private ImmutableIntegrationRevision(
      Optional<Integer> version,
      Integer parentVersion,
      IntegrationRevisionSpec spec,
      IntegrationRevisionState targetState,
      IntegrationRevisionState currentState,
      Optional<String> currentMessage,
      Optional<String> targetMessage,
      Optional<BigInteger> timesUsed,
      Optional<Date> lastUpdated,
      Optional<Date> createdDate) {
    this.version = version.orElse(null);
    this.parentVersion = parentVersion;
    this.spec = spec;
    this.targetState = targetState;
    this.currentState = currentState;
    this.currentMessage = currentMessage.orElse(null);
    this.targetMessage = targetMessage.orElse(null);
    this.timesUsed = timesUsed.orElse(null);
    this.lastUpdated = lastUpdated.orElse(null);
    this.createdDate = createdDate.orElse(null);
  }

  private ImmutableIntegrationRevision(
      ImmutableIntegrationRevision original,
      Integer version,
      Integer parentVersion,
      IntegrationRevisionSpec spec,
      IntegrationRevisionState targetState,
      IntegrationRevisionState currentState,
      String currentMessage,
      String targetMessage,
      BigInteger timesUsed,
      Date lastUpdated,
      Date createdDate) {
    this.version = version;
    this.parentVersion = parentVersion;
    this.spec = spec;
    this.targetState = targetState;
    this.currentState = currentState;
    this.currentMessage = currentMessage;
    this.targetMessage = targetMessage;
    this.timesUsed = timesUsed;
    this.lastUpdated = lastUpdated;
    this.createdDate = createdDate;
  }

  /**
   * @return The value of the {@code version} attribute
   */
  @JsonProperty("version")
  @Override
  public Optional<Integer> getVersion() {
    return Optional.ofNullable(version);
  }

  /**
   * @return The value of the {@code parentVersion} attribute
   */
  @JsonProperty("parentVersion")
  @Override
  public Integer getParentVersion() {
    return parentVersion;
  }

  /**
   * @return The value of the {@code spec} attribute
   */
  @JsonProperty("spec")
  @Override
  public IntegrationRevisionSpec getSpec() {
    return spec;
  }

  /**
   * @return The value of the {@code targetState} attribute
   */
  @JsonProperty("targetState")
  @Override
  public IntegrationRevisionState getTargetState() {
    return targetState;
  }

  /**
   * @return The value of the {@code currentState} attribute
   */
  @JsonProperty("currentState")
  @Override
  public IntegrationRevisionState getCurrentState() {
    return currentState;
  }

  /**
   * @return The value of the {@code currentMessage} attribute
   */
  @JsonProperty("currentMessage")
  @Override
  public Optional<String> getCurrentMessage() {
    return Optional.ofNullable(currentMessage);
  }

  /**
   * @return The value of the {@code targetMessage} attribute
   */
  @JsonProperty("targetMessage")
  @Override
  public Optional<String> getTargetMessage() {
    return Optional.ofNullable(targetMessage);
  }

  /**
   * @return The value of the {@code timesUsed} attribute
   */
  @JsonProperty("timesUsed")
  @Override
  public Optional<BigInteger> getTimesUsed() {
    return Optional.ofNullable(timesUsed);
  }

  /**
   * @return The value of the {@code lastUpdated} attribute
   */
  @JsonProperty("lastUpdated")
  @Override
  public Optional<Date> getLastUpdated() {
    return Optional.ofNullable(lastUpdated);
  }

  /**
   * @return The value of the {@code createdDate} attribute
   */
  @JsonProperty("createdDate")
  @Override
  public Optional<Date> getCreatedDate() {
    return Optional.ofNullable(createdDate);
  }

  /**
   * Copy the current immutable object by setting a <i>present</i> value for the optional {@link IntegrationRevision#getVersion() version} attribute.
   * @param value The value for version
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegrationRevision withVersion(int value) {
    Integer newValue = value;
    if (Objects.equals(this.version, newValue)) return this;
    return validate(new ImmutableIntegrationRevision(
        this,
        newValue,
        this.parentVersion,
        this.spec,
        this.targetState,
        this.currentState,
        this.currentMessage,
        this.targetMessage,
        this.timesUsed,
        this.lastUpdated,
        this.createdDate));
  }

  /**
   * Copy the current immutable object by setting an optional value for the {@link IntegrationRevision#getVersion() version} attribute.
   * An equality check is used on inner nullable value to prevent copying of the same value by returning {@code this}.
   * @param optional A value for version
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegrationRevision withVersion(Optional<Integer> optional) {
    Integer value = optional.orElse(null);
    if (Objects.equals(this.version, value)) return this;
    return validate(new ImmutableIntegrationRevision(
        this,
        value,
        this.parentVersion,
        this.spec,
        this.targetState,
        this.currentState,
        this.currentMessage,
        this.targetMessage,
        this.timesUsed,
        this.lastUpdated,
        this.createdDate));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IntegrationRevision#getParentVersion() parentVersion} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for parentVersion (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIntegrationRevision withParentVersion(Integer value) {
    if (Objects.equals(this.parentVersion, value)) return this;
    return validate(new ImmutableIntegrationRevision(
        this,
        this.version,
        value,
        this.spec,
        this.targetState,
        this.currentState,
        this.currentMessage,
        this.targetMessage,
        this.timesUsed,
        this.lastUpdated,
        this.createdDate));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IntegrationRevision#getSpec() spec} attribute.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for spec (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIntegrationRevision withSpec(IntegrationRevisionSpec value) {
    if (this.spec == value) return this;
    return validate(new ImmutableIntegrationRevision(
        this,
        this.version,
        this.parentVersion,
        value,
        this.targetState,
        this.currentState,
        this.currentMessage,
        this.targetMessage,
        this.timesUsed,
        this.lastUpdated,
        this.createdDate));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IntegrationRevision#getTargetState() targetState} attribute.
   * A value equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for targetState (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIntegrationRevision withTargetState(IntegrationRevisionState value) {
    if (this.targetState == value) return this;
    return validate(new ImmutableIntegrationRevision(
        this,
        this.version,
        this.parentVersion,
        this.spec,
        value,
        this.currentState,
        this.currentMessage,
        this.targetMessage,
        this.timesUsed,
        this.lastUpdated,
        this.createdDate));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IntegrationRevision#getCurrentState() currentState} attribute.
   * A value equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for currentState (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIntegrationRevision withCurrentState(IntegrationRevisionState value) {
    if (this.currentState == value) return this;
    return validate(new ImmutableIntegrationRevision(
        this,
        this.version,
        this.parentVersion,
        this.spec,
        this.targetState,
        value,
        this.currentMessage,
        this.targetMessage,
        this.timesUsed,
        this.lastUpdated,
        this.createdDate));
  }

  /**
   * Copy the current immutable object by setting a <i>present</i> value for the optional {@link IntegrationRevision#getCurrentMessage() currentMessage} attribute.
   * @param value The value for currentMessage
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegrationRevision withCurrentMessage(String value) {
    String newValue = Objects.requireNonNull(value, "currentMessage");
    if (Objects.equals(this.currentMessage, newValue)) return this;
    return validate(new ImmutableIntegrationRevision(
        this,
        this.version,
        this.parentVersion,
        this.spec,
        this.targetState,
        this.currentState,
        newValue,
        this.targetMessage,
        this.timesUsed,
        this.lastUpdated,
        this.createdDate));
  }

  /**
   * Copy the current immutable object by setting an optional value for the {@link IntegrationRevision#getCurrentMessage() currentMessage} attribute.
   * An equality check is used on inner nullable value to prevent copying of the same value by returning {@code this}.
   * @param optional A value for currentMessage
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegrationRevision withCurrentMessage(Optional<String> optional) {
    String value = optional.orElse(null);
    if (Objects.equals(this.currentMessage, value)) return this;
    return validate(new ImmutableIntegrationRevision(
        this,
        this.version,
        this.parentVersion,
        this.spec,
        this.targetState,
        this.currentState,
        value,
        this.targetMessage,
        this.timesUsed,
        this.lastUpdated,
        this.createdDate));
  }

  /**
   * Copy the current immutable object by setting a <i>present</i> value for the optional {@link IntegrationRevision#getTargetMessage() targetMessage} attribute.
   * @param value The value for targetMessage
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegrationRevision withTargetMessage(String value) {
    String newValue = Objects.requireNonNull(value, "targetMessage");
    if (Objects.equals(this.targetMessage, newValue)) return this;
    return validate(new ImmutableIntegrationRevision(
        this,
        this.version,
        this.parentVersion,
        this.spec,
        this.targetState,
        this.currentState,
        this.currentMessage,
        newValue,
        this.timesUsed,
        this.lastUpdated,
        this.createdDate));
  }

  /**
   * Copy the current immutable object by setting an optional value for the {@link IntegrationRevision#getTargetMessage() targetMessage} attribute.
   * An equality check is used on inner nullable value to prevent copying of the same value by returning {@code this}.
   * @param optional A value for targetMessage
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegrationRevision withTargetMessage(Optional<String> optional) {
    String value = optional.orElse(null);
    if (Objects.equals(this.targetMessage, value)) return this;
    return validate(new ImmutableIntegrationRevision(
        this,
        this.version,
        this.parentVersion,
        this.spec,
        this.targetState,
        this.currentState,
        this.currentMessage,
        value,
        this.timesUsed,
        this.lastUpdated,
        this.createdDate));
  }

  /**
   * Copy the current immutable object by setting a <i>present</i> value for the optional {@link IntegrationRevision#getTimesUsed() timesUsed} attribute.
   * @param value The value for timesUsed
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegrationRevision withTimesUsed(BigInteger value) {
    BigInteger newValue = Objects.requireNonNull(value, "timesUsed");
    if (Objects.equals(this.timesUsed, newValue)) return this;
    return validate(new ImmutableIntegrationRevision(
        this,
        this.version,
        this.parentVersion,
        this.spec,
        this.targetState,
        this.currentState,
        this.currentMessage,
        this.targetMessage,
        newValue,
        this.lastUpdated,
        this.createdDate));
  }

  /**
   * Copy the current immutable object by setting an optional value for the {@link IntegrationRevision#getTimesUsed() timesUsed} attribute.
   * An equality check is used on inner nullable value to prevent copying of the same value by returning {@code this}.
   * @param optional A value for timesUsed
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegrationRevision withTimesUsed(Optional<? extends BigInteger> optional) {
    BigInteger value = optional.orElse(null);
    if (Objects.equals(this.timesUsed, value)) return this;
    return validate(new ImmutableIntegrationRevision(
        this,
        this.version,
        this.parentVersion,
        this.spec,
        this.targetState,
        this.currentState,
        this.currentMessage,
        this.targetMessage,
        value,
        this.lastUpdated,
        this.createdDate));
  }

  /**
   * Copy the current immutable object by setting a <i>present</i> value for the optional {@link IntegrationRevision#getLastUpdated() lastUpdated} attribute.
   * @param value The value for lastUpdated
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegrationRevision withLastUpdated(Date value) {
    Date newValue = Objects.requireNonNull(value, "lastUpdated");
    if (this.lastUpdated == newValue) return this;
    return validate(new ImmutableIntegrationRevision(
        this,
        this.version,
        this.parentVersion,
        this.spec,
        this.targetState,
        this.currentState,
        this.currentMessage,
        this.targetMessage,
        this.timesUsed,
        newValue,
        this.createdDate));
  }

  /**
   * Copy the current immutable object by setting an optional value for the {@link IntegrationRevision#getLastUpdated() lastUpdated} attribute.
   * A shallow reference equality check is used on unboxed optional value to prevent copying of the same value by returning {@code this}.
   * @param optional A value for lastUpdated
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegrationRevision withLastUpdated(Optional<? extends Date> optional) {
    Date value = optional.orElse(null);
    if (this.lastUpdated == value) return this;
    return validate(new ImmutableIntegrationRevision(
        this,
        this.version,
        this.parentVersion,
        this.spec,
        this.targetState,
        this.currentState,
        this.currentMessage,
        this.targetMessage,
        this.timesUsed,
        value,
        this.createdDate));
  }

  /**
   * Copy the current immutable object by setting a <i>present</i> value for the optional {@link IntegrationRevision#getCreatedDate() createdDate} attribute.
   * @param value The value for createdDate
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegrationRevision withCreatedDate(Date value) {
    Date newValue = Objects.requireNonNull(value, "createdDate");
    if (this.createdDate == newValue) return this;
    return validate(new ImmutableIntegrationRevision(
        this,
        this.version,
        this.parentVersion,
        this.spec,
        this.targetState,
        this.currentState,
        this.currentMessage,
        this.targetMessage,
        this.timesUsed,
        this.lastUpdated,
        newValue));
  }

  /**
   * Copy the current immutable object by setting an optional value for the {@link IntegrationRevision#getCreatedDate() createdDate} attribute.
   * A shallow reference equality check is used on unboxed optional value to prevent copying of the same value by returning {@code this}.
   * @param optional A value for createdDate
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegrationRevision withCreatedDate(Optional<? extends Date> optional) {
    Date value = optional.orElse(null);
    if (this.createdDate == value) return this;
    return validate(new ImmutableIntegrationRevision(
        this,
        this.version,
        this.parentVersion,
        this.spec,
        this.targetState,
        this.currentState,
        this.currentMessage,
        this.targetMessage,
        this.timesUsed,
        this.lastUpdated,
        value));
  }

  /**
   * Prints the immutable value {@code IntegrationRevision} with attribute values.
   * @return A string representation of the value
   */
  @Override
  public String toString() {
    StringBuilder builder = new StringBuilder("IntegrationRevision{");
    if (version != null) {
      builder.append("version=").append(version);
    }
    if (parentVersion != null) {
      if (builder.length() > 20) builder.append(", ");
      builder.append("parentVersion=").append(parentVersion);
    }
    if (spec != null) {
      if (builder.length() > 20) builder.append(", ");
      builder.append("spec=").append(spec);
    }
    if (targetState != null) {
      if (builder.length() > 20) builder.append(", ");
      builder.append("targetState=").append(targetState);
    }
    if (currentState != null) {
      if (builder.length() > 20) builder.append(", ");
      builder.append("currentState=").append(currentState);
    }
    if (currentMessage != null) {
      if (builder.length() > 20) builder.append(", ");
      builder.append("currentMessage=").append(currentMessage);
    }
    if (targetMessage != null) {
      if (builder.length() > 20) builder.append(", ");
      builder.append("targetMessage=").append(targetMessage);
    }
    if (timesUsed != null) {
      if (builder.length() > 20) builder.append(", ");
      builder.append("timesUsed=").append(timesUsed);
    }
    if (lastUpdated != null) {
      if (builder.length() > 20) builder.append(", ");
      builder.append("lastUpdated=").append(lastUpdated);
    }
    if (createdDate != null) {
      if (builder.length() > 20) builder.append(", ");
      builder.append("createdDate=").append(createdDate);
    }
    return builder.append("}").toString();
  }

  /**
   * Construct a new immutable {@code IntegrationRevision} instance.
   * @param version The value for the {@code version} attribute
   * @param parentVersion The value for the {@code parentVersion} attribute
   * @param spec The value for the {@code spec} attribute
   * @param targetState The value for the {@code targetState} attribute
   * @param currentState The value for the {@code currentState} attribute
   * @param currentMessage The value for the {@code currentMessage} attribute
   * @param targetMessage The value for the {@code targetMessage} attribute
   * @param timesUsed The value for the {@code timesUsed} attribute
   * @param lastUpdated The value for the {@code lastUpdated} attribute
   * @param createdDate The value for the {@code createdDate} attribute
   * @return An immutable IntegrationRevision instance
   */
  public static ImmutableIntegrationRevision of(Optional<Integer> version, Integer parentVersion, IntegrationRevisionSpec spec, IntegrationRevisionState targetState, IntegrationRevisionState currentState, Optional<String> currentMessage, Optional<String> targetMessage, Optional<BigInteger> timesUsed, Optional<Date> lastUpdated, Optional<Date> createdDate) {
    return validate(new ImmutableIntegrationRevision(version, parentVersion, spec, targetState, currentState, currentMessage, targetMessage, timesUsed, lastUpdated, createdDate));
  }

  private static final Validator validator = Validation.buildDefaultValidatorFactory().getValidator();

  private static ImmutableIntegrationRevision validate(ImmutableIntegrationRevision instance) {
    Set<ConstraintViolation<ImmutableIntegrationRevision>> constraintViolations = validator.validate(instance);
    if (!constraintViolations.isEmpty()) {
      throw new ConstraintViolationException(constraintViolations);
    }
    return instance;
  }

  /**
   * Creates an immutable copy of a {@link IntegrationRevision} value.
   * Uses accessors to get values to initialize the new immutable instance.
   * If an instance is already immutable, it is returned as is.
   * @param instance The instance to copy
   * @return A copied immutable IntegrationRevision instance
   */
  public static ImmutableIntegrationRevision copyOf(IntegrationRevision instance) {
    if (instance instanceof ImmutableIntegrationRevision) {
      return (ImmutableIntegrationRevision) instance;
    }
    return new IntegrationRevision.Builder()
        .createFrom(instance)
        .build();
  }

  /**
   * Builds instances of type {@link ImmutableIntegrationRevision ImmutableIntegrationRevision}.
   * Initialize attributes and then invoke the {@link #build()} method to create an
   * immutable instance.
   * <p><em>{@code Builder} is not thread-safe and generally should not be stored in a field or collection,
   * but instead used immediately to create instances.</em>
   */
  static class Builder {
    private Integer version;
    private Integer parentVersion;
    private IntegrationRevisionSpec spec;
    private IntegrationRevisionState targetState;
    private IntegrationRevisionState currentState;
    private String currentMessage;
    private String targetMessage;
    private BigInteger timesUsed;
    private Date lastUpdated;
    private Date createdDate;

    /**
     * Creates a builder for {@link ImmutableIntegrationRevision ImmutableIntegrationRevision} instances.
     */
    Builder() {
      if (!(this instanceof IntegrationRevision.Builder)) {
        throw new UnsupportedOperationException("Use: new IntegrationRevision.Builder()");
      }
    }

    /**
     * Fill a builder with attribute values from the provided {@code IntegrationRevision} instance.
     * Regular attribute values will be replaced with those from the given instance.
     * Absent optional values will not replace present values.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    public final IntegrationRevision.Builder createFrom(IntegrationRevision instance) {
      Objects.requireNonNull(instance, "instance");
      Optional<Integer> versionOptional = instance.getVersion();
      if (versionOptional.isPresent()) {
        version(versionOptional);
      }
      Integer parentVersionValue = instance.getParentVersion();
      if (parentVersionValue != null) {
        parentVersion(parentVersionValue);
      }
      IntegrationRevisionSpec specValue = instance.getSpec();
      if (specValue != null) {
        spec(specValue);
      }
      IntegrationRevisionState targetStateValue = instance.getTargetState();
      if (targetStateValue != null) {
        targetState(targetStateValue);
      }
      IntegrationRevisionState currentStateValue = instance.getCurrentState();
      if (currentStateValue != null) {
        currentState(currentStateValue);
      }
      Optional<String> currentMessageOptional = instance.getCurrentMessage();
      if (currentMessageOptional.isPresent()) {
        currentMessage(currentMessageOptional);
      }
      Optional<String> targetMessageOptional = instance.getTargetMessage();
      if (targetMessageOptional.isPresent()) {
        targetMessage(targetMessageOptional);
      }
      Optional<BigInteger> timesUsedOptional = instance.getTimesUsed();
      if (timesUsedOptional.isPresent()) {
        timesUsed(timesUsedOptional);
      }
      Optional<Date> lastUpdatedOptional = instance.getLastUpdated();
      if (lastUpdatedOptional.isPresent()) {
        lastUpdated(lastUpdatedOptional);
      }
      Optional<Date> createdDateOptional = instance.getCreatedDate();
      if (createdDateOptional.isPresent()) {
        createdDate(createdDateOptional);
      }
      return (IntegrationRevision.Builder) this;
    }

    /**
     * Initializes the optional value {@link IntegrationRevision#getVersion() version} to version.
     * @param version The value for version
     * @return {@code this} builder for chained invocation
     */
    public final IntegrationRevision.Builder version(int version) {
      this.version = version;
      return (IntegrationRevision.Builder) this;
    }

    /**
     * Initializes the optional value {@link IntegrationRevision#getVersion() version} to version.
     * @param version The value for version
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("version")
    public final IntegrationRevision.Builder version(Optional<Integer> version) {
      this.version = version.orElse(null);
      return (IntegrationRevision.Builder) this;
    }

    /**
     * Initializes the value for the {@link IntegrationRevision#getParentVersion() parentVersion} attribute.
     * @param parentVersion The value for parentVersion (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("parentVersion")
    public final IntegrationRevision.Builder parentVersion(Integer parentVersion) {
      this.parentVersion = parentVersion;
      return (IntegrationRevision.Builder) this;
    }

    /**
     * Initializes the value for the {@link IntegrationRevision#getSpec() spec} attribute.
     * @param spec The value for spec (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("spec")
    public final IntegrationRevision.Builder spec(IntegrationRevisionSpec spec) {
      this.spec = spec;
      return (IntegrationRevision.Builder) this;
    }

    /**
     * Initializes the value for the {@link IntegrationRevision#getTargetState() targetState} attribute.
     * @param targetState The value for targetState (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("targetState")
    public final IntegrationRevision.Builder targetState(IntegrationRevisionState targetState) {
      this.targetState = targetState;
      return (IntegrationRevision.Builder) this;
    }

    /**
     * Initializes the value for the {@link IntegrationRevision#getCurrentState() currentState} attribute.
     * @param currentState The value for currentState (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("currentState")
    public final IntegrationRevision.Builder currentState(IntegrationRevisionState currentState) {
      this.currentState = currentState;
      return (IntegrationRevision.Builder) this;
    }

    /**
     * Initializes the optional value {@link IntegrationRevision#getCurrentMessage() currentMessage} to currentMessage.
     * @param currentMessage The value for currentMessage
     * @return {@code this} builder for chained invocation
     */
    public final IntegrationRevision.Builder currentMessage(String currentMessage) {
      this.currentMessage = Objects.requireNonNull(currentMessage, "currentMessage");
      return (IntegrationRevision.Builder) this;
    }

    /**
     * Initializes the optional value {@link IntegrationRevision#getCurrentMessage() currentMessage} to currentMessage.
     * @param currentMessage The value for currentMessage
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("currentMessage")
    public final IntegrationRevision.Builder currentMessage(Optional<String> currentMessage) {
      this.currentMessage = currentMessage.orElse(null);
      return (IntegrationRevision.Builder) this;
    }

    /**
     * Initializes the optional value {@link IntegrationRevision#getTargetMessage() targetMessage} to targetMessage.
     * @param targetMessage The value for targetMessage
     * @return {@code this} builder for chained invocation
     */
    public final IntegrationRevision.Builder targetMessage(String targetMessage) {
      this.targetMessage = Objects.requireNonNull(targetMessage, "targetMessage");
      return (IntegrationRevision.Builder) this;
    }

    /**
     * Initializes the optional value {@link IntegrationRevision#getTargetMessage() targetMessage} to targetMessage.
     * @param targetMessage The value for targetMessage
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("targetMessage")
    public final IntegrationRevision.Builder targetMessage(Optional<String> targetMessage) {
      this.targetMessage = targetMessage.orElse(null);
      return (IntegrationRevision.Builder) this;
    }

    /**
     * Initializes the optional value {@link IntegrationRevision#getTimesUsed() timesUsed} to timesUsed.
     * @param timesUsed The value for timesUsed
     * @return {@code this} builder for chained invocation
     */
    public final IntegrationRevision.Builder timesUsed(BigInteger timesUsed) {
      this.timesUsed = Objects.requireNonNull(timesUsed, "timesUsed");
      return (IntegrationRevision.Builder) this;
    }

    /**
     * Initializes the optional value {@link IntegrationRevision#getTimesUsed() timesUsed} to timesUsed.
     * @param timesUsed The value for timesUsed
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("timesUsed")
    public final IntegrationRevision.Builder timesUsed(Optional<? extends BigInteger> timesUsed) {
      this.timesUsed = timesUsed.orElse(null);
      return (IntegrationRevision.Builder) this;
    }

    /**
     * Initializes the optional value {@link IntegrationRevision#getLastUpdated() lastUpdated} to lastUpdated.
     * @param lastUpdated The value for lastUpdated
     * @return {@code this} builder for chained invocation
     */
    public final IntegrationRevision.Builder lastUpdated(Date lastUpdated) {
      this.lastUpdated = Objects.requireNonNull(lastUpdated, "lastUpdated");
      return (IntegrationRevision.Builder) this;
    }

    /**
     * Initializes the optional value {@link IntegrationRevision#getLastUpdated() lastUpdated} to lastUpdated.
     * @param lastUpdated The value for lastUpdated
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("lastUpdated")
    public final IntegrationRevision.Builder lastUpdated(Optional<? extends Date> lastUpdated) {
      this.lastUpdated = lastUpdated.orElse(null);
      return (IntegrationRevision.Builder) this;
    }

    /**
     * Initializes the optional value {@link IntegrationRevision#getCreatedDate() createdDate} to createdDate.
     * @param createdDate The value for createdDate
     * @return {@code this} builder for chained invocation
     */
    public final IntegrationRevision.Builder createdDate(Date createdDate) {
      this.createdDate = Objects.requireNonNull(createdDate, "createdDate");
      return (IntegrationRevision.Builder) this;
    }

    /**
     * Initializes the optional value {@link IntegrationRevision#getCreatedDate() createdDate} to createdDate.
     * @param createdDate The value for createdDate
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("createdDate")
    public final IntegrationRevision.Builder createdDate(Optional<? extends Date> createdDate) {
      this.createdDate = createdDate.orElse(null);
      return (IntegrationRevision.Builder) this;
    }

    /**
     * Builds a new {@link ImmutableIntegrationRevision ImmutableIntegrationRevision}.
     * @return An immutable instance of IntegrationRevision
     * @throws java.lang.IllegalStateException if any required attributes are missing
     */
    public ImmutableIntegrationRevision build() {
      return ImmutableIntegrationRevision.validate(new ImmutableIntegrationRevision(
          null,
          version,
          parentVersion,
          spec,
          targetState,
          currentState,
          currentMessage,
          targetMessage,
          timesUsed,
          lastUpdated,
          createdDate));
    }
  }
}
