/*
 * Copyright (C) 2016 Red Hat, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package io.syndesis.model.integration;

import com.fasterxml.jackson.annotation.JsonProperty;
import edu.umd.cs.findbugs.annotations.SuppressFBWarnings;
import io.syndesis.model.connection.Connection;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.Objects;
import java.util.Optional;
import java.util.Set;
import javax.annotation.Generated;
import javax.validation.ConstraintViolation;
import javax.validation.ConstraintViolationException;
import javax.validation.Validation;
import javax.validation.Validator;

/**
 * Immutable implementation of {@link IntegrationRevisionSpec}.
 * <p>
 * Use the builder to create immutable instances:
 * {@code new IntegrationRevisionSpec.Builder()}.
 * Use the static factory method to create immutable instances:
 * {@code ImmutableIntegrationRevisionSpec.of()}.
 */
@SuppressWarnings({"all"})
@SuppressFBWarnings
@Generated({"Immutables.generator", "IntegrationRevisionSpec"})
final class ImmutableIntegrationRevisionSpec
    implements IntegrationRevisionSpec {
  private final String configuration;
  private final List<Connection> connections;
  private final List<? extends Step> steps;

  private ImmutableIntegrationRevisionSpec(
      Optional<String> configuration,
      Iterable<? extends Connection> connections,
      List<? extends Step> steps) {
    this.configuration = configuration.orElse(null);
    this.connections = createUnmodifiableList(false, createSafeList(connections, true, false));
    this.steps = Objects.requireNonNull(steps, "steps");
    initShim.connections(this.connections);
    initShim.steps(this.steps);
    this.initShim = null;
  }

  private ImmutableIntegrationRevisionSpec(ImmutableIntegrationRevisionSpec.Builder builder) {
    this.configuration = builder.configuration;
    if (builder.connectionsIsSet()) {
      initShim.connections(createUnmodifiableList(true, builder.connections));
    }
    if (builder.steps != null) {
      initShim.steps(builder.steps);
    }
    this.connections = initShim.getConnections();
    this.steps = initShim.getSteps();
    this.initShim = null;
  }

  private ImmutableIntegrationRevisionSpec(
      ImmutableIntegrationRevisionSpec original,
      String configuration,
      List<Connection> connections,
      List<? extends Step> steps) {
    this.configuration = configuration;
    this.connections = connections;
    this.steps = steps;
    this.initShim = null;
  }

  private static final int STAGE_INITIALIZING = -1;
  private static final int STAGE_UNINITIALIZED = 0;
  private static final int STAGE_INITIALIZED = 1;
  private transient volatile InitShim initShim = new InitShim();

  private final class InitShim {
    private List<Connection> connections;
    private int connectionsBuildStage;

    List<Connection> getConnections() {
      if (connectionsBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (connectionsBuildStage == STAGE_UNINITIALIZED) {
        connectionsBuildStage = STAGE_INITIALIZING;
        this.connections = createUnmodifiableList(false, createSafeList(getConnectionsInitialize(), true, false));
        connectionsBuildStage = STAGE_INITIALIZED;
      }
      return this.connections;
    }

    void connections(List<Connection> connections) {
      this.connections = connections;
      connectionsBuildStage = STAGE_INITIALIZED;
    }
    private List<? extends Step> steps;
    private int stepsBuildStage;

    List<? extends Step> getSteps() {
      if (stepsBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (stepsBuildStage == STAGE_UNINITIALIZED) {
        stepsBuildStage = STAGE_INITIALIZING;
        this.steps = Objects.requireNonNull(getStepsInitialize(), "steps");
        stepsBuildStage = STAGE_INITIALIZED;
      }
      return this.steps;
    }

    void steps(List<? extends Step> steps) {
      this.steps = steps;
      stepsBuildStage = STAGE_INITIALIZED;
    }

    private String formatInitCycleMessage() {
      ArrayList<String> attributes = new ArrayList<String>();
      if (connectionsBuildStage == STAGE_INITIALIZING) attributes.add("connections");
      if (stepsBuildStage == STAGE_INITIALIZING) attributes.add("steps");
      return "Cannot build IntegrationRevisionSpec, attribute initializers form cycle" + attributes;
    }
  }

  private List<Connection> getConnectionsInitialize() {
    return IntegrationRevisionSpec.super.getConnections();
  }

  private List<? extends Step> getStepsInitialize() {
    return IntegrationRevisionSpec.super.getSteps();
  }

  /**
   * @return The value of the {@code configuration} attribute
   */
  @JsonProperty("configuration")
  @Override
  public Optional<String> getConfiguration() {
    return Optional.ofNullable(configuration);
  }

  /**
   * @return The value of the {@code connections} attribute
   */
  @JsonProperty("connections")
  @Override
  public List<Connection> getConnections() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getConnections()
        : this.connections;
  }

  /**
   * @return The value of the {@code steps} attribute
   */
  @JsonProperty("steps")
  @Override
  public List<? extends Step> getSteps() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getSteps()
        : this.steps;
  }

  /**
   * Copy the current immutable object by setting a <i>present</i> value for the optional {@link IntegrationRevisionSpec#getConfiguration() configuration} attribute.
   * @param value The value for configuration
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegrationRevisionSpec withConfiguration(String value) {
    String newValue = Objects.requireNonNull(value, "configuration");
    if (Objects.equals(this.configuration, newValue)) return this;
    return validate(new ImmutableIntegrationRevisionSpec(this, newValue, this.connections, this.steps));
  }

  /**
   * Copy the current immutable object by setting an optional value for the {@link IntegrationRevisionSpec#getConfiguration() configuration} attribute.
   * An equality check is used on inner nullable value to prevent copying of the same value by returning {@code this}.
   * @param optional A value for configuration
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegrationRevisionSpec withConfiguration(Optional<String> optional) {
    String value = optional.orElse(null);
    if (Objects.equals(this.configuration, value)) return this;
    return validate(new ImmutableIntegrationRevisionSpec(this, value, this.connections, this.steps));
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link IntegrationRevisionSpec#getConnections() connections}.
   * @param elements The elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegrationRevisionSpec withConnections(Connection... elements) {
    List<Connection> newValue = createUnmodifiableList(false, createSafeList(Arrays.asList(elements), true, false));
    return validate(new ImmutableIntegrationRevisionSpec(this, this.configuration, newValue, this.steps));
  }

  /**
   * Copy the current immutable object with elements that replace the content of {@link IntegrationRevisionSpec#getConnections() connections}.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param elements An iterable of connections elements to set
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegrationRevisionSpec withConnections(Iterable<? extends Connection> elements) {
    if (this.connections == elements) return this;
    List<Connection> newValue = createUnmodifiableList(false, createSafeList(elements, true, false));
    return validate(new ImmutableIntegrationRevisionSpec(this, this.configuration, newValue, this.steps));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IntegrationRevisionSpec#getSteps() steps} attribute.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for steps
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIntegrationRevisionSpec withSteps(List<? extends Step> value) {
    if (this.steps == value) return this;
    List<? extends Step> newValue = Objects.requireNonNull(value, "steps");
    return validate(new ImmutableIntegrationRevisionSpec(this, this.configuration, this.connections, newValue));
  }

  /**
   * This instance is equal to all instances of {@code ImmutableIntegrationRevisionSpec} that have equal attribute values.
   * @return {@code true} if {@code this} is equal to {@code another} instance
   */
  @Override
  public boolean equals(Object another) {
    if (this == another) return true;
    return another instanceof ImmutableIntegrationRevisionSpec
        && equalTo((ImmutableIntegrationRevisionSpec) another);
  }

  private boolean equalTo(ImmutableIntegrationRevisionSpec another) {
    return Objects.equals(configuration, another.configuration)
        && connections.equals(another.connections)
        && steps.equals(another.steps);
  }

  /**
   * Computes a hash code from attributes: {@code configuration}, {@code connections}, {@code steps}.
   * @return hashCode value
   */
  @Override
  public int hashCode() {
    int h = 5381;
    h += (h << 5) + Objects.hashCode(configuration);
    h += (h << 5) + connections.hashCode();
    h += (h << 5) + steps.hashCode();
    return h;
  }

  /**
   * Prints the immutable value {@code IntegrationRevisionSpec} with attribute values.
   * @return A string representation of the value
   */
  @Override
  public String toString() {
    StringBuilder builder = new StringBuilder("IntegrationRevisionSpec{");
    if (configuration != null) {
      builder.append("configuration=").append(configuration);
    }
    if (builder.length() > 24) builder.append(", ");
    builder.append("connections=").append(connections);
    builder.append(", ");
    builder.append("steps=").append(steps);
    return builder.append("}").toString();
  }

  /**
   * Construct a new immutable {@code IntegrationRevisionSpec} instance.
   * @param configuration The value for the {@code configuration} attribute
   * @param connections The value for the {@code connections} attribute
   * @param steps The value for the {@code steps} attribute
   * @return An immutable IntegrationRevisionSpec instance
   */
  public static ImmutableIntegrationRevisionSpec of(Optional<String> configuration, List<Connection> connections, List<? extends Step> steps) {
    return of(configuration, (Iterable<? extends Connection>) connections, steps);
  }

  /**
   * Construct a new immutable {@code IntegrationRevisionSpec} instance.
   * @param configuration The value for the {@code configuration} attribute
   * @param connections The value for the {@code connections} attribute
   * @param steps The value for the {@code steps} attribute
   * @return An immutable IntegrationRevisionSpec instance
   */
  public static ImmutableIntegrationRevisionSpec of(Optional<String> configuration, Iterable<? extends Connection> connections, List<? extends Step> steps) {
    return validate(new ImmutableIntegrationRevisionSpec(configuration, connections, steps));
  }

  private static final Validator validator = Validation.buildDefaultValidatorFactory().getValidator();

  private static ImmutableIntegrationRevisionSpec validate(ImmutableIntegrationRevisionSpec instance) {
    Set<ConstraintViolation<ImmutableIntegrationRevisionSpec>> constraintViolations = validator.validate(instance);
    if (!constraintViolations.isEmpty()) {
      throw new ConstraintViolationException(constraintViolations);
    }
    return instance;
  }

  /**
   * Creates an immutable copy of a {@link IntegrationRevisionSpec} value.
   * Uses accessors to get values to initialize the new immutable instance.
   * If an instance is already immutable, it is returned as is.
   * @param instance The instance to copy
   * @return A copied immutable IntegrationRevisionSpec instance
   */
  public static ImmutableIntegrationRevisionSpec copyOf(IntegrationRevisionSpec instance) {
    if (instance instanceof ImmutableIntegrationRevisionSpec) {
      return (ImmutableIntegrationRevisionSpec) instance;
    }
    return new IntegrationRevisionSpec.Builder()
        .createFrom(instance)
        .build();
  }

  /**
   * Builds instances of type {@link ImmutableIntegrationRevisionSpec ImmutableIntegrationRevisionSpec}.
   * Initialize attributes and then invoke the {@link #build()} method to create an
   * immutable instance.
   * <p><em>{@code Builder} is not thread-safe and generally should not be stored in a field or collection,
   * but instead used immediately to create instances.</em>
   */
  static class Builder {
    private static final long OPT_BIT_CONNECTIONS = 0x1L;
    private long optBits;

    private String configuration;
    private List<Connection> connections = new ArrayList<Connection>();
    private List<? extends Step> steps;

    /**
     * Creates a builder for {@link ImmutableIntegrationRevisionSpec ImmutableIntegrationRevisionSpec} instances.
     */
    Builder() {
      if (!(this instanceof IntegrationRevisionSpec.Builder)) {
        throw new UnsupportedOperationException("Use: new IntegrationRevisionSpec.Builder()");
      }
    }

    /**
     * Fill a builder with attribute values from the provided {@code IntegrationRevisionSpec} instance.
     * Regular attribute values will be replaced with those from the given instance.
     * Absent optional values will not replace present values.
     * Collection elements and entries will be added, not replaced.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    public final IntegrationRevisionSpec.Builder createFrom(IntegrationRevisionSpec instance) {
      Objects.requireNonNull(instance, "instance");
      Optional<String> configurationOptional = instance.getConfiguration();
      if (configurationOptional.isPresent()) {
        configuration(configurationOptional);
      }
      addAllConnections(instance.getConnections());
      steps(instance.getSteps());
      return (IntegrationRevisionSpec.Builder) this;
    }

    /**
     * Initializes the optional value {@link IntegrationRevisionSpec#getConfiguration() configuration} to configuration.
     * @param configuration The value for configuration
     * @return {@code this} builder for chained invocation
     */
    public final IntegrationRevisionSpec.Builder configuration(String configuration) {
      this.configuration = Objects.requireNonNull(configuration, "configuration");
      return (IntegrationRevisionSpec.Builder) this;
    }

    /**
     * Initializes the optional value {@link IntegrationRevisionSpec#getConfiguration() configuration} to configuration.
     * @param configuration The value for configuration
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("configuration")
    public final IntegrationRevisionSpec.Builder configuration(Optional<String> configuration) {
      this.configuration = configuration.orElse(null);
      return (IntegrationRevisionSpec.Builder) this;
    }

    /**
     * Adds one element to {@link IntegrationRevisionSpec#getConnections() connections} list.
     * @param element A connections element
     * @return {@code this} builder for use in a chained invocation
     */
    public final IntegrationRevisionSpec.Builder addConnection(Connection element) {
      this.connections.add(Objects.requireNonNull(element, "connections element"));
      optBits |= OPT_BIT_CONNECTIONS;
      return (IntegrationRevisionSpec.Builder) this;
    }

    /**
     * Adds elements to {@link IntegrationRevisionSpec#getConnections() connections} list.
     * @param elements An array of connections elements
     * @return {@code this} builder for use in a chained invocation
     */
    public final IntegrationRevisionSpec.Builder addConnection(Connection... elements) {
      for (Connection element : elements) {
        this.connections.add(Objects.requireNonNull(element, "connections element"));
      }
      optBits |= OPT_BIT_CONNECTIONS;
      return (IntegrationRevisionSpec.Builder) this;
    }

    /**
     * Sets or replaces all elements for {@link IntegrationRevisionSpec#getConnections() connections} list.
     * @param elements An iterable of connections elements
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("connections")
    public final IntegrationRevisionSpec.Builder connections(Iterable<? extends Connection> elements) {
      this.connections.clear();
      return addAllConnections(elements);
    }

    /**
     * Adds elements to {@link IntegrationRevisionSpec#getConnections() connections} list.
     * @param elements An iterable of connections elements
     * @return {@code this} builder for use in a chained invocation
     */
    public final IntegrationRevisionSpec.Builder addAllConnections(Iterable<? extends Connection> elements) {
      for (Connection element : elements) {
        this.connections.add(Objects.requireNonNull(element, "connections element"));
      }
      optBits |= OPT_BIT_CONNECTIONS;
      return (IntegrationRevisionSpec.Builder) this;
    }

    /**
     * Initializes the value for the {@link IntegrationRevisionSpec#getSteps() steps} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link IntegrationRevisionSpec#getSteps() steps}.</em>
     * @param steps The value for steps 
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("steps")
    public final IntegrationRevisionSpec.Builder steps(List<? extends Step> steps) {
      this.steps = Objects.requireNonNull(steps, "steps");
      return (IntegrationRevisionSpec.Builder) this;
    }

    /**
     * Builds a new {@link ImmutableIntegrationRevisionSpec ImmutableIntegrationRevisionSpec}.
     * @return An immutable instance of IntegrationRevisionSpec
     * @throws java.lang.IllegalStateException if any required attributes are missing
     */
    public ImmutableIntegrationRevisionSpec build() {
      return ImmutableIntegrationRevisionSpec.validate(new ImmutableIntegrationRevisionSpec(this));
    }

    private boolean connectionsIsSet() {
      return (optBits & OPT_BIT_CONNECTIONS) != 0;
    }
  }

  private static <T> List<T> createSafeList(Iterable<? extends T> iterable, boolean checkNulls, boolean skipNulls) {
    ArrayList<T> list;
    if (iterable instanceof Collection<?>) {
      int size = ((Collection<?>) iterable).size();
      if (size == 0) return Collections.emptyList();
      list = new ArrayList<T>();
    } else {
      list = new ArrayList<T>();
    }
    for (T element : iterable) {
      if (skipNulls && element == null) continue;
      if (checkNulls) Objects.requireNonNull(element, "element");
      list.add(element);
    }
    return list;
  }

  private static <T> List<T> createUnmodifiableList(boolean clone, List<T> list) {
    switch(list.size()) {
    case 0: return Collections.emptyList();
    case 1: return Collections.singletonList(list.get(0));
    default:
      if (clone) {
        return Collections.unmodifiableList(new ArrayList<T>(list));
      } else {
        if (list instanceof ArrayList<?>) {
          ((ArrayList<?>) list).trimToSize();
        }
        return Collections.unmodifiableList(list);
      }
    }
  }
}
