/**
 * Copyright (C) 2016 Red Hat, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package io.syndesis.model.integration;

import com.fasterxml.jackson.annotation.JsonProperty;
import edu.umd.cs.findbugs.annotations.SuppressFBWarnings;
import io.syndesis.model.environment.Environment;
import java.io.ObjectStreamException;
import java.util.Objects;
import java.util.Optional;
import java.util.Set;
import javax.annotation.Generated;
import javax.validation.ConstraintViolation;
import javax.validation.ConstraintViolationException;
import javax.validation.Validation;
import javax.validation.Validator;

/**
 * Immutable implementation of {@link IntegrationRuntime}.
 * <p>
 * Use the builder to create immutable instances:
 * {@code new IntegrationRuntime.Builder()}.
 * Use the static factory method to create immutable instances:
 * {@code ImmutableIntegrationRuntime.of()}.
 */
@SuppressWarnings({"all"})
@SuppressFBWarnings
@Generated({"Immutables.generator", "IntegrationRuntime"})
final class ImmutableIntegrationRuntime implements IntegrationRuntime {
  private final String state;
  private final Integration integration;
  private final Environment environment;
  private final String id;

  private ImmutableIntegrationRuntime(
      String state,
      Integration integration,
      Environment environment,
      Optional<String> id) {
    this.state = state;
    this.integration = integration;
    this.environment = environment;
    this.id = id.orElse(null);
  }

  private ImmutableIntegrationRuntime(
      ImmutableIntegrationRuntime original,
      String state,
      Integration integration,
      Environment environment,
      String id) {
    this.state = state;
    this.integration = integration;
    this.environment = environment;
    this.id = id;
  }

  /**
   * @return The value of the {@code state} attribute
   */
  @JsonProperty("state")
  @Override
  public String getState() {
    return state;
  }

  /**
   * @return The value of the {@code integration} attribute
   */
  @JsonProperty("integration")
  @Override
  public Integration getIntegration() {
    return integration;
  }

  /**
   * @return The value of the {@code environment} attribute
   */
  @JsonProperty("environment")
  @Override
  public Environment getEnvironment() {
    return environment;
  }

  /**
   * @return The value of the {@code id} attribute
   */
  @JsonProperty("id")
  @Override
  public Optional<String> getId() {
    return Optional.ofNullable(id);
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IntegrationRuntime#getState() state} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for state (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIntegrationRuntime withState(String value) {
    if (Objects.equals(this.state, value)) return this;
    return validate(new ImmutableIntegrationRuntime(this, value, this.integration, this.environment, this.id));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IntegrationRuntime#getIntegration() integration} attribute.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for integration (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIntegrationRuntime withIntegration(Integration value) {
    if (this.integration == value) return this;
    return validate(new ImmutableIntegrationRuntime(this, this.state, value, this.environment, this.id));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IntegrationRuntime#getEnvironment() environment} attribute.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for environment (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIntegrationRuntime withEnvironment(Environment value) {
    if (this.environment == value) return this;
    return validate(new ImmutableIntegrationRuntime(this, this.state, this.integration, value, this.id));
  }

  /**
   * Copy the current immutable object by setting a <i>present</i> value for the optional {@link IntegrationRuntime#getId() id} attribute.
   * @param value The value for id
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegrationRuntime withId(String value) {
    String newValue = Objects.requireNonNull(value, "id");
    if (Objects.equals(this.id, newValue)) return this;
    return validate(new ImmutableIntegrationRuntime(this, this.state, this.integration, this.environment, newValue));
  }

  /**
   * Copy the current immutable object by setting an optional value for the {@link IntegrationRuntime#getId() id} attribute.
   * An equality check is used on inner nullable value to prevent copying of the same value by returning {@code this}.
   * @param optional A value for id
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegrationRuntime withId(Optional<String> optional) {
    String value = optional.orElse(null);
    if (Objects.equals(this.id, value)) return this;
    return validate(new ImmutableIntegrationRuntime(this, this.state, this.integration, this.environment, value));
  }

  /**
   * This instance is equal to all instances of {@code ImmutableIntegrationRuntime} that have equal attribute values.
   * @return {@code true} if {@code this} is equal to {@code another} instance
   */
  @Override
  public boolean equals(Object another) {
    if (this == another) return true;
    return another instanceof ImmutableIntegrationRuntime
        && equalTo((ImmutableIntegrationRuntime) another);
  }

  private boolean equalTo(ImmutableIntegrationRuntime another) {
    return Objects.equals(state, another.state)
        && Objects.equals(integration, another.integration)
        && Objects.equals(environment, another.environment)
        && Objects.equals(id, another.id);
  }

  /**
   * Computes a hash code from attributes: {@code state}, {@code integration}, {@code environment}, {@code id}.
   * @return hashCode value
   */
  @Override
  public int hashCode() {
    int h = 5381;
    h += (h << 5) + Objects.hashCode(state);
    h += (h << 5) + Objects.hashCode(integration);
    h += (h << 5) + Objects.hashCode(environment);
    h += (h << 5) + Objects.hashCode(id);
    return h;
  }

  /**
   * Prints the immutable value {@code IntegrationRuntime} with attribute values.
   * @return A string representation of the value
   */
  @Override
  public String toString() {
    StringBuilder builder = new StringBuilder("IntegrationRuntime{");
    if (state != null) {
      builder.append("state=").append(state);
    }
    if (integration != null) {
      if (builder.length() > 19) builder.append(", ");
      builder.append("integration=").append(integration);
    }
    if (environment != null) {
      if (builder.length() > 19) builder.append(", ");
      builder.append("environment=").append(environment);
    }
    if (id != null) {
      if (builder.length() > 19) builder.append(", ");
      builder.append("id=").append(id);
    }
    return builder.append("}").toString();
  }

  /**
   * Construct a new immutable {@code IntegrationRuntime} instance.
   * @param state The value for the {@code state} attribute
   * @param integration The value for the {@code integration} attribute
   * @param environment The value for the {@code environment} attribute
   * @param id The value for the {@code id} attribute
   * @return An immutable IntegrationRuntime instance
   */
  public static ImmutableIntegrationRuntime of(String state, Integration integration, Environment environment, Optional<String> id) {
    return validate(new ImmutableIntegrationRuntime(state, integration, environment, id));
  }

  private static final Validator validator = Validation.buildDefaultValidatorFactory().getValidator();

  private static ImmutableIntegrationRuntime validate(ImmutableIntegrationRuntime instance) {
    Set<ConstraintViolation<ImmutableIntegrationRuntime>> constraintViolations = validator.validate(instance);
    if (!constraintViolations.isEmpty()) {
      throw new ConstraintViolationException(constraintViolations);
    }
    return instance;
  }

  /**
   * Creates an immutable copy of a {@link IntegrationRuntime} value.
   * Uses accessors to get values to initialize the new immutable instance.
   * If an instance is already immutable, it is returned as is.
   * @param instance The instance to copy
   * @return A copied immutable IntegrationRuntime instance
   */
  public static ImmutableIntegrationRuntime copyOf(IntegrationRuntime instance) {
    if (instance instanceof ImmutableIntegrationRuntime) {
      return (ImmutableIntegrationRuntime) instance;
    }
    return new IntegrationRuntime.Builder()
        .createFrom(instance)
        .build();
  }

  private Object readResolve() throws ObjectStreamException {
    return validate(this);
  }

  /**
   * Builds instances of type {@link ImmutableIntegrationRuntime ImmutableIntegrationRuntime}.
   * Initialize attributes and then invoke the {@link #build()} method to create an
   * immutable instance.
   * <p><em>{@code Builder} is not thread-safe and generally should not be stored in a field or collection,
   * but instead used immediately to create instances.</em>
   */
  static class Builder {
    private String state;
    private Integration integration;
    private Environment environment;
    private String id;

    /**
     * Creates a builder for {@link ImmutableIntegrationRuntime ImmutableIntegrationRuntime} instances.
     */
    Builder() {
      if (!(this instanceof IntegrationRuntime.Builder)) {
        throw new UnsupportedOperationException("Use: new IntegrationRuntime.Builder()");
      }
    }

    /**
     * Fill a builder with attribute values from the provided {@code IntegrationRuntime} instance.
     * Regular attribute values will be replaced with those from the given instance.
     * Absent optional values will not replace present values.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    public final IntegrationRuntime.Builder createFrom(IntegrationRuntime instance) {
      Objects.requireNonNull(instance, "instance");
      String stateValue = instance.getState();
      if (stateValue != null) {
        state(stateValue);
      }
      Integration integrationValue = instance.getIntegration();
      if (integrationValue != null) {
        integration(integrationValue);
      }
      Environment environmentValue = instance.getEnvironment();
      if (environmentValue != null) {
        environment(environmentValue);
      }
      Optional<String> idOptional = instance.getId();
      if (idOptional.isPresent()) {
        id(idOptional);
      }
      return (IntegrationRuntime.Builder) this;
    }

    /**
     * Initializes the value for the {@link IntegrationRuntime#getState() state} attribute.
     * @param state The value for state (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("state")
    public final IntegrationRuntime.Builder state(String state) {
      this.state = state;
      return (IntegrationRuntime.Builder) this;
    }

    /**
     * Initializes the value for the {@link IntegrationRuntime#getIntegration() integration} attribute.
     * @param integration The value for integration (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("integration")
    public final IntegrationRuntime.Builder integration(Integration integration) {
      this.integration = integration;
      return (IntegrationRuntime.Builder) this;
    }

    /**
     * Initializes the value for the {@link IntegrationRuntime#getEnvironment() environment} attribute.
     * @param environment The value for environment (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("environment")
    public final IntegrationRuntime.Builder environment(Environment environment) {
      this.environment = environment;
      return (IntegrationRuntime.Builder) this;
    }

    /**
     * Initializes the optional value {@link IntegrationRuntime#getId() id} to id.
     * @param id The value for id
     * @return {@code this} builder for chained invocation
     */
    public final IntegrationRuntime.Builder id(String id) {
      this.id = Objects.requireNonNull(id, "id");
      return (IntegrationRuntime.Builder) this;
    }

    /**
     * Initializes the optional value {@link IntegrationRuntime#getId() id} to id.
     * @param id The value for id
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("id")
    public final IntegrationRuntime.Builder id(Optional<String> id) {
      this.id = id.orElse(null);
      return (IntegrationRuntime.Builder) this;
    }

    /**
     * Builds a new {@link ImmutableIntegrationRuntime ImmutableIntegrationRuntime}.
     * @return An immutable instance of IntegrationRuntime
     * @throws java.lang.IllegalStateException if any required attributes are missing
     */
    public ImmutableIntegrationRuntime build() {
      return ImmutableIntegrationRuntime.validate(new ImmutableIntegrationRuntime(null, state, integration, environment, id));
    }
  }
}
