/*
 * Copyright (C) 2016 Red Hat, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package io.syndesis.model.integration;

import com.fasterxml.jackson.annotation.JsonProperty;
import edu.umd.cs.findbugs.annotations.SuppressFBWarnings;
import java.util.Objects;
import java.util.Optional;
import java.util.Set;
import javax.annotation.Generated;
import javax.validation.ConstraintViolation;
import javax.validation.ConstraintViolationException;
import javax.validation.Validation;
import javax.validation.Validator;

/**
 * Immutable implementation of {@link IntegrationStatus}.
 * <p>
 * Use the builder to create immutable instances:
 * {@code new IntegrationStatus.Builder()}.
 * Use the static factory method to create immutable instances:
 * {@code ImmutableIntegrationStatus.of()}.
 */
@SuppressWarnings({"all"})
@SuppressFBWarnings
@Generated({"Immutables.generator", "IntegrationStatus"})
final class ImmutableIntegrationStatus implements IntegrationStatus {
  private final IntegrationRevisionState targetState;
  private final IntegrationRevisionState currentState;
  private final String currentMessage;
  private final String targetMessage;

  private ImmutableIntegrationStatus(
      IntegrationRevisionState targetState,
      IntegrationRevisionState currentState,
      Optional<String> currentMessage,
      Optional<String> targetMessage) {
    this.targetState = targetState;
    this.currentState = currentState;
    this.currentMessage = currentMessage.orElse(null);
    this.targetMessage = targetMessage.orElse(null);
  }

  private ImmutableIntegrationStatus(
      ImmutableIntegrationStatus original,
      IntegrationRevisionState targetState,
      IntegrationRevisionState currentState,
      String currentMessage,
      String targetMessage) {
    this.targetState = targetState;
    this.currentState = currentState;
    this.currentMessage = currentMessage;
    this.targetMessage = targetMessage;
  }

  /**
   * @return The value of the {@code targetState} attribute
   */
  @JsonProperty("targetState")
  @Override
  public IntegrationRevisionState getTargetState() {
    return targetState;
  }

  /**
   * @return The value of the {@code currentState} attribute
   */
  @JsonProperty("currentState")
  @Override
  public IntegrationRevisionState getCurrentState() {
    return currentState;
  }

  /**
   * @return The value of the {@code currentMessage} attribute
   */
  @JsonProperty("currentMessage")
  @Override
  public Optional<String> getCurrentMessage() {
    return Optional.ofNullable(currentMessage);
  }

  /**
   * @return The value of the {@code targetMessage} attribute
   */
  @JsonProperty("targetMessage")
  @Override
  public Optional<String> getTargetMessage() {
    return Optional.ofNullable(targetMessage);
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IntegrationStatus#getTargetState() targetState} attribute.
   * A value equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for targetState (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIntegrationStatus withTargetState(IntegrationRevisionState value) {
    if (this.targetState == value) return this;
    return validate(new ImmutableIntegrationStatus(this, value, this.currentState, this.currentMessage, this.targetMessage));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link IntegrationStatus#getCurrentState() currentState} attribute.
   * A value equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for currentState (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableIntegrationStatus withCurrentState(IntegrationRevisionState value) {
    if (this.currentState == value) return this;
    return validate(new ImmutableIntegrationStatus(this, this.targetState, value, this.currentMessage, this.targetMessage));
  }

  /**
   * Copy the current immutable object by setting a <i>present</i> value for the optional {@link IntegrationStatus#getCurrentMessage() currentMessage} attribute.
   * @param value The value for currentMessage
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegrationStatus withCurrentMessage(String value) {
    String newValue = Objects.requireNonNull(value, "currentMessage");
    if (Objects.equals(this.currentMessage, newValue)) return this;
    return validate(new ImmutableIntegrationStatus(this, this.targetState, this.currentState, newValue, this.targetMessage));
  }

  /**
   * Copy the current immutable object by setting an optional value for the {@link IntegrationStatus#getCurrentMessage() currentMessage} attribute.
   * An equality check is used on inner nullable value to prevent copying of the same value by returning {@code this}.
   * @param optional A value for currentMessage
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegrationStatus withCurrentMessage(Optional<String> optional) {
    String value = optional.orElse(null);
    if (Objects.equals(this.currentMessage, value)) return this;
    return validate(new ImmutableIntegrationStatus(this, this.targetState, this.currentState, value, this.targetMessage));
  }

  /**
   * Copy the current immutable object by setting a <i>present</i> value for the optional {@link IntegrationStatus#getTargetMessage() targetMessage} attribute.
   * @param value The value for targetMessage
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegrationStatus withTargetMessage(String value) {
    String newValue = Objects.requireNonNull(value, "targetMessage");
    if (Objects.equals(this.targetMessage, newValue)) return this;
    return validate(new ImmutableIntegrationStatus(this, this.targetState, this.currentState, this.currentMessage, newValue));
  }

  /**
   * Copy the current immutable object by setting an optional value for the {@link IntegrationStatus#getTargetMessage() targetMessage} attribute.
   * An equality check is used on inner nullable value to prevent copying of the same value by returning {@code this}.
   * @param optional A value for targetMessage
   * @return A modified copy of {@code this} object
   */
  public final ImmutableIntegrationStatus withTargetMessage(Optional<String> optional) {
    String value = optional.orElse(null);
    if (Objects.equals(this.targetMessage, value)) return this;
    return validate(new ImmutableIntegrationStatus(this, this.targetState, this.currentState, this.currentMessage, value));
  }

  /**
   * This instance is equal to all instances of {@code ImmutableIntegrationStatus} that have equal attribute values.
   * @return {@code true} if {@code this} is equal to {@code another} instance
   */
  @Override
  public boolean equals(Object another) {
    if (this == another) return true;
    return another instanceof ImmutableIntegrationStatus
        && equalTo((ImmutableIntegrationStatus) another);
  }

  private boolean equalTo(ImmutableIntegrationStatus another) {
    return Objects.equals(targetState, another.targetState)
        && Objects.equals(currentState, another.currentState)
        && Objects.equals(currentMessage, another.currentMessage)
        && Objects.equals(targetMessage, another.targetMessage);
  }

  /**
   * Computes a hash code from attributes: {@code targetState}, {@code currentState}, {@code currentMessage}, {@code targetMessage}.
   * @return hashCode value
   */
  @Override
  public int hashCode() {
    int h = 5381;
    h += (h << 5) + Objects.hashCode(targetState);
    h += (h << 5) + Objects.hashCode(currentState);
    h += (h << 5) + Objects.hashCode(currentMessage);
    h += (h << 5) + Objects.hashCode(targetMessage);
    return h;
  }

  /**
   * Prints the immutable value {@code IntegrationStatus} with attribute values.
   * @return A string representation of the value
   */
  @Override
  public String toString() {
    StringBuilder builder = new StringBuilder("IntegrationStatus{");
    if (targetState != null) {
      builder.append("targetState=").append(targetState);
    }
    if (currentState != null) {
      if (builder.length() > 18) builder.append(", ");
      builder.append("currentState=").append(currentState);
    }
    if (currentMessage != null) {
      if (builder.length() > 18) builder.append(", ");
      builder.append("currentMessage=").append(currentMessage);
    }
    if (targetMessage != null) {
      if (builder.length() > 18) builder.append(", ");
      builder.append("targetMessage=").append(targetMessage);
    }
    return builder.append("}").toString();
  }

  /**
   * Construct a new immutable {@code IntegrationStatus} instance.
   * @param targetState The value for the {@code targetState} attribute
   * @param currentState The value for the {@code currentState} attribute
   * @param currentMessage The value for the {@code currentMessage} attribute
   * @param targetMessage The value for the {@code targetMessage} attribute
   * @return An immutable IntegrationStatus instance
   */
  public static ImmutableIntegrationStatus of(IntegrationRevisionState targetState, IntegrationRevisionState currentState, Optional<String> currentMessage, Optional<String> targetMessage) {
    return validate(new ImmutableIntegrationStatus(targetState, currentState, currentMessage, targetMessage));
  }

  private static final Validator validator = Validation.buildDefaultValidatorFactory().getValidator();

  private static ImmutableIntegrationStatus validate(ImmutableIntegrationStatus instance) {
    Set<ConstraintViolation<ImmutableIntegrationStatus>> constraintViolations = validator.validate(instance);
    if (!constraintViolations.isEmpty()) {
      throw new ConstraintViolationException(constraintViolations);
    }
    return instance;
  }

  /**
   * Creates an immutable copy of a {@link IntegrationStatus} value.
   * Uses accessors to get values to initialize the new immutable instance.
   * If an instance is already immutable, it is returned as is.
   * @param instance The instance to copy
   * @return A copied immutable IntegrationStatus instance
   */
  public static ImmutableIntegrationStatus copyOf(IntegrationStatus instance) {
    if (instance instanceof ImmutableIntegrationStatus) {
      return (ImmutableIntegrationStatus) instance;
    }
    return new IntegrationStatus.Builder()
        .createFrom(instance)
        .build();
  }

  /**
   * Builds instances of type {@link ImmutableIntegrationStatus ImmutableIntegrationStatus}.
   * Initialize attributes and then invoke the {@link #build()} method to create an
   * immutable instance.
   * <p><em>{@code Builder} is not thread-safe and generally should not be stored in a field or collection,
   * but instead used immediately to create instances.</em>
   */
  static class Builder {
    private IntegrationRevisionState targetState;
    private IntegrationRevisionState currentState;
    private String currentMessage;
    private String targetMessage;

    /**
     * Creates a builder for {@link ImmutableIntegrationStatus ImmutableIntegrationStatus} instances.
     */
    Builder() {
      if (!(this instanceof IntegrationStatus.Builder)) {
        throw new UnsupportedOperationException("Use: new IntegrationStatus.Builder()");
      }
    }

    /**
     * Fill a builder with attribute values from the provided {@code IntegrationStatus} instance.
     * Regular attribute values will be replaced with those from the given instance.
     * Absent optional values will not replace present values.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    public final IntegrationStatus.Builder createFrom(IntegrationStatus instance) {
      Objects.requireNonNull(instance, "instance");
      IntegrationRevisionState targetStateValue = instance.getTargetState();
      if (targetStateValue != null) {
        targetState(targetStateValue);
      }
      IntegrationRevisionState currentStateValue = instance.getCurrentState();
      if (currentStateValue != null) {
        currentState(currentStateValue);
      }
      Optional<String> currentMessageOptional = instance.getCurrentMessage();
      if (currentMessageOptional.isPresent()) {
        currentMessage(currentMessageOptional);
      }
      Optional<String> targetMessageOptional = instance.getTargetMessage();
      if (targetMessageOptional.isPresent()) {
        targetMessage(targetMessageOptional);
      }
      return (IntegrationStatus.Builder) this;
    }

    /**
     * Initializes the value for the {@link IntegrationStatus#getTargetState() targetState} attribute.
     * @param targetState The value for targetState (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("targetState")
    public final IntegrationStatus.Builder targetState(IntegrationRevisionState targetState) {
      this.targetState = targetState;
      return (IntegrationStatus.Builder) this;
    }

    /**
     * Initializes the value for the {@link IntegrationStatus#getCurrentState() currentState} attribute.
     * @param currentState The value for currentState (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("currentState")
    public final IntegrationStatus.Builder currentState(IntegrationRevisionState currentState) {
      this.currentState = currentState;
      return (IntegrationStatus.Builder) this;
    }

    /**
     * Initializes the optional value {@link IntegrationStatus#getCurrentMessage() currentMessage} to currentMessage.
     * @param currentMessage The value for currentMessage
     * @return {@code this} builder for chained invocation
     */
    public final IntegrationStatus.Builder currentMessage(String currentMessage) {
      this.currentMessage = Objects.requireNonNull(currentMessage, "currentMessage");
      return (IntegrationStatus.Builder) this;
    }

    /**
     * Initializes the optional value {@link IntegrationStatus#getCurrentMessage() currentMessage} to currentMessage.
     * @param currentMessage The value for currentMessage
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("currentMessage")
    public final IntegrationStatus.Builder currentMessage(Optional<String> currentMessage) {
      this.currentMessage = currentMessage.orElse(null);
      return (IntegrationStatus.Builder) this;
    }

    /**
     * Initializes the optional value {@link IntegrationStatus#getTargetMessage() targetMessage} to targetMessage.
     * @param targetMessage The value for targetMessage
     * @return {@code this} builder for chained invocation
     */
    public final IntegrationStatus.Builder targetMessage(String targetMessage) {
      this.targetMessage = Objects.requireNonNull(targetMessage, "targetMessage");
      return (IntegrationStatus.Builder) this;
    }

    /**
     * Initializes the optional value {@link IntegrationStatus#getTargetMessage() targetMessage} to targetMessage.
     * @param targetMessage The value for targetMessage
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("targetMessage")
    public final IntegrationStatus.Builder targetMessage(Optional<String> targetMessage) {
      this.targetMessage = targetMessage.orElse(null);
      return (IntegrationStatus.Builder) this;
    }

    /**
     * Builds a new {@link ImmutableIntegrationStatus ImmutableIntegrationStatus}.
     * @return An immutable instance of IntegrationStatus
     * @throws java.lang.IllegalStateException if any required attributes are missing
     */
    public ImmutableIntegrationStatus build() {
      return ImmutableIntegrationStatus.validate(new ImmutableIntegrationStatus(null, targetState, currentState, currentMessage, targetMessage));
    }
  }
}
