/*
 * Copyright (C) 2016 Red Hat, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package io.syndesis.model.integration;

import com.fasterxml.jackson.annotation.JsonProperty;
import edu.umd.cs.findbugs.annotations.SuppressFBWarnings;
import io.syndesis.model.Kind;
import io.syndesis.model.WithKind;
import io.syndesis.model.connection.Action;
import io.syndesis.model.connection.Connection;
import java.io.ObjectStreamException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.Objects;
import java.util.Optional;
import java.util.Set;
import javax.annotation.Generated;
import javax.validation.ConstraintViolation;
import javax.validation.ConstraintViolationException;
import javax.validation.Validation;
import javax.validation.Validator;

/**
 * Immutable implementation of {@link SimpleStep}.
 * <p>
 * Use the builder to create immutable instances:
 * {@code new SimpleStep.Builder()}.
 * Use the static factory method to create immutable instances:
 * {@code ImmutableSimpleStep.of()}.
 */
@SuppressWarnings({"all"})
@SuppressFBWarnings
@Generated({"Immutables.generator", "SimpleStep"})
final class ImmutableSimpleStep implements SimpleStep {
  private final Kind kind;
  private final Action action;
  private final Connection connection;
  private final String stepKind;
  private final Map<String, String> configuredProperties;
  private final String name;
  private final String id;

  private ImmutableSimpleStep(
      Kind kind,
      Optional<Action> action,
      Optional<Connection> connection,
      String stepKind,
      Map<String, ? extends String> configuredProperties,
      String name,
      Optional<String> id) {
    this.kind = Objects.requireNonNull(kind, "kind");
    this.action = action.orElse(null);
    this.connection = connection.orElse(null);
    this.stepKind = stepKind;
    this.configuredProperties = createUnmodifiableMap(true, false, configuredProperties);
    this.name = name;
    this.id = id.orElse(null);
    initShim.kind(this.kind);
    initShim.configuredProperties(this.configuredProperties);
    this.initShim = null;
  }

  private ImmutableSimpleStep(ImmutableSimpleStep.Builder builder) {
    this.action = builder.action;
    this.connection = builder.connection;
    this.stepKind = builder.stepKind;
    this.name = builder.name;
    this.id = builder.id;
    if (builder.kind != null) {
      initShim.kind(builder.kind);
    }
    if (builder.configuredPropertiesIsSet()) {
      initShim.configuredProperties(createUnmodifiableMap(false, false, builder.configuredProperties));
    }
    this.kind = initShim.getKind();
    this.configuredProperties = initShim.getConfiguredProperties();
    this.initShim = null;
  }

  private ImmutableSimpleStep(
      ImmutableSimpleStep original,
      Kind kind,
      Action action,
      Connection connection,
      String stepKind,
      Map<String, String> configuredProperties,
      String name,
      String id) {
    this.kind = kind;
    this.action = action;
    this.connection = connection;
    this.stepKind = stepKind;
    this.configuredProperties = configuredProperties;
    this.name = name;
    this.id = id;
    this.initShim = null;
  }

  private static final int STAGE_INITIALIZING = -1;
  private static final int STAGE_UNINITIALIZED = 0;
  private static final int STAGE_INITIALIZED = 1;
  private transient volatile InitShim initShim = new InitShim();

  private final class InitShim {
    private Kind kind;
    private int kindBuildStage;

    Kind getKind() {
      if (kindBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (kindBuildStage == STAGE_UNINITIALIZED) {
        kindBuildStage = STAGE_INITIALIZING;
        this.kind = Objects.requireNonNull(getKindInitialize(), "kind");
        kindBuildStage = STAGE_INITIALIZED;
      }
      return this.kind;
    }

    void kind(Kind kind) {
      this.kind = kind;
      kindBuildStage = STAGE_INITIALIZED;
    }
    private Map<String, String> configuredProperties;
    private int configuredPropertiesBuildStage;

    Map<String, String> getConfiguredProperties() {
      if (configuredPropertiesBuildStage == STAGE_INITIALIZING) throw new IllegalStateException(formatInitCycleMessage());
      if (configuredPropertiesBuildStage == STAGE_UNINITIALIZED) {
        configuredPropertiesBuildStage = STAGE_INITIALIZING;
        this.configuredProperties = createUnmodifiableMap(true, false, getConfiguredPropertiesInitialize());
        configuredPropertiesBuildStage = STAGE_INITIALIZED;
      }
      return this.configuredProperties;
    }

    void configuredProperties(Map<String, String> configuredProperties) {
      this.configuredProperties = configuredProperties;
      configuredPropertiesBuildStage = STAGE_INITIALIZED;
    }

    private String formatInitCycleMessage() {
      ArrayList<String> attributes = new ArrayList<String>();
      if (kindBuildStage == STAGE_INITIALIZING) attributes.add("kind");
      if (configuredPropertiesBuildStage == STAGE_INITIALIZING) attributes.add("configuredProperties");
      return "Cannot build SimpleStep, attribute initializers form cycle" + attributes;
    }
  }

  private Kind getKindInitialize() {
    return SimpleStep.super.getKind();
  }

  private Map<String, String> getConfiguredPropertiesInitialize() {
    return SimpleStep.super.getConfiguredProperties();
  }

  /**
   * @return The value of the {@code kind} attribute
   */
  @JsonProperty("kind")
  @Override
  public Kind getKind() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getKind()
        : this.kind;
  }

  /**
   * @return The value of the {@code action} attribute
   */
  @JsonProperty("action")
  @Override
  public Optional<Action> getAction() {
    return Optional.ofNullable(action);
  }

  /**
   * @return The value of the {@code connection} attribute
   */
  @JsonProperty("connection")
  @Override
  public Optional<Connection> getConnection() {
    return Optional.ofNullable(connection);
  }

  /**
   * @return The value of the {@code stepKind} attribute
   */
  @JsonProperty("stepKind")
  @Override
  public String getStepKind() {
    return stepKind;
  }

  /**
   * @return The value of the {@code configuredProperties} attribute
   */
  @JsonProperty("configuredProperties")
  @Override
  public Map<String, String> getConfiguredProperties() {
    InitShim shim = this.initShim;
    return shim != null
        ? shim.getConfiguredProperties()
        : this.configuredProperties;
  }

  /**
   * @return The value of the {@code name} attribute
   */
  @JsonProperty("name")
  @Override
  public String getName() {
    return name;
  }

  /**
   * @return The value of the {@code id} attribute
   */
  @JsonProperty("id")
  @Override
  public Optional<String> getId() {
    return Optional.ofNullable(id);
  }

  /**
   * Copy the current immutable object by setting a value for the {@link SimpleStep#getKind() kind} attribute.
   * A value equality check is used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for kind
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableSimpleStep withKind(Kind value) {
    if (this.kind == value) return this;
    Kind newValue = Objects.requireNonNull(value, "kind");
    return validate(new ImmutableSimpleStep(
        this,
        newValue,
        this.action,
        this.connection,
        this.stepKind,
        this.configuredProperties,
        this.name,
        this.id));
  }

  /**
   * Copy the current immutable object by setting a <i>present</i> value for the optional {@link SimpleStep#getAction() action} attribute.
   * @param value The value for action
   * @return A modified copy of {@code this} object
   */
  public final ImmutableSimpleStep withAction(Action value) {
    Action newValue = Objects.requireNonNull(value, "action");
    if (this.action == newValue) return this;
    return validate(new ImmutableSimpleStep(
        this,
        this.kind,
        newValue,
        this.connection,
        this.stepKind,
        this.configuredProperties,
        this.name,
        this.id));
  }

  /**
   * Copy the current immutable object by setting an optional value for the {@link SimpleStep#getAction() action} attribute.
   * A shallow reference equality check is used on unboxed optional value to prevent copying of the same value by returning {@code this}.
   * @param optional A value for action
   * @return A modified copy of {@code this} object
   */
  public final ImmutableSimpleStep withAction(Optional<? extends Action> optional) {
    Action value = optional.orElse(null);
    if (this.action == value) return this;
    return validate(new ImmutableSimpleStep(
        this,
        this.kind,
        value,
        this.connection,
        this.stepKind,
        this.configuredProperties,
        this.name,
        this.id));
  }

  /**
   * Copy the current immutable object by setting a <i>present</i> value for the optional {@link SimpleStep#getConnection() connection} attribute.
   * @param value The value for connection
   * @return A modified copy of {@code this} object
   */
  public final ImmutableSimpleStep withConnection(Connection value) {
    Connection newValue = Objects.requireNonNull(value, "connection");
    if (this.connection == newValue) return this;
    return validate(new ImmutableSimpleStep(
        this,
        this.kind,
        this.action,
        newValue,
        this.stepKind,
        this.configuredProperties,
        this.name,
        this.id));
  }

  /**
   * Copy the current immutable object by setting an optional value for the {@link SimpleStep#getConnection() connection} attribute.
   * A shallow reference equality check is used on unboxed optional value to prevent copying of the same value by returning {@code this}.
   * @param optional A value for connection
   * @return A modified copy of {@code this} object
   */
  public final ImmutableSimpleStep withConnection(Optional<? extends Connection> optional) {
    Connection value = optional.orElse(null);
    if (this.connection == value) return this;
    return validate(new ImmutableSimpleStep(
        this,
        this.kind,
        this.action,
        value,
        this.stepKind,
        this.configuredProperties,
        this.name,
        this.id));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link SimpleStep#getStepKind() stepKind} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for stepKind (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableSimpleStep withStepKind(String value) {
    if (Objects.equals(this.stepKind, value)) return this;
    return validate(new ImmutableSimpleStep(
        this,
        this.kind,
        this.action,
        this.connection,
        value,
        this.configuredProperties,
        this.name,
        this.id));
  }

  /**
   * Copy the current immutable object by replacing the {@link SimpleStep#getConfiguredProperties() configuredProperties} map with the specified map.
   * Nulls are not permitted as keys or values.
   * A shallow reference equality check is used to prevent copying of the same value by returning {@code this}.
   * @param entries The entries to be added to the configuredProperties map
   * @return A modified copy of {@code this} object
   */
  public final ImmutableSimpleStep withConfiguredProperties(Map<String, ? extends String> entries) {
    if (this.configuredProperties == entries) return this;
    Map<String, String> newValue = createUnmodifiableMap(true, false, entries);
    return validate(new ImmutableSimpleStep(this, this.kind, this.action, this.connection, this.stepKind, newValue, this.name, this.id));
  }

  /**
   * Copy the current immutable object by setting a value for the {@link SimpleStep#getName() name} attribute.
   * An equals check used to prevent copying of the same value by returning {@code this}.
   * @param value A new value for name (can be {@code null})
   * @return A modified copy of the {@code this} object
   */
  public final ImmutableSimpleStep withName(String value) {
    if (Objects.equals(this.name, value)) return this;
    return validate(new ImmutableSimpleStep(
        this,
        this.kind,
        this.action,
        this.connection,
        this.stepKind,
        this.configuredProperties,
        value,
        this.id));
  }

  /**
   * Copy the current immutable object by setting a <i>present</i> value for the optional {@link SimpleStep#getId() id} attribute.
   * @param value The value for id
   * @return A modified copy of {@code this} object
   */
  public final ImmutableSimpleStep withId(String value) {
    String newValue = Objects.requireNonNull(value, "id");
    if (Objects.equals(this.id, newValue)) return this;
    return validate(new ImmutableSimpleStep(
        this,
        this.kind,
        this.action,
        this.connection,
        this.stepKind,
        this.configuredProperties,
        this.name,
        newValue));
  }

  /**
   * Copy the current immutable object by setting an optional value for the {@link SimpleStep#getId() id} attribute.
   * An equality check is used on inner nullable value to prevent copying of the same value by returning {@code this}.
   * @param optional A value for id
   * @return A modified copy of {@code this} object
   */
  public final ImmutableSimpleStep withId(Optional<String> optional) {
    String value = optional.orElse(null);
    if (Objects.equals(this.id, value)) return this;
    return validate(new ImmutableSimpleStep(
        this,
        this.kind,
        this.action,
        this.connection,
        this.stepKind,
        this.configuredProperties,
        this.name,
        value));
  }

  /**
   * This instance is equal to all instances of {@code ImmutableSimpleStep} that have equal attribute values.
   * @return {@code true} if {@code this} is equal to {@code another} instance
   */
  @Override
  public boolean equals(Object another) {
    if (this == another) return true;
    return another instanceof ImmutableSimpleStep
        && equalTo((ImmutableSimpleStep) another);
  }

  private boolean equalTo(ImmutableSimpleStep another) {
    return kind.equals(another.kind)
        && Objects.equals(action, another.action)
        && Objects.equals(connection, another.connection)
        && Objects.equals(stepKind, another.stepKind)
        && configuredProperties.equals(another.configuredProperties)
        && Objects.equals(name, another.name)
        && Objects.equals(id, another.id);
  }

  /**
   * Computes a hash code from attributes: {@code kind}, {@code action}, {@code connection}, {@code stepKind}, {@code configuredProperties}, {@code name}, {@code id}.
   * @return hashCode value
   */
  @Override
  public int hashCode() {
    int h = 5381;
    h += (h << 5) + kind.hashCode();
    h += (h << 5) + Objects.hashCode(action);
    h += (h << 5) + Objects.hashCode(connection);
    h += (h << 5) + Objects.hashCode(stepKind);
    h += (h << 5) + configuredProperties.hashCode();
    h += (h << 5) + Objects.hashCode(name);
    h += (h << 5) + Objects.hashCode(id);
    return h;
  }

  /**
   * Prints the immutable value {@code SimpleStep} with attribute values.
   * @return A string representation of the value
   */
  @Override
  public String toString() {
    StringBuilder builder = new StringBuilder("SimpleStep{");
    builder.append("kind=").append(kind);
    if (action != null) {
      builder.append(", ");
      builder.append("action=").append(action);
    }
    if (connection != null) {
      builder.append(", ");
      builder.append("connection=").append(connection);
    }
    if (stepKind != null) {
      builder.append(", ");
      builder.append("stepKind=").append(stepKind);
    }
    builder.append(", ");
    builder.append("configuredProperties=").append(configuredProperties);
    if (name != null) {
      builder.append(", ");
      builder.append("name=").append(name);
    }
    if (id != null) {
      builder.append(", ");
      builder.append("id=").append(id);
    }
    return builder.append("}").toString();
  }

  /**
   * Construct a new immutable {@code SimpleStep} instance.
   * @param kind The value for the {@code kind} attribute
   * @param action The value for the {@code action} attribute
   * @param connection The value for the {@code connection} attribute
   * @param stepKind The value for the {@code stepKind} attribute
   * @param configuredProperties The value for the {@code configuredProperties} attribute
   * @param name The value for the {@code name} attribute
   * @param id The value for the {@code id} attribute
   * @return An immutable SimpleStep instance
   */
  public static ImmutableSimpleStep of(Kind kind, Optional<Action> action, Optional<Connection> connection, String stepKind, Map<String, ? extends String> configuredProperties, String name, Optional<String> id) {
    return validate(new ImmutableSimpleStep(kind, action, connection, stepKind, configuredProperties, name, id));
  }

  private static final Validator validator = Validation.buildDefaultValidatorFactory().getValidator();

  private static ImmutableSimpleStep validate(ImmutableSimpleStep instance) {
    Set<ConstraintViolation<ImmutableSimpleStep>> constraintViolations = validator.validate(instance);
    if (!constraintViolations.isEmpty()) {
      throw new ConstraintViolationException(constraintViolations);
    }
    return instance;
  }

  /**
   * Creates an immutable copy of a {@link SimpleStep} value.
   * Uses accessors to get values to initialize the new immutable instance.
   * If an instance is already immutable, it is returned as is.
   * @param instance The instance to copy
   * @return A copied immutable SimpleStep instance
   */
  public static ImmutableSimpleStep copyOf(SimpleStep instance) {
    if (instance instanceof ImmutableSimpleStep) {
      return (ImmutableSimpleStep) instance;
    }
    return new SimpleStep.Builder()
        .createFrom(instance)
        .build();
  }

  private Object readResolve() throws ObjectStreamException {
    return validate(this);
  }

  /**
   * Builds instances of type {@link ImmutableSimpleStep ImmutableSimpleStep}.
   * Initialize attributes and then invoke the {@link #build()} method to create an
   * immutable instance.
   * <p><em>{@code Builder} is not thread-safe and generally should not be stored in a field or collection,
   * but instead used immediately to create instances.</em>
   */
  static class Builder {
    private static final long OPT_BIT_CONFIGURED_PROPERTIES = 0x1L;
    private long optBits;

    private Kind kind;
    private Action action;
    private Connection connection;
    private String stepKind;
    private Map<String, String> configuredProperties = new LinkedHashMap<String, String>();
    private String name;
    private String id;

    /**
     * Creates a builder for {@link ImmutableSimpleStep ImmutableSimpleStep} instances.
     */
    Builder() {
      if (!(this instanceof SimpleStep.Builder)) {
        throw new UnsupportedOperationException("Use: new SimpleStep.Builder()");
      }
    }

    /**
     * Fill a builder with attribute values from the provided {@code io.syndesis.model.integration.Step} instance.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    public final SimpleStep.Builder createFrom(Step instance) {
      Objects.requireNonNull(instance, "instance");
      from((Object) instance);
      return (SimpleStep.Builder) this;
    }

    /**
     * Fill a builder with attribute values from the provided {@code io.syndesis.model.WithKind} instance.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    public final SimpleStep.Builder createFrom(WithKind instance) {
      Objects.requireNonNull(instance, "instance");
      from((Object) instance);
      return (SimpleStep.Builder) this;
    }

    /**
     * Fill a builder with attribute values from the provided {@code io.syndesis.model.integration.SimpleStep} instance.
     * @param instance The instance from which to copy values
     * @return {@code this} builder for use in a chained invocation
     */
    public final SimpleStep.Builder createFrom(SimpleStep instance) {
      Objects.requireNonNull(instance, "instance");
      from((Object) instance);
      return (SimpleStep.Builder) this;
    }

    private void from(Object object) {
      long bits = 0;
      if (object instanceof Step) {
        Step instance = (Step) object;
        if ((bits & 0x1L) == 0) {
          putAllConfiguredProperties(instance.getConfiguredProperties());
          bits |= 0x1L;
        }
        if ((bits & 0x8L) == 0) {
          String nameValue = instance.getName();
          if (nameValue != null) {
            name(nameValue);
          }
          bits |= 0x8L;
        }
        if ((bits & 0x2L) == 0) {
          String stepKindValue = instance.getStepKind();
          if (stepKindValue != null) {
            stepKind(stepKindValue);
          }
          bits |= 0x2L;
        }
        if ((bits & 0x10L) == 0) {
          Optional<Action> actionOptional = instance.getAction();
          if (actionOptional.isPresent()) {
            action(actionOptional);
          }
          bits |= 0x10L;
        }
        if ((bits & 0x20L) == 0) {
          Optional<Connection> connectionOptional = instance.getConnection();
          if (connectionOptional.isPresent()) {
            connection(connectionOptional);
          }
          bits |= 0x20L;
        }
        if ((bits & 0x4L) == 0) {
          kind(instance.getKind());
          bits |= 0x4L;
        }
      }
      if (object instanceof WithKind) {
        WithKind instance = (WithKind) object;
        if ((bits & 0x4L) == 0) {
          kind(instance.getKind());
          bits |= 0x4L;
        }
      }
      if (object instanceof SimpleStep) {
        SimpleStep instance = (SimpleStep) object;
        if ((bits & 0x1L) == 0) {
          putAllConfiguredProperties(instance.getConfiguredProperties());
          bits |= 0x1L;
        }
        if ((bits & 0x2L) == 0) {
          String stepKindValue = instance.getStepKind();
          if (stepKindValue != null) {
            stepKind(stepKindValue);
          }
          bits |= 0x2L;
        }
        if ((bits & 0x4L) == 0) {
          kind(instance.getKind());
          bits |= 0x4L;
        }
        if ((bits & 0x8L) == 0) {
          String nameValue = instance.getName();
          if (nameValue != null) {
            name(nameValue);
          }
          bits |= 0x8L;
        }
        if ((bits & 0x10L) == 0) {
          Optional<Action> actionOptional = instance.getAction();
          if (actionOptional.isPresent()) {
            action(actionOptional);
          }
          bits |= 0x10L;
        }
        if ((bits & 0x20L) == 0) {
          Optional<Connection> connectionOptional = instance.getConnection();
          if (connectionOptional.isPresent()) {
            connection(connectionOptional);
          }
          bits |= 0x20L;
        }
        Optional<String> idOptional = instance.getId();
        if (idOptional.isPresent()) {
          id(idOptional);
        }
      }
    }

    /**
     * Initializes the value for the {@link SimpleStep#getKind() kind} attribute.
     * <p><em>If not set, this attribute will have a default value as returned by the initializer of {@link SimpleStep#getKind() kind}.</em>
     * @param kind The value for kind 
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("kind")
    public final SimpleStep.Builder kind(Kind kind) {
      this.kind = Objects.requireNonNull(kind, "kind");
      return (SimpleStep.Builder) this;
    }

    /**
     * Initializes the optional value {@link SimpleStep#getAction() action} to action.
     * @param action The value for action
     * @return {@code this} builder for chained invocation
     */
    public final SimpleStep.Builder action(Action action) {
      this.action = Objects.requireNonNull(action, "action");
      return (SimpleStep.Builder) this;
    }

    /**
     * Initializes the optional value {@link SimpleStep#getAction() action} to action.
     * @param action The value for action
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("action")
    public final SimpleStep.Builder action(Optional<? extends Action> action) {
      this.action = action.orElse(null);
      return (SimpleStep.Builder) this;
    }

    /**
     * Initializes the optional value {@link SimpleStep#getConnection() connection} to connection.
     * @param connection The value for connection
     * @return {@code this} builder for chained invocation
     */
    public final SimpleStep.Builder connection(Connection connection) {
      this.connection = Objects.requireNonNull(connection, "connection");
      return (SimpleStep.Builder) this;
    }

    /**
     * Initializes the optional value {@link SimpleStep#getConnection() connection} to connection.
     * @param connection The value for connection
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("connection")
    public final SimpleStep.Builder connection(Optional<? extends Connection> connection) {
      this.connection = connection.orElse(null);
      return (SimpleStep.Builder) this;
    }

    /**
     * Initializes the value for the {@link SimpleStep#getStepKind() stepKind} attribute.
     * @param stepKind The value for stepKind (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("stepKind")
    public final SimpleStep.Builder stepKind(String stepKind) {
      this.stepKind = stepKind;
      return (SimpleStep.Builder) this;
    }

    /**
     * Put one entry to the {@link SimpleStep#getConfiguredProperties() configuredProperties} map.
     * @param key The key in the configuredProperties map
     * @param value The associated value in the configuredProperties map
     * @return {@code this} builder for use in a chained invocation
     */
    public final SimpleStep.Builder putConfiguredProperty(String key, String value) {
      this.configuredProperties.put(
          Objects.requireNonNull(key, "configuredProperties key"),
          Objects.requireNonNull(value, "configuredProperties value"));
      optBits |= OPT_BIT_CONFIGURED_PROPERTIES;
      return (SimpleStep.Builder) this;
    }

    /**
     * Put one entry to the {@link SimpleStep#getConfiguredProperties() configuredProperties} map. Nulls are not permitted
     * @param entry The key and value entry
     * @return {@code this} builder for use in a chained invocation
     */
    public final SimpleStep.Builder putConfiguredProperty(Map.Entry<String, ? extends String> entry) {
      String k = entry.getKey();
      String v = entry.getValue();
      this.configuredProperties.put(
          Objects.requireNonNull(k, "configuredProperties key"),
          Objects.requireNonNull(v, "configuredProperties value"));
      optBits |= OPT_BIT_CONFIGURED_PROPERTIES;
      return (SimpleStep.Builder) this;
    }

    /**
     * Sets or replaces all mappings from the specified map as entries for the {@link SimpleStep#getConfiguredProperties() configuredProperties} map. Nulls are not permitted
     * @param configuredProperties The entries that will be added to the configuredProperties map
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("configuredProperties")
    public final SimpleStep.Builder configuredProperties(Map<String, ? extends String> configuredProperties) {
      this.configuredProperties.clear();
      optBits |= OPT_BIT_CONFIGURED_PROPERTIES;
      return putAllConfiguredProperties(configuredProperties);
    }

    /**
     * Put all mappings from the specified map as entries to {@link SimpleStep#getConfiguredProperties() configuredProperties} map. Nulls are not permitted
     * @param configuredProperties The entries that will be added to the configuredProperties map
     * @return {@code this} builder for use in a chained invocation
     */
    public final SimpleStep.Builder putAllConfiguredProperties(Map<String, ? extends String> configuredProperties) {
      for (Map.Entry<String, ? extends String> entry : configuredProperties.entrySet()) {
        String k = entry.getKey();
        String v = entry.getValue();
        this.configuredProperties.put(
            Objects.requireNonNull(k, "configuredProperties key"),
            Objects.requireNonNull(v, "configuredProperties value"));
      }
      optBits |= OPT_BIT_CONFIGURED_PROPERTIES;
      return (SimpleStep.Builder) this;
    }

    /**
     * Initializes the value for the {@link SimpleStep#getName() name} attribute.
     * @param name The value for name (can be {@code null})
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("name")
    public final SimpleStep.Builder name(String name) {
      this.name = name;
      return (SimpleStep.Builder) this;
    }

    /**
     * Initializes the optional value {@link SimpleStep#getId() id} to id.
     * @param id The value for id
     * @return {@code this} builder for chained invocation
     */
    public final SimpleStep.Builder id(String id) {
      this.id = Objects.requireNonNull(id, "id");
      return (SimpleStep.Builder) this;
    }

    /**
     * Initializes the optional value {@link SimpleStep#getId() id} to id.
     * @param id The value for id
     * @return {@code this} builder for use in a chained invocation
     */
    @JsonProperty("id")
    public final SimpleStep.Builder id(Optional<String> id) {
      this.id = id.orElse(null);
      return (SimpleStep.Builder) this;
    }

    /**
     * Builds a new {@link ImmutableSimpleStep ImmutableSimpleStep}.
     * @return An immutable instance of SimpleStep
     * @throws java.lang.IllegalStateException if any required attributes are missing
     */
    public ImmutableSimpleStep build() {
      return ImmutableSimpleStep.validate(new ImmutableSimpleStep(this));
    }

    private boolean configuredPropertiesIsSet() {
      return (optBits & OPT_BIT_CONFIGURED_PROPERTIES) != 0;
    }
  }

  private static <K, V> Map<K, V> createUnmodifiableMap(boolean checkNulls, boolean skipNulls, Map<? extends K, ? extends V> map) {
    switch (map.size()) {
    case 0: return Collections.emptyMap();
    case 1: {
      Map.Entry<? extends K, ? extends V> e = map.entrySet().iterator().next();
      K k = e.getKey();
      V v = e.getValue();
      if (checkNulls) {
        Objects.requireNonNull(k, "key");
        Objects.requireNonNull(v, "value");
      }
      if (skipNulls && (k == null || v == null)) {
        return Collections.emptyMap();
      }
      return Collections.singletonMap(k, v);
    }
    default: {
      Map<K, V> linkedMap = new LinkedHashMap<K, V>(map.size());
      if (skipNulls || checkNulls) {
        for (Map.Entry<? extends K, ? extends V> e : map.entrySet()) {
          K k = e.getKey();
          V v = e.getValue();
          if (skipNulls) {
            if (k == null || v == null) continue;
          } else if (checkNulls) {
            Objects.requireNonNull(k, "key");
            Objects.requireNonNull(v, "value");
          }
          linkedMap.put(k, v);
        }
      } else {
        linkedMap.putAll(map);
      }
      return Collections.unmodifiableMap(linkedMap);
    }
    }
  }
}
